@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,6 +16,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 import copy, types, re
+
 from page import Page
 from appy.px import Px
 from appy import Object
@@ -193,13 +194,15 @@ class Field:
       layoutTarget=field">:field.getPx(minimal, \
                                        hostLayout or layoutType, _ctx_)</x>''')
 
-    def doRender(self, layoutType, obj, name=None, minimal=False, tagCss=None):
+    def doRender(self, layoutType, obj, name=None, minimal=False, tagCss=None,
+                 specific=None):
         '''Allows to call pxRender from code, to display the content of this
            field on p_obj in some specific context, for example in a Computed
            field.'''
         context = {'layoutType': layoutType, 'field': self, 'minimal': minimal,
                    'name': name or self.name, 'obj': obj, 'zobj': obj.o}
         if tagCss: context['tagCss'] = tagCss
+        if specific: context.update(specific)
         # Get the current PX context: we will need to copy some entries from it.
         # If no such context exists, create one.
         req = obj.request
@@ -994,6 +997,22 @@ class Field:
            if this value is mutable, get a copy of it.'''
         return self.getValue(obj)
 
+    def getComparableValue(self, obj):
+        '''Get the value of this field on p_obj, as can be compared with its new
+           version, coming from the validator.'''
+        # For immutable values, it simply returns the currently stored value.
+        # For mutable values, there are 2 cases:
+        # (a) the value as produced by the validator completely overwrites the
+        #     existing value. Thus, the previous and new versions can safely be
+        #     compared, because being completely distinct. In this case, as for
+        #     immutable values, the currently stored value can also be used
+        #     as-is, as "comparable" value ;
+        # (b) the value as produced by the validator updates the stored value.
+        #     In this case, comparing the previous and new versions would lead
+        #     to comparing the same value. Here, the "comparable" value must be
+        #     a deep copy of the previous value and not the stored value itself.
+        return self.getValue(obj)
+
     def getFormattedValue(self, obj, value, layoutType='view',
                           showChanges=False, language=None):
         '''p_value is a real p_obj(ect) value from a field from this type. This
@@ -1083,6 +1102,13 @@ class Field:
             r = '<span class="editable" %s>%s</span>' % (onClick, value)
         return r
 
+    def getFakeObject(self, value, request):
+        '''Returns a fake object, to mimic an object storing this p_value for
+           field p_self.'''
+        r = Object()
+        setattr(r, self.name, value)
+        return Object(aq_base=r, REQUEST=request, id='0')
+
     def getIndexType(self):
         '''Returns the name of the technical, Zope-level index type for this
            field.'''
@@ -1306,9 +1332,10 @@ class Field:
         tool.log(SEARCH_TRIGGER % (self.type, className, self.name, search))
         # (Re-)execute the query, but without any limit on the number of
         # results; return Appy objects.
+        class_ = tool.getAppyClass(className)
         objs = tool.executeQuery(className, searchName=search, sortBy=sortKey,
                                  sortOrder=sortOrder, maxResults='NO_LIMIT',
-                                 filters=sutils.getDictFrom(filters),
+                                 filters=tool.getFilters(class_, filters),
                                  refObject=refObject, refField=refField)
         r = [o.appy() for o in objs.objects]
         tool.log(SEARCH_DONE % len(r))
@@ -1557,6 +1584,20 @@ class Field:
             # attribute.
         return ';'.join(res)
 
+    def getRadioStyle(self, isSearch=False):
+        '''If this field must be rendered as radio buttons or checkboxes, and
+           its height is specified as a string, restrict the visible part of the
+           widget to this height, with a scrollbar when appropriate.'''
+        prefix = isSearch and 's' or ''
+        r = []
+        height = getattr(self, '%sheight' % prefix)
+        if isinstance(height, str):
+            r.append('height:%s;overflow-y:auto' % height)
+        width = getattr(self, '%swidth' % prefix)
+        if isinstance(width, str):
+            r.append('width:%s;overflow-x:auto' % width)
+        return ';'.join(r)
+
     def getWidthInChars(self, isSearch):
         '''If attribute "width" contains an integer value, it contains the
            number of chars shown in this field (a kind of "logical" width). If
