@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -513,15 +513,17 @@ class Pod(Field):
             res = elems[0]
         return res
 
-    def getDownloadName(self, obj, template, format, queryRelated):
+    def getDownloadName(self, obj, template, format=None, queryRelated=False,
+                        bypassMethod=False):
         '''Gets the name of the pod result as will be seen by the user that will
            download it. Ensure the returned name is not too long for the OS that
            will store the downloaded file with this name.'''
         # Use method self.downloadName if present and if it returns something
-        # for p_template
-        if self.downloadName:
+        # for p_template.
+        if not bypassMethod and self.downloadName:
             name = self.downloadName(obj, template)
-            if name: return '%s.%s' % (name, format)
+            if name:
+                return format and ('%s.%s' % (name, format)) or name
         # Compute the default download name
         norm = obj.tool.normalize
         fileName = norm(self.getTemplateName(obj, template))[:40]
@@ -530,7 +532,7 @@ class Pod(Field):
             # the object title.
             title = obj.o.getShownValue('title')
             fileName = '%s-%s' % (norm(title)[:40], fileName)
-        return fileName + '.' + format
+        return format and ('%s.%s' % (fileName, format)) or fileName
 
     def getVisibleTemplates(self, obj):
         '''Returns, among self.template, the template(s) that can be shown'''
@@ -691,7 +693,7 @@ class Pod(Field):
         if not queryData:
             frozen = self.isFrozen(obj, template, format)
             if frozen:
-                fileName = self.getDownloadName(obj, template, format, False)
+                fileName = self.getDownloadName(obj, template, format)
                 return FileInfo(frozen, inDb=False, uploadName=fileName)
         # We must call pod to compute a pod document from "template"
         tool = obj.tool
