@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -23,6 +23,7 @@ from appy.shared.diff import HtmlDiff
 from appy.shared.data import countries
 from appy.shared import utils as sutils
 from appy.fields.select import Selection
+from appy.shared.xml_parser import Escape
 from appy.shared.xml_parser import XhtmlCleaner
 from appy.gen.indexer import XhtmlTextExtractor, ValidValue
 
@@ -736,7 +737,7 @@ class String(Field):
             res = self.getDiffValue(obj, res, language)
         elif self.format == String.TEXT:
             if layoutType in ('view', 'cell'):
-                res = obj.formatText(res, format='html_from_text')
+                res = Escape.xhtml(res, p=True)
         # If value starts with a carriage return, add a space; else, it will
         # be ignored.
         if isinstance(res, basestring) and \
@@ -1029,6 +1030,10 @@ class String(Field):
         return value
 
     def getStorableValue(self, obj, value, complete=False):
+        # If no object is passed, assume the field is unilingual
+        if obj is None:
+            return self.getUnilingualStorableValue(obj, value)
+        # Check unilinguality
         languages = self.getAttribute(obj, 'languages')
         if len(languages) == 1:
             return self.getUnilingualStorableValue(obj, value)
