@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -22,6 +22,7 @@ from appy.px import Px
 from appy.fields import Field
 from appy.gen.layout import Layouts
 from appy.shared import utils as sutils
+from appy.shared.xml_parser import Escape
 from appy.shared.xml_parser import XmlParser
 from appy.fields.multilingual import Multilingual
 
@@ -718,10 +719,10 @@ class Text(Multilingual, Field):
         if Field.isEmptyValue(self, obj, value) and not showChanges: return ''
         r = value
         if layoutType in ('view', 'cell'):
-            r = obj.formatText(r, format='html_from_text')
+            r = Escape.xhtml(res, p=True)
         # If value starts with a carriage return, add a space; else, it will
         # be ignored.
-        if isinstance(r, basestring) and \
+        elif isinstance(r, basestring) and \
            (r.startswith('\n') or r.startswith('\r\n')): r = ' ' + r
         return r
 
