@@ -403,7 +403,8 @@ class Layouts:
         # We specify "edit": that way, "view" inherits from it, and all these
         # layouts are ready in case an info must be shown on an "edit" layout.
         b = {'edit': 'l'}
-        d = {'edit': 'l-d'}
+        d = {'edit': Table('l-d', width=None),
+             'view': Table('l-d', width=None)}
         do = {'edit': 'f', 'view': 'd'} # Description only
         c = {'edit': 'l|'} # Centered
         dc = {'edit': 'l|-d|'}
@@ -424,6 +425,9 @@ class Layouts:
         w = {'view': Table('lrv-f', width='100%')}
         # "d" stands for "description": a description label is added, on view
         wd = {'view': Table('l-d-f', width='100%')}
+        # Wide, but not on edit
+        w_e = {'edit': Table('lrv-f', width=None),
+               'view': Table('lrv-f', width='100%')}
         # "cell" layout varies from the default
         cell = Table('f|', width='100%', css_class='no')
 
