@@ -212,7 +212,11 @@ class ToolMixin(BaseMixin):
             if text:
                 if res: res += ' :: '
                 res += text
-        return self.truncateValue(res, width=100)
+        try:
+            res = self.truncateValue(res, width=100)
+        except UnicodeDecodeError:
+            res = '?'
+        return res
 
     def getCatalog(self):
         '''Returns the catalog object'''
@@ -336,14 +340,17 @@ class ToolMixin(BaseMixin):
         # Perform the switch by updating the authentication cookie
         gutils.updateCookie(rq, ctx)
         authContext._setOnRequest(atool, rq, ctx)
-        # Return to the home page if we were displaying an error message or if
-        # defined that way. Else, stay on the same page (=go to the referer
-        # page).
-        referer = self.getReferer()
-        if authContext.homeAfterSwitch or ('/config/info' in referer):
-            back = self.getHomePage()
-        else:
-            back = referer
+        # Return to the URL specified in the request, or, by default:
+        # a) return to the home page if we were displaying an error message or
+        #    if required by the auhentication context;
+        # b) else, stay on the same page (=go to the referer page).
+        back = rq.get('url')
+        if not back:
+            referer = self.getReferer()
+            if authContext.homeAfterSwitch or ('/config/info' in referer):
+                back = self.getHomePage()
+            else:
+                back = referer
         return self.goto(back)
 
     def flipLanguageDirection(self, align, dir):
@@ -538,6 +545,22 @@ class ToolMixin(BaseMixin):
                 res.append('user:%s' % login)
         return res
 
+    def getFilters(self, class_, filters=None):
+        '''Extracts, from the request, filters defined on fields belonging to
+           this p_class_'''
+        r = sutils.getDictFrom(filters or self.REQUEST.get('filters'))
+        # Apply a potential transform on every filter value
+        for name in r.keys():
+            # Get the corresponding field on p_class_
+            field = getattr(class_, name, None)
+            if field:
+                try:
+                    r[name] = field.getStorableValue(None, r[name])
+                except Exception, e:
+                    # The encoded value is invalid. Ignore it.
+                    del r[name]
+        return r
+
     def executeQuery(self, className, searchName=None, startNumber=0,
                      search=None, remember=False, brainsOnly=False,
                      maxResults=None, noSecurity=False, sortBy=None,
