@@ -147,6 +147,7 @@ function XhrObject() { // Wraps a XmlHttpRequest object
    this chunk of HTML are not executed. This function, typically used as "onGet"
    param for the askAjaxChunk function below, will evaluate those scripts. */
 function evalInnerScripts(xhrObject, hookElem) {
+  if (!hookElem) return;
   var scripts = hookElem.getElementsByTagName('script');
   for (var i=0; i<scripts.length; i++) eval(scripts[i].innerHTML);
 }
@@ -159,6 +160,11 @@ function injectChunk(elem, content, inner, searchTop){
       // Replace p_elem with a new node filled with p_content and return it
       var id = elem.id;
       if (id && searchTop) id = ':' + id;
+      if ((elem.tagName == 'TR') && (content.substr(0,2) == '<!')) {
+        /* p_content is a complete (error) page, it cannot be injected in a
+           table row. Replace the whole table instead. */
+        elem = elem.parentNode.parentNode;
+      }
       elem.outerHTML = content;
       if (id) res = getNode(id); // Get the new element
     }
@@ -413,7 +419,7 @@ function askBunchFiltered(hookId, filterKey) {
       value = filter.value;
   if (value && (filter.nodeName == 'INPUT')) {
     // Remove reserved chars and ensure it contains at least 3 chars
-    value = encodeURIComponent(value.trim()).replace(',','').replace(':','');
+    value = encodeURIComponent(value.trim().replace(',','.').replace(':',''));
     if (value.length < 3) {
       filter.style.background = wrongTextInput;
       return;
