@@ -1,5 +1,6 @@
 # ------------------------------------------------------------------------------
-import re, os, os.path, base64, urllib
+import re, os, os.path, base64, urllib, types
+
 from appy import Object
 from appy.px import Px
 from appy.shared import utils as sutils
@@ -244,7 +245,9 @@ def callMethod(obj, method, klass=None, cache=True):
     methodType = method.__class__.__name__
     if methodType == 'staticmethod':
         method = method.__get__(klass)
-    elif methodType == 'instancemethod':
+    elif (methodType == 'instancemethod') and \
+         (type(method.im_self) != types.ClassType):
+        # An instance method. If im_self is a class, it is a classmethod.
         method = method.im_func
     # Disable caching for lambda functions
     funName = method.func_name
@@ -311,6 +314,14 @@ def updateCookie(request, ctx, onResponse=False):
        p_onResponse is True, we get the cookie from the response object instead
        of the request object.'''
     login, password, oldCtx = readCookie(request, onResponse=onResponse)
+    if login is None:
+        # There is no cookie. When SSO is enabled, cookies are normally not
+        # used, excepted if an authentication context comes into play. In that
+        # case, the cookie is only used to store this context. In order to
+        # represent this fact, set, in the cookie, the user login as "_s_s_o_".
+        # Such a cookie means: "this is a special cookie, only used to store the
+        # authentication context. Check SSO to get the user login".
+        login = '_s_s_o_'
     writeCookie(login, password, ctx, request)
 
 # ------------------------------------------------------------------------------
