@@ -1,4 +1,4 @@
-    # ------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 from appy.px import Px
 from appy.fields.string import String
 from appy.gen import WorkflowOwner, Layouts, Show
@@ -106,11 +106,15 @@ class UserWrapper(AbstractWrapper):
         if not isLocal: return Show.E_
         return True
 
-    def showRoles(tool):
-        '''Only the admin can edit roles'''
-        user = tool.user
+    def showRoles(self):
+        '''Global roles are not visible or editable under all circumstances'''
+        # Local users can't be granted global roles
+        if self.container != self.tool: return
+        # Only a Manager is allowed to grant global roles
+        user = self.user
         if user.hasRole('Manager'): return True
-        if user.hasRole('Owner', user): return Show.E_
+        # The user itself, owner of himself, can consult his roles
+        if user.hasRole('Owner', self): return Show.E_
 
     def showResetPassword(self):
         '''Action "reset password" is available to anyone having write access to
