@@ -333,8 +333,8 @@ class AbstractWrapper(object):
               modified in this change. -->
          <table class="changes" width="100%" if="event.get('changes')">
           <tr>
-           <th align=":dleft" width="30%">:_('modified_field')</th>
-           <th align=":dleft" width="70%">:_('previous_value')</th>
+           <th align=":dleft">:_('modified_field')</th>
+           <th align=":dleft">:_('previous_value')</th>
           </tr>
           <tr for="change in event['changes'].items()" valign="top"
               var2="elems=change[0].split('-');
@@ -380,7 +380,8 @@ class AbstractWrapper(object):
           var2="hasHistory=zobj.hasHistory();
                 collapsible=collapsible|True;
                 collapse=zobj.getHistoryCollapse();
-                creator=zobj.Creator()">
+                creator=zobj.Creator();
+                createComment=zobj.getCreationComment()">
       <table width="100%" class="header" cellpadding="0" cellspacing="0">
        <tr>
         <td colspan="2" class="by">
@@ -407,6 +408,9 @@ class AbstractWrapper(object):
          <x if="zobj.showState()">&mdash;
           <x>:_('workflow_state')</x> : <b>:_(zobj.getWorkflowLabel())</b>
          </x>
+
+         <!-- Initial comment -->
+         <div if="createComment" class="topSpace">::createComment</div>
         </td>
        </tr>
 
@@ -596,6 +600,7 @@ class AbstractWrapper(object):
              render=field.getRenderMode(layoutType);
              prefixedName='%s:%s' % (zobj.id, refFieldName);
              create=field.getAttribute(zobj, 'create');
+             createFromSearch=not isinstance(create, basestring);
              selector=field.getSelector(obj, req);
              selectable=bool(selector) and inPopup;
              linkList=field.link == 'list';
@@ -1083,7 +1088,8 @@ class AbstractWrapper(object):
         return r
 
     def createFrom(self, fieldNameOrClass, other, noSecurity=False,
-                   executeMethods=True, exclude=(), keepBase=False, fields={}):
+                   executeMethods=True, exclude=(), keepBase=False,
+                   keepHistory=False, fields={}):
         '''Similar to m_create above, excepted that another object is used
            (p_other) as base for filling in data for the object to create.
            p_exclude can list fields (by their names) that will not be copied on
@@ -1105,6 +1111,12 @@ class AbstractWrapper(object):
         if keepBase:
             for name in ('creator', 'created', 'modified'):
                 setattr(r.o, name, getattr(other.o, name))
+        # Copy p_other's history as-is if requested
+        if keepHistory:
+            from persistent.mapping import PersistentMapping
+            history = PersistentMapping()
+            history['appy'] = tuple([event.copy() for event in other.history])
+            r.o.workflow_history = history
         return r
 
     def freeze(self, name, template=None, format='pdf', noSecurity=True,
@@ -1334,7 +1346,7 @@ class AbstractWrapper(object):
                 marshaller = CsvMarshaller(at, include=include, exclude=exclude)
                 marshaller.marshall(self)
 
-    def historize(self, data):
+    def historize(self, data, comment=None):
         '''This method allows to add "manually" a "data-change" event into the
            object's history. Indeed, data changes are "automatically" recorded
            only when an object is edited through the edit form, not when a
@@ -1342,7 +1354,7 @@ class AbstractWrapper(object):
 
            p_data must be a dictionary whose keys are field names (strings) and
            whose values are the previous field values.'''
-        self.o.addDataChange(data)
+        self.o.addDataChange(data, comment)
 
     def getLastEvent(self, transition, notBefore=None, history=None):
         '''Gets, from the object history (or from p_history if not None), the
@@ -1593,7 +1605,7 @@ class AbstractWrapper(object):
 
     def sendMailIf(self, privilege, subject, body, attachments=None,
                    privilegeType='permission', excludeExpression='False',
-                   userMethod=None):
+                   userMethod=None, replyTo=None):
         '''Sends a mail related to this object to any active user having
            p_privilege on it.'''
         # If p_privilegeType...
@@ -1668,7 +1680,8 @@ class AbstractWrapper(object):
                 continue
             recipients.append(recipient)
         if recipients:
-            self.tool.sendMail(recipients, subject, body, attachments)
+            self.tool.sendMail(recipients, subject, body, attachments,
+                               replyTo=replyTo)
         else:
             self.log('no recipient for sending mail about %s with %s %s.' % \
                      (self.id, pt, privilege), type='warning')
