@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,11 +16,12 @@
 # ------------------------------------------------------------------------------
 import re, sys
 from xml.sax.saxutils import quoteattr
-from appy.shared.xml_parser import xmlPrologue, Escape
+
 from appy.pod import PodError
-from appy.shared.utils import Traceback
 from appy.pod.elements import *
-from appy.pod import actions
+from appy.pod import actions, getUuid
+from appy.shared.utils import Traceback
+from appy.shared.xml_parser import xmlPrologue, Escape
 
 # ------------------------------------------------------------------------------
 class ParsingError(Exception): pass
@@ -136,6 +137,16 @@ class Buffer:
 
     def getLength(self): pass # To be overridden
 
+    def patchProtected(self, elem, attrs):
+        '''Add or patch the attribute allowing to protect this p_elem (a cell or
+           section), by evaluating variable "cellProtected" or
+           "sectionProtected".'''
+        type = elem[:elem.index(':')]
+        name = '%s:protected' % type
+        var = (type == 'table') and 'cell' or 'section'
+        value = attrs.get(name, "false").strip() or "false"
+        attrs[name] = ':str(%sProtected).lower()|"%s"' % (var, value)
+
     def patchElement(self, elem, attrs):
         '''Modifies p_elem's attributes when relevant and make them dependent of
            some variable definitions.'''
@@ -145,7 +156,8 @@ class Buffer:
             # ie, for naming calc sheets (which are implemented as tables).
             attrs = attrs._attrs
             name = 'table:name'
-            attrs[name] = ':tableName|"%s"' % attrs[name]
+            currentName = attrs.get(name) or getUuid(removeDots=True,prefix='T')
+            attrs[name] = ':tableName|"%s"' % currentName
         elif elem == 'table:table-column':
             # Convert attribute "number-columns-repeated" of every table column
             # (or add it if it does not exist) to let the user define how he
@@ -155,18 +167,25 @@ class Buffer:
             columnNumber = self.env.getTable().nbOfColumns -1
             nb = (key in attrs) and attrs[key] or '1'
             attrs[key] = ':columnsRepeated[%d]|%s' % (columnNumber, nb)
-        elif self.env.parser.caller.protection and \
-             (elem in ('table:table-cell', 'text:section')):
-            # Add or patch the attribute allowing to protect the cell or
-            # section, by evaluating variable "cellProtected" or
-            # "sectionProtected".
+        elif elem == 'table:table-cell':
             attrs = attrs._attrs
-            type = elem[:elem.index(':')]
-            name = '%s:protected' % type
-            var = (type == 'table') and 'cell' or 'section'
-            value = attrs.get(name, "false").strip() or "false"
-            attrs[name] = ':str(%sProtected).lower()|"%s"' % (var, value)
-        elif (elem == 'text:list-item') and attrs.has_key('text:start-value'):
+            key = 'table:style-name'
+            style = attrs.get(key)
+            if style and style.startswith('S'):
+                # By convention, if a style name starts with letter "S", POD
+                # will allow to replace the name of the cell style with another
+                # one, via function "styleFor" if found in the context. The
+                # function will be passed 2 args: the name of the style that
+                # should normally have been applied, and the context itself. It
+                # must return the name of the style to apply. This mechanism is
+                # restricted to styles starting with "S" for performance
+                # reasons.
+                attrs[key] = ':styleFor("%s",_ctx_)|"%s"' % (style, style)
+            elif self.env.parser.caller.protection:
+                self.patchProtected(elem, attrs)
+        elif elem == 'text:section' and self.env.parser.caller.protection:
+            self.patchProtected(elem, attrs._attrs)
+        elif elem == 'text:list-item' and attrs.has_key('text:start-value'):
             attrs = attrs._attrs
             val = int(attrs['text:start-value'])
             attrs['text:start-value'] = ':loop._all_[-1].nb+%d|%d' % (val, val)
