@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -70,6 +70,42 @@ for name in ('content', 'styles'):
     POD_STYLES[name] = f.read()
     f.close()
 
+# ------------------------------------------------------------------------------
+class CsvOptions:
+    '''When the POD result is of type CSV, use this class to define CSV
+       options.'''
+
+    # Available encodings for CSV output files
+    encodings = {'utf-8': 76, 'latin-1': 12}
+
+    def __init__(self, fieldSeparator=';', textDelimiter='"', encoding='utf-8'):
+        # The delimiter used between every field on a single line
+        self.fieldSeparator = fieldSeparator
+        # The character used to "wrap" every value. Can be the empty string.
+        self.textDelimiter = textDelimiter
+        # The ODT result's file encoding. Defaults is "utf-8". "latin-1" is also
+        # allowed, or any numeric value as defined at the URL mentioned in
+        # appy/pod/converter.py, at variable HELP_CSV_URL.
+        self.encoding = encoding
+
+    def asString(self):
+        '''Returns options as a string, ready-to-be sent to the Converter'''
+        # Manage the optional text delimiter
+        if self.textDelimiter:
+            delim = ord(self.textDelimiter)
+        else:
+            delim = ''
+        # Manage encoding
+        enc = self.encoding
+        if isinstance(enc, int) or enc.isdigit():
+            encoding = enc
+        else:
+            encoding = CsvOptions.encodings.get(enc) or 76
+        return '%s,%s,%s,1' % (ord(self.fieldSeparator), delim, encoding)
+
+# Create a default instance
+CsvOptions.default = CsvOptions()
+
 # ------------------------------------------------------------------------------
 class Renderer:
     templateTypes = ('odt', 'ods') # Types of POD templates
@@ -77,6 +113,9 @@ class Renderer:
     metaRex = re.compile('<dc:title>.*?</dc:title>', re.S)
     metaHook = '</office:meta>'
 
+    # Make some elements directly available here
+    CsvOptions = CsvOptions
+
     # ODF tags that may hold POD expressions
     allExpressionsHolders = (
       'if',      # Conditional fields
@@ -90,11 +129,12 @@ class Renderer:
       ooServer='localhost', ooPort=2002, stream='auto', stylesMapping={},
       html=False, forceOoCall=False, finalizeFunction=None,
       overwriteExisting=False, raiseOnError=False, imageResolver=None,
-      stylesTemplate=None, optimalColumnWidths=False, distributeColumns=False,
-      script=None, managePageStyles=None, resolveFields=False,
-      expressionsHolders=defaultExpressionsHolders, metadata=True,
-      pdfOptions='ExportNotes=True', deleteTempFolder=True, protection=False,
-      pageStart=1, tabbedCR=False, fonts=None):
+      rotateImages=False, stylesTemplate=None, optimalColumnWidths=False,
+      distributeColumns=False, script=None, managePageStyles=None,
+      resolveFields=False, expressionsHolders=defaultExpressionsHolders,
+      metadata=True, pdfOptions='ExportNotes=True', csvOptions=None,
+      deleteTempFolder=True, protection=False, pageStart=1, tabbedCR=False,
+      fonts=None):
         '''Base on a document template (whose path is in p_template), which is
            an ODT or ODS file containing special expressions and statements
            written in Python, this renderer generates an ODT file (whose path is
@@ -164,6 +204,13 @@ class Renderer:
         # permission to) perform a HTTP GET on those images. Currently, the
         # resolver can only be a Zope application object.
 
+        # If p_rotateImages is True, for every image to inject in the result,
+        # Appy will call ImageMagick to read EXIF data. If an orientation is
+        # defined, ImageMagick will be called to effectively apply the
+        # corresponding rotation to the image. Because reading EXIF data on
+        # every image to import may take some processing, p_rotateImages is
+        # False by default.
+
         # p_stylesTemplate can be the path to a LibreOffice file (ie, a .ott
         # file) whose styles will be imported within the result.
 
@@ -237,6 +284,12 @@ class Renderer:
 
         # More info in appy/pod/converter.py.
 
+        # If the result must be produced in CSV, options being specific to this
+        # format can be passed in p_csvOptions, as an instance of class
+        # CsvOptions, available on class Renderer as Renderer.CsvOptions. If
+        # p_csvOptions is None, defauls CSV options will be used from
+        # Renderer.CsvOptions.default.
+
         # If this renderer is called by another one (its "parent"), it may
         # receive an additional parameter, p_deleteTempFolder. If False, the
         # temp folder may be reused by other children of the parent renderer, so
@@ -268,6 +321,7 @@ class Renderer:
         #                      fc-list :lang=en family
 
         self.template = template
+        self.templateType = self.getTemplateType(template)
         self.result = result
         self.resultType = os.path.splitext(result)[1].strip('.')
         self.contentXml = None # Content (string) of content.xml
@@ -291,6 +345,7 @@ class Renderer:
         self.overwriteExisting = overwriteExisting
         self.raiseOnError = raiseOnError
         self.imageResolver = imageResolver
+        self.rotateImages = rotateImages
         self.stylesTemplate = stylesTemplate
         self.optimalColumnWidths = optimalColumnWidths
         self.distributeColumns = distributeColumns
@@ -300,6 +355,7 @@ class Renderer:
         self.expressionsHolders = expressionsHolders
         self.metadata = metadata
         self.pdfOptions = pdfOptions
+        self.csvOptions = csvOptions
         self.deleteTempFolder = deleteTempFolder
         self.protection = protection
         self.pageStart = pageStart
@@ -350,9 +406,9 @@ class Renderer:
 
     # Attributes to clone to a sub-renderer when using m_clone hereafter
     cloneAttributes = ('pythonWithUnoPath', 'ooServer', 'ooPort', 'html',
-      'raiseOnError', 'imageResolver', 'stylesTemplate', 'optimalColumnWidths',
-      'distributeColumns', 'expressionsHolders', 'protection', 'tabbedCR',
-      'fonts')
+      'raiseOnError', 'imageResolver', 'rotateImages', 'stylesTemplate',
+      'optimalColumnWidths', 'distributeColumns', 'expressionsHolders',
+      'protection', 'tabbedCR', 'fonts')
 
     def clone(self, template, context, result, **params):
         '''Creates another Renderer instance, similar to p_self, but for
@@ -544,7 +600,7 @@ class Renderer:
         # * 'duplex' insert 2 page breaks if the sub-document has an odd number
         #            of pages, 1 else (useful for duplex printing).
 
-        # If p_convertOptions are given (for images only), imagemagick will be
+        # If p_convertOptions are given (for images only), ImageMagick will be
         # called with these options to perform some transformation on the image.
         # For example, if you specify
         #                    convertOptions="-rotate 90"
@@ -835,6 +891,11 @@ class Renderer:
         else:
             result = format or self.resultType
             resultType = result
+        # Get CSV options if we must produce a CSV file
+        if resultType == 'csv':
+            csvOptions = (self.csvOptions or CsvOptions.default).asString()
+        else:
+            csvOptions = None
         try:
             if (not isinstance(self.ooPort, int)) and \
                (not isinstance(self.ooPort, long)):
@@ -845,7 +906,7 @@ class Renderer:
                     Converter(resultName, result, self.ooServer, self.ooPort,
                       self.stylesTemplate, self.optimalColumnWidths,
                       self.distributeColumns, self.script, self.resolveFields,
-                      self.pdfOptions, self.ppp, self.stream,
+                      self.pdfOptions, csvOptions, self.ppp, self.stream,
                       self.pageStart).run()
                 except ConverterError, ce:
                     raise PodError(CONVERT_ERROR % str(ce))
@@ -865,15 +926,17 @@ class Renderer:
                 if self.stylesTemplate:
                     add('-t'); add(self.stylesTemplate)
                 if self.optimalColumnWidths:
-                    add('-o'); add('%s' % str(self.optimalColumnWidths))
+                    add('-o'); add(str(self.optimalColumnWidths))
                 if self.distributeColumns:
-                    add('-d'); add('%s' % str(self.distributeColumns))
+                    add('-d'); add(str(self.distributeColumns))
                 if self.script:
-                    add('-s'); add('%s' % self.script)
+                    add('-s'); add(self.script)
                 if self.resolveFields:
-                    add('-r'); add('%s' % self.resolveFields)
+                    add('-r'); add(self.resolveFields)
                 if self.pdfOptions:
-                    add('-f'); add('%s' % self.pdfOptions)
+                    add('-f'); add(self.pdfOptions)
+                if csvOptions:
+                    add('-i'); add(csvOptions)
                 if self.ppp: add('-c')
                 if self.stream != 'auto':
                     add('-a'); add(str(self.stream))
@@ -891,23 +954,23 @@ class Renderer:
                 raise pe
         return loOutput
 
-    def getTemplateType(self):
-        '''Identifies the type of the pod template in self.template
-           (ods or odt). If self.template is a string, it is a file name and we
-           simply get its extension. Else, it is a binary file in a StringIO
-           instance, and we seek the mime type from the first bytes.'''
-        if isinstance(self.template, basestring):
-            res = os.path.splitext(self.template)[1][1:]
+    def getTemplateType(self, template):
+        '''Identifies the type of this POD p_template (ods or odt). If
+           p_template is a string, it is a file name: simply get its extension.
+           Else, it is a binary file in a StringIO instance: seek the MIME type
+           from the first bytes.'''
+        if isinstance(template, basestring):
+            r = os.path.splitext(template)[1][1:]
         else:
             # A StringIO instance
-            self.template.seek(0)
-            firstBytes = self.template.read(90)
+            template.seek(0)
+            firstBytes = template.read(90)
             firstBytes = firstBytes[firstBytes.index('mimetype')+8:]
             if firstBytes.startswith(mimeTypes['ods']):
-                res = 'ods'
+                r = 'ods'
             else:
-                res = 'odt' # We suppose this is ODT
-        return res
+                r = 'odt' # We suppose this is ODT
+        return r
 
     def finalize(self):
         '''Re-zip the result and potentially call LibreOffice if target format
@@ -952,8 +1015,8 @@ class Renderer:
                 print(WARNING_FINALIZE_ERROR % str(e))
         # Re-zip the result, first as an OpenDocument file of the same type as
         # the POD template (odt, ods...)
-        resultExt = self.getTemplateType()
-        resultName = os.path.join(self.tempFolder, 'result.%s' % resultExt)
+        resultName = os.path.join(self.tempFolder,
+                                  'result.%s' % self.templateType)
         resultType = self.resultType
         zip(resultName, self.unzipFolder, odf=True)
         if (resultType in self.templateTypes) and not self.forceOoCall:
