@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -20,13 +20,12 @@ import re, os.path, random, string
 
 from appy.pod import *
 from appy import commercial
-from appy.pod.doc_importers import getUuid
 from appy.shared.errors import CommercialError
 from appy.shared.utils import getElementAt, formatNumber
 from appy.pod.odf_parser import OdfEnvironment, OdfParser
 from appy.shared.css import parseStyleAttribute, CssStyles, CssValue, px2cm
 
-# Possible states for the parser
+# Possible states for the styles parser
 READING = 0 # Default state
 PARSING_STYLE = 1 # Parsing a style definition
 PARSING_MASTER_STYLES = 2 # Parsing section "master-styles"
@@ -430,7 +429,7 @@ class PageStyle:
 
     def rename(self):
         '''Give to this style a new, unique name'''
-        self.uniqueName = getUuid(removeDots=True)
+        self.uniqueName = getUuid(prefix='PS', removeDots=True)
         # Update attributes
         for name in PageStyle.nameAttributes:
             if name in self.attrs:
@@ -864,7 +863,7 @@ class Css2odf:
     notNegInTable = 'self.inTable() and (val < 0)'
     ignore = {
       # Ignore negative text-indent and margin-left attributes within tables
-      'textindent': notNegInTable, 'marginleft': notNegInTable
+      'textindent': notNegInTable, 'marginleft': notNegInTable,
     }
 
     def __init__(self):
@@ -892,7 +891,7 @@ class Css2odf:
             return
         # Standardize the attribute for use within an ODF document
         unit = value.unit or ''
-        if unit:
+        if unit and (unit != '%'):
             ratio = self.px2cmRatios.get(name, px2cm)
             try:
                 val = value.cm(value=val, ratio=ratio)
@@ -922,7 +921,7 @@ class Css2odf:
         return (name in self.combined) and (' ' in value.value)
 
     def inTable(self):
-        '''In the XHTML > ODT convertion, are we currently parsing a table ?'''
+        '''In the XHTML > ODT conversion, are we currently parsing a table ?'''
         parser = self.xhtmlParser
         return parser and parser.env.currentTables
 
@@ -1018,6 +1017,8 @@ class StylesGenerator:
         self.stylesManager = stylesManager
         # Allows to convert CSS to ODF attributes
         self.css2odf = Css2odf()
+        # Are we generating an ODT or ODS file ?
+        self.isOds = stylesManager.renderer.templateType == 'ods'
 
     def addStyle(self, style, target):
         '''Adds the style definition dynamic styles'''
@@ -1128,10 +1129,12 @@ class StylesGenerator:
         family = self.getStyleFamily(xhtmlElem)
         style = '<style:style style:name="%s" style:family="%s"%s>%s%s' \
                 '</style:style>' % (styleName,family,parent,textProps,paraProps)
-        # I could have added this style in content.xml. But in some cases it
-        # does not work properly. For example, a percentage value for attribute
+        # Within an ODT file, injecting styles in content.xml does not work
+        # properly in some cases. For example, a percentage value for attribute
         # "fo-font-size" will be ignored if the style is dumped in content.xml.
-        self.addStyle(style, 'styles_base')
+        # Inversely, in an ODS file, it is preferable to dump everything in
+        # content.xml.
+        self.addStyle(style, self.isOds and 'content' or 'styles_base')
         # If p_add is True, also add the style to styleManager's defined styles
         # (as a Style instance). Else, it won't be found when it will need to be
         # applied on an inner paragraph.
