@@ -115,7 +115,7 @@ xhtmlInput = '''
 </table>
 <table border="undefined"><tr><td>No border</td></tr></table>
 <p style="text-indent: -1.00cm">Negative text indent kept (outside a table)</p>
-<table>
+<table style="margin-right:auto">
   <tbody>
     <tr>
     </tr>
