@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -195,16 +195,17 @@ class CssValue:
             r = value
         if unit == 'px': # Convert pixels to cm
             r = r / ratio
-            if formatted:
-                r = formatNumber(r, sep='.', precision=3)
         elif unit in ('em', 'ex'):
             # For "ex", the ratio must be adapted
             if unit == 'ex': ratio = ratio * 1.6
             r = (r / ratio) * 10
-            if formatted:
-                r = formatNumber(r, sep='.', precision=3)
+        elif unit == 'pt':
+            # Use a specific ratio
+            r *= 0.0352778
         elif unit != 'cm':
             r = None
+        if formatted and (r is not None):
+            r = formatNumber(r, sep='.', precision=3)
         return r
 
 # ------------------------------------------------------------------------------
@@ -231,7 +232,9 @@ class CssStyles:
     # Values, on combined attributes, that prevent splitting
     unsplittable = ('auto',)
     # Values that must be ignored
-    ignore = {'width': ['auto']}
+    ignore = {'*': {'auto':None, 'initial':None, 'inherit':None}}
+    # Values specified in ignore['*'] must not be ignored for these attributes
+    ignoreExcept = {'auto': {'table-layout':None}}
     # The '!important' CSS rule
     importantRule = '!important'
 
@@ -301,10 +304,13 @@ class CssStyles:
         if '-' in name: name = name.replace('-', '')
         return getattr(self, name, None)
 
-    def addClass(self, value):
+    def addClass(self, value, append=True):
         '''Add CSS class(es) p_value (as a string) to self.classes. p_value can
            hold several CSS space-separated classes.'''
-        self.classes = utils.sadd(self.classes, value)
+        # If p_append is True, CSS classes in p_value are appended to
+        # p_self.classes. Else, they are inserted at the start of
+        # p_self.classes.
+        self.classes = utils.sadd(self.classes, value, append=append)
 
     def deleteClass(self, value):
         '''Remove CSS class(es) in p_value from self.classes'''
@@ -341,7 +347,13 @@ class CssStyles:
         if isinstance(value, basestring) and value.endswith(self.importantRule):
             value = value[:-len(self.importantRule)].strip()
         # Do not add anything if the value must be ignored
-        if (name in self.ignore) and (value in self.ignore[name]): return
+        ignore = self.ignore
+        if (value in ignore['*']) or \
+           ((name in ignore) and (value in ignore[name])):
+               # Ignore it, excepted if there is a match in "ignoreExcept"
+               if (value not in self.ignoreExcept) or \
+                  (name not in self.ignoreExcept[value]):
+                   return
         attrName = name.replace('-', '')
         if not override and hasattr(self, attrName): return
         # Define the new value or combine it with the existing value when
