@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -36,9 +36,13 @@ class SsoConfig:
     ssoAttributes = {'loginKey': None, 'emailKey':'email', 'nameKey': 'name',
                      'firstNameKey': 'firstName', 'fullNameKey': 'title'}
     otherPrerogative = {'role': 'group', 'group': 'role'}
+
     # The connection to a SSO server can't really be tested
     testable = False
 
+    # Empty dict
+    emptyDict = {}
+
     def __init__(self):
         # One can give a name to the SSO reverse proxy that will call us
         self.name = ''
@@ -142,7 +146,11 @@ class SsoConfig:
         # appy.shared.ldap.LdapConfig.
         self.ldap = None
 
-    def init(self, tool): pass
+    def init(self, tool):
+        '''Lazy initialisation'''
+        # Lazy-initialise the sub-ldap config when present
+        if self.ldap:
+            self.ldap.init(tool)
 
     def __repr__(self):
         name = self.name or 'SSO reverse proxy'
@@ -281,16 +289,10 @@ class SsoConfig:
     def setGroups(self, tool, user, groups):
         '''Puts p_user into p_groups. Ensure those p_groups exist.'''
         for login in groups:
-            if login.startswith('*'):
-                # "login" is a pattern of groups. Query all groups and link the
-                # user to those matching the pattern.
-                suffix = login[1:]
-                for group in tool.search('Group', noSecurity=True):
-                    if group.login.endswith(suffix):
-                        group.link('users', user)
-                        if self.afterLinkGroup:
-                            self.afterLinkGroup(user, group)
-            else:
+            # Every "login" may represent a single group or, if it contains an
+            # asterisc, several groups.
+            i = login.find('*')
+            if i == -1:
                 # "login" is the login of a single group
                 group = tool.search1('Group', noSecurity=True, login=login)
                 if group:
@@ -299,6 +301,26 @@ class SsoConfig:
                         self.afterLinkGroup(user, group)
                 else:
                     tool.log(INVALID_GROUP % login, type='warning', noUser=True)
+            else:
+                # "login" is a pattern of groups. Search all groups matching the
+                # pattern.
+                if i == 0:
+                    # Groups share the same prefix. Get them all as basis.
+                    attrs = self.emptyDict
+                    suffix = login[1:]
+                else:
+                    # Limit the search to groups having the defined prefix
+                    attrs = {'login': login[:i+1]}
+                    if i == (len(login) - 1):
+                        suffix = ''
+                    else:
+                        suffix = login[i+1:]
+                # Search groups and retain those matching the pattern
+                for group in tool.search('Group', noSecurity=True, **attrs):
+                    if group.login.endswith(suffix):
+                        group.link('users', user)
+                        if self.afterLinkGroup:
+                            self.afterLinkGroup(user, group)
 
     def extractUserInfo(self, source, isRequest=True):
         '''By default, this method extracts, from HTTP headers present in the
