@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -414,28 +414,36 @@ def getDictFrom(s):
     res = {}
     if s:
         for part in s.split(','):
-            key, value = part.split(':')
-            res[key] = value
+            key, value = part.split(':', 1)
+            if value:
+                res[key] = value
     return res
 
-def sadd(s, sub, sep=' '):
-    '''Adds sub-string p_sub into p_s, which is a list of sub-strings separated
-       by p_sep, and returns the updated string.'''
+def sadd(s, sub, sep=' ', append=True):
+    '''Appends sub-string p_sub into p_s, which is a list of sub-strings
+       separated by p_sep, and returns the updated string. If p_append is False,
+       p_sub is inserted at the start of p_s instead.'''
     if not sub: return s
     if not s: return sub
-    elems = set(s.split(sep)).union(set(sub.split(sep)))
-    return sep.join(elems)
+    r = s.split(sep)
+    for part in sub.split(sep):
+        if part not in r:
+            if append:
+                r.append(part)
+            else:
+                r.insert(0, part)
+    return sep.join(r)
 
 def sremove(s, sub, sep=' '):
     '''Removes sub-string p_sub from p_s, which is a list of sub-strings
-       separated by p_sep, and returns the updated string.'''
+       separated by p_sep, and returns the updated string.'''    
     if not sub: return s
     if not s: return s
-    elems = set(s.split(sep))
-    for elem in sub.split(sep):
-        if elem in elems:
-            elems.remove(elem)
-    return sep.join(elems)
+    r = s.split(sep)
+    for part in sub.split(sep):
+        if part in r:
+            r.remove(part)
+    return sep.join(r)
 
 def stringIsAmong(s, l):
     '''Is p_s among list of strings p_l ? p_s can be a string or a
