@@ -1,7 +1,7 @@
 '''Appy is the simpliest way to build complex webapps'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -66,12 +66,18 @@ class Config:
     model = None
     # Place here a appy.ui.Config instance defining user-interface options
     ui = None
-    # Place here a appy.deploy.Config instance defining how to deploy this app
-    # on distant servers.
-    deploy = None
     # When using a SMTP mail server for sending emails from your app, place an
     # instance of class appy.utils.mail.Config in the field below.
     mail = None
+    # Place here a appy.server.scheduler.Config instance, defining cron-like
+    # actions being automatically triggered.
+    jobs = None
+    # Place here an instance of appy.server.backup.Config if you want to backup
+    # data and logs from a Appy site.
+    backup = None
+    # Place here a appy.deploy.Config instance defining how to deploy this app
+    # on distant servers.
+    deploy = None
     # When using Google analytics, specify here the Analytics ID
     googleAnalyticsId = None
     # When using Ogone, place an instance of appy.model.fields.ogone.Config in
@@ -104,4 +110,8 @@ class Config:
         self.server.check()
         self.server.static.check()
         self.security.check()
+        if self.jobs:
+            self.jobs.check()
+        if self.backup:
+            self.backup.check()
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
