@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -87,7 +87,7 @@ site = '{self.folder}'
 # Complete the config  - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def complete(c):
     c.model.set(app)
-    c.server.set(app)
+    c.server.set(app, site)
     c.server.port = {self.port}
     c.database.set(site + '/var')
     c.log.set(site + '/var/site.log', site + '/var/app.log')
@@ -98,7 +98,7 @@ def complete(c):
 appInit = '''
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from appy.database import log
-from appy.server import guard
+from appy.server import guard, scheduler, backup
 from appy import database, model, ui, server, deploy, peer
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -106,6 +106,8 @@ import appy
 class Config(appy.Config):
     server = server.Config()
     security = guard.Config()
+    backup = backup.Config()
+    jobs = scheduler.Config()
     database = database.Config()
     log = log.Config()
     model = model.Config()
@@ -378,7 +380,7 @@ class Make(Program):
     HELP_O       = "[site, unix/linux only] the owner of the site. If " \
                    "specified, the whole site tree will be chown'ed to this " \
                    "owner. Optionally, you may specify the group by " \
-                   "prefixing the owner by a colon and the group name, ie: " \
+                   "suffixing the owner by a colon and the group name, ie: " \
                    "appy:appy."
     HELP_D       = '[site] dependencies, as a series of local paths or ' \
                    'specifiers. For every local path, a symlink will be ' \
