@@ -1,7 +1,7 @@
 '''Runs a site'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -40,6 +40,9 @@ class Run(Program):
                   'and pack the database) or "run" (execute a specific method).'
     HELP_METHOD = 'When action in "run", specify the name of the method to ' \
                   'execute on your app\'s tool.'
+    HELP_WAIT   = 'When action is "stop" or "restart", specify a number of ' \
+                  'seconds to wait before sending the SIGINT signal to the ' \
+                  'Appy server.'
 
     # Error messages
     WRONG_ACTION = 'Unknown action "%s".'
@@ -57,6 +60,7 @@ class Run(Program):
         parser = self.parser
         parser.add_argument('action', help=Run.HELP_ACTION)
         parser.add_argument('-m', '--method', type=str, help=Run.HELP_METHOD)
+        parser.add_argument('-w', '--wait', type=int, help=Run.HELP_WAIT)
 
     def analyseArguments(self):
         '''Check arguments'''
@@ -65,13 +69,16 @@ class Run(Program):
         if action not in Run.allowedActions:
             self.exit(self.WRONG_ACTION % action)
         # Check that a method name is given when action is "run"
-        method = None
+        method = wait = None
         if action == 'run':
             method = self.args.method
             if method is None:
                 self.exit(self.MISSING_METHOD)
+        elif action in ('restart', 'stop'):
+            wait = self.args.wait
         self.action = action
         self.method = method
+        self.wait = wait
 
     def getPid(self, config):
         '''Return a pathlib.Path instance to the "pid" file, which, when the
@@ -113,9 +120,8 @@ class Run(Program):
     def start(self, config, exitIfInUse=True):
         '''Starts the server'''
         self.checkStart(config, exitIfInUse=exitIfInUse)
-        # Spawn a child process and run the server in it, in "bg" mode 
-        args = list(sys.argv)
-        args[-1] = 'bg'
+        # Spawn a child process and run the server in it, in "bg" mode
+        args = [sys.argv[0], 'bg']
         # Start the server in the background, in another process
         pid = subprocess.Popen(args).pid
         # Create a file storing the process ID, besides the DB file
@@ -129,10 +135,18 @@ class Run(Program):
            immediately restarted afterwards.'''
         pidFile = self.getPid(config)
         pid = self.checkStop(pidFile, restart=restart)
+        wasRunning = True
         if pid is not None:
-            os.kill(pid, signal.SIGINT)
-            pidFile.unlink()
-        if not restart:
+            # Wait first, if required
+            if self.wait: time.sleep(self.wait)
+            try:
+                os.kill(pid, signal.SIGINT)
+            except ProcessLookupError:
+                print('Server was not running (probably abnormally stopped).')
+                wasRunning = False
+            finally:
+                pidFile.unlink()
+        if not restart and wasRunning:
             print('Server stopped.')
 
     def run(self, site, app, ext=None):
@@ -165,16 +179,11 @@ class Run(Program):
             server = Server(config, action, self.method, ext=ext)
             # In classic modes "fg" or "bg", run the server
             if action in ('fg', 'bg'):
-                try:
-                    server.serve_forever()
-                except KeyboardInterrupt:
-                    server.shutdown()
-                    server.server_close()
-                    sys.exit(0)
+                server.serveForever()
             else:
                 # Everything has been done during server initialisation
                 server.shutdown()
-                sys.exit(0)
+            sys.exit(0)
 
         elif action == 'start':
             self.start(config)
