@@ -1,7 +1,7 @@
 '''Appy module managing database files'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -46,7 +46,7 @@ O_STORE_NF   = 'Object store does not exist in %s.'
 TOOL_NF      = 'Tool does not exist in %s.'
 METH_NF      = 'Method "%s" does not exist on the tool.'
 TMP_STORE_E  = 'No temp object has been deleted in %s.'
-TMP_OBJS_DEL = '%s temp object(s) removed from %s.'
+TMP_OBJS_DEL = '%s temp object(s) removed.'
 O_EXISTS     = 'An object with id "%s" already exists.'
 CLASS_NF     = 'Class "%s" does not exist.'
 CUS_ID_NO_IK = 'It is not possible to get an ikey for object having a ' \
@@ -59,6 +59,7 @@ MISS_O_CLASS = 'Catalog for class "%s": object not found for ID "%s".'
 MISS_O       = 'Object not found for ID "%s".'
 MISS_IDS_DEL = 'Catalog "%s": ID(s) removed because no corresponding ' \
                'object(s) found: %s.'
+FOLDER_DEL   = 'Folder %s deleted and moved to /tmp for object %d (%s).'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -140,6 +141,15 @@ class Config:
         r = os.stat(self.filePath).st_size
         return putils.getShownSize(r) if formatted else r
 
+    def getZodbVersion(self):
+        '''Gets the version of the ZODB'''
+        try:
+            import pkg_resources
+            r = pkg_resources.get_distribution('zodb').version
+        except ImportError:
+            r = 'unknown'
+        return r
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Database:
     '''Represents the app's database'''
@@ -324,7 +334,7 @@ class Database:
             Catalog.manageAll(root, handler)
             # Let the tool initialise itself and create sub-objects as required
             tool.init(handler, poFiles)
-            # Commit changes
+            # Commit changes (forced, even if handler.commit is False)
             transaction.commit()
         except self.Error as e:
             transaction.abort()
@@ -365,7 +375,7 @@ class Database:
         if not catalog:
             raise self.Error(SRCH_NO_CAT % className)
         return catalog.reindexObject(o, **kwargs)
-        
+
     def count(self, className): pass
     def compute(self, className): pass
 
@@ -389,8 +399,24 @@ class Database:
             logger.info(DB_PACKED % (putils.getShownSize(size),
                                      putils.getShownSize(newSize)))
 
+    def cleanTemp(self, handler, logger, count=None):
+        '''Removes any object from the temp store'''
+        connection = handler.dbConnection
+        if count is None:
+            # We have not counted the temp objects yet
+            store = getattr(connection.root, 'temp', None)
+            count = len(store) if store else 0
+        # For removing temp objects, it is useless to call m_delete on each one,
+        # because they are not yet linked to any other object, do not have yet a
+        # related filesystem space, etc.
+        connection.root.temp = PersistentMapping()
+        # Reset the ID counter
+        connection.root.lastTempId = 0
+        logger.info(TMP_OBJS_DEL % count)
+
     def clean(self, handler, logger):
-        '''Cleaning the database means removing any temp object from it'''
+        '''Cleaning the database means (a) removing any temp object from it and
+           (b) packing it.'''
         # Create a specific connection
         connection = handler.dbConnection = self.db.open()
         # Get the temp store
@@ -403,13 +429,7 @@ class Database:
         if count == 0:
             logger.info(TMP_STORE_E % dbPath)
         else:
-            # For removing temp objects, it is useless to call m_delete, because
-            # the object is not yet linked to any other object, does not have
-            # yet a related filesystem space, etc.
-            connection.root.temp = PersistentMapping()
-            # Reset the ID counter
-            connection.root.lastTempId = 0
-            logger.info(TMP_OBJS_DEL % (count, dbPath))
+            self.cleanTemp(handler, logger, count=count)
             transaction.commit()
         connection.close()
         # Pack the database
@@ -441,8 +461,9 @@ class Database:
         except Exception as err:
             handler.server.logTraceback()
             return abort(connection)
-        # Suppose p_method has changed something in the database
-        transaction.commit()
+        # Make a commit when relevant
+        if handler.commit:
+            transaction.commit()
         connection.close()
 
     def getIkey(self, id=None, o=None):
@@ -701,13 +722,15 @@ class Database:
                     className = o.translate(tied.class_.name)
                     tied.history.add('Unlink', deletion=True,
                                      comment='%s: %s' % (className, title))
-        # Unindex p_o if it was indexed
-        if o.class_.isIndexable() and not isTemp: o.reindex(unindex=True)
-        # Delete the filesystem folder corresponding to this object
-        folder = self.getFolder(o, create=False)
-        if folder.exists():
-            # Try to move it to the OS temp folder; if it fails, delete it
-            putils.FolderDeleter.delete(folder, move=True)
+        if not isTemp:
+            # Unindex p_o if it was indexed
+            if o.class_.isIndexable(): o.reindex(unindex=True)
+            # Delete the filesystem folder corresponding to this object
+            folder = self.getFolder(o, create=False)
+            if folder.exists():
+                # Try to move it to the OS temp folder; if it fails, delete it
+                putils.FolderDeleter.delete(folder, move=True)
+                o.log(FOLDER_DEL % (folder, o.iid, o.title))
         # Get the store containing p_o
         root = root or o.H().dbConnection.root
         iid = o.iid
@@ -829,10 +852,14 @@ class Database:
 
     view = Px('''
      <h2>Main configuration</h2>
-     <table class="small" var="cfg=config.database">
+     <table class="small"
+            var="cfg=config.database; root=handler.dbConnection.root">
       <tr><th>Database (ZODB)</th><td>:cfg.filePath</td></tr>
       <tr><th>Database size</th><td>:cfg.getDatabaseSize(True)</td></tr>
       <tr><th>Binaries</th><td>:cfg.binariesFolder</td></tr>
+      <tr><th>Last object ID</th><td>:root.lastId</td></tr>
+      <tr><th>Last temp ID</th><td>:root.lastTempId</td></tr>
+      <tr><th>ZODB version</th><td>:cfg.getZodbVersion()</td></tr>
      </table>
 
      <x var="connections=handler.server.database.listConnections()">
