@@ -1,7 +1,7 @@
 '''Management of the model behind a Appy application'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -69,7 +69,8 @@ class Model:
     # Names of base classes forming the base Appy model, in package appy.model,
     # and standard workflows defined in package appy.model.workflows.standard.
     # These classes and workflows are injected into any app's model.
-    baseClasses = ('Page', 'User', 'Group', 'Tool', 'Translation', 'Document')
+    baseClasses = ('Page', 'User', 'Group', 'Tool', 'Translation', 'Carousel',
+                   'Document', 'Query')
     baseWorkflows = ('Anonymous', 'Authenticated', 'Owner', 'TooPermissive')
 
     def __init__(self, config, classes, workflows):
