@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -19,7 +19,7 @@
 import persistent
 from appy.px import Px
 from appy.ui import Iframe
-from appy.utils import multicall
+from appy.ui.layout import Layout
 from appy.model.fields import Show
 from appy.database.lock import Lock
 from appy.ui.template import Template
@@ -31,27 +31,27 @@ from appy.model.fields.text import Text
 from appy.model.fields.date import Date
 from appy.model.utils import Object as O
 from appy.database.catalog import Catalog
-from appy.ui.layout import Layout, Layouts
 from appy.xml.marshaller import Marshaller
 from appy.model.fields.string import String
 from appy.model.fields.action import Action
 from appy.model.fields.computed import Computed
 from appy.model.workflow.history import History
 from appy.model.fields.phase import Page, UiPhases
+from appy.utils import multicall, MessageException
 from appy.model.workflow import standard as workflows
 from appy.model.fields.select import Select, Selection
 from appy.utils.string import Normalize, produceNiceMessage
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-READ_ONLY_EXPR = 'This is a read-only property: you cannot set a value for it.'
-FIELD_NOT_FOUND = 'Field "%s" on class "%s" does not exist.'
-MISSING_CLASS = '"new" called without classname.'
-REF_INSERT_NONE = 'Param "insert" for Ref field %s is None.'
-GET_CONTAINER_ERROR = "Container can't be retrieved that way."
-INDEX_TEMP = "A temp object can't be (un)indexed."
-CATALOG_NOT_FOUND = 'Catalog not found for %s instance.'
-NO_ID = 'getObject call with no ID.'
-NO_STR = 'Specify the name of the class as a string.'
+RO_EXPR     = 'This is a read-only property: you cannot set a value for it.'
+FIELD_KO    = 'Field "%s" on class "%s" does not exist.'
+MISS_CLS    = '"new" called without class name.'
+REF_INS_KO  = 'Param "insert" for Ref field %s is None.'
+CONT_ERR    = "Container can't be retrieved that way."
+INDEX_TEMP  = "A temp object can't be (un)indexed."
+CATALOG_KO  = 'Catalog not found for %s instance.'
+NO_ID       = 'Call to getObject with no ID.'
+NO_STR      = 'Specify the name of the class as a string.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Base:
@@ -164,7 +164,7 @@ class Base:
     # objects, like the tool or the currently logged user, or (ii) commonly
     # used functions.
     # ~~~ All these properties will have this setter, raising an exception
-    def raiseReadOnly(self, v): raise Exception(READ_ONLY_EXPR)
+    def raiseReadOnly(self, v): raise Exception(RO_EXPR)
 
     # The guard
     guard = property(lambda o: o.H().guard, raiseReadOnly)
@@ -254,10 +254,11 @@ class Base:
         # The layout may be forced by request parameter "pageLayout". Useful
         # when displaying an object in a particular context, ie, in a popup.
         req = self.req
-        if 'pageLayout' in req: return Layout(req.pageLayout)
-        # Retrieve the layout from the class, or get a default one
-        layouts = getattr(self.class_.python, 'layouts', Layouts.Page.defaults)
-        r = layouts[layout]
+        if 'pageLayout' in req:
+            pl = req.pageLayout
+            return pl if isinstance(pl, Layout) else Layout(pl)
+        # Retrieve layouts being applicable for p_self
+        r = self.class_.getPageLayouts(self)[layout]
         return r if not isinstance(r, str) else Layout(r)
 
     def getDefaultPage(self, layout):
@@ -309,6 +310,10 @@ class Base:
         '''Raise a security-related error'''
         raise self.guard.Error(msg)
 
+    def raiseMessage(self, message):
+        '''Raise a nicely-rendered error, without technical details'''
+        raise MessageException(message)
+
     def allows(self, permission='read', raiseError=False):
         '''Check the guard's homonym method'''
         return self.guard.allows(self, permission, raiseError=raiseError)
@@ -317,7 +322,7 @@ class Base:
     # URL methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    def buildUrl(self, name='', base='appy', ram=False, bg=False):
+    def buildUrl(self, name='', base=None, ram=False, bg=False):
         '''See doc in appy.server.Server::buildUrl'''
         H = self.H()
         return H.server.buildUrl(H, name=name, base=base, ram=ram, bg=bg)
@@ -339,7 +344,7 @@ class Base:
                                                           else self.getClass()
         r = class_.fields.get(name)
         if r is None and raiseError:
-            raise AttributeError(FIELD_NOT_FOUND % (name, class_.name))
+            raise AttributeError(FIELD_KO % (name, class_.name))
         return r
 
     def getFields(self, layout, page=None, phase=None, type=None, fields=None):
@@ -554,7 +559,7 @@ class Base:
             field = self.getField(name)
             insertMethod = field.insert
             if not insertMethod:
-                raise Exception(REF_INSERT_NONE % name)
+                raise Exception(REF_INS_KO % name)
             if not callable(insertMethod): insertMethod = insertMethod[1]
             keyMethod = lambda o: insertMethod(self, o)
         elif isinstance(sortKey, str):
@@ -624,7 +629,7 @@ class Base:
                 # In that case we cannot return the name of the back reference,
                 # because the initiator gives the name of the forward reference
                 # instead.
-                raise Exception(GET_CONTAINER_ERROR)
+                raise Exception(CONT_ERR)
             o = initiator.o
             return o if objectOnly else (o, initiator.field.name)
         # Get the container ID
@@ -719,7 +724,7 @@ class Base:
         '''Returns the catalog storing p_self (if indexed)'''
         r = self.H().dbConnection.root.catalogs.get(self.class_.name)
         if not r and raiseError:
-            raise Exception(CATALOG_NOT_FOUND % self.class_.name)
+            raise Exception(CATALOG_KO % self.class_.name)
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -781,8 +786,8 @@ class Base:
         <a class="clickable"
            onclick=":'submitAppyForm(%s,%s,%s)' % \
                       (q('save'), q(current.name), q('view'))">
-         <img src=":url('save.svg')" class=":picto"/><br/>
-         <x>::label</x>
+         <img src=":url('save.svg')" class=":picto"/>
+         <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
         </a>
        </div>
       </div>
@@ -793,8 +798,8 @@ class Base:
         <a class="clickable" name="cancel"
            onclick=":'submitAppyForm(%s,%s,%s)' % \
                       (q('cancel'), q(current.name), q('view'))">
-         <img src=":url('cancel.svg')" class=":picto"/><br/>
-         <x>::label</x>
+         <img src=":url('cancel.svg')" class=":picto"/>
+         <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
         </a>
        </div>
       </div>
@@ -804,8 +809,8 @@ class Base:
                not o.Lock.isSet(o, user, page)" var2="label=_('object_delete')">
        <div>
         <a class="clickable" onclick=":'onDeleteObject(%s)' % q(o.url)">
-         <img src=":url('delete.svg')" class=":picto"/><br/>
-         <x>::label</x>
+         <img src=":url('delete.svg')" class=":picto"/>
+         <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
         </a>
        </div>
       </div>
@@ -818,10 +823,10 @@ class Base:
       </x>
      </x>
 
-     <!-- Fields on layout "buttons". Layout is "defined as view" and not
+     <!-- Fields on layout "buttons". Layout is defined as "view" and not
           "cell", because "cell" would render a too minimalistic widget. -->
      <x if="not isEdit"
-        var2="fields=o.getFields('buttons', phase=phase.name);
+        var2="fields=o.getFields('buttons', page=req.page or 'main');
               layout='view'">
       <!-- Call view and not pxRender to avoid having a table -->
       <div for="field in fields"
@@ -916,7 +921,7 @@ class Base:
         return r
 
     def createFrom(self, _name, other, secure=False, executeMethods=True,
-                   exclude=(), keepBase=False, fields={}):
+                   exclude=(), keepBase=False, keepHistory=False, fields={}):
         '''Similar to m_create above, excepted that another object (p_other) is
            used as base for filling in data for the object to create.'''
         # p_exclude lists fields (by their names) that will not be copied from
@@ -934,8 +939,11 @@ class Base:
         # Create the new instance
         r = self.create(_name, secure=secure, raiseOnWrongAttribute=False,
                         executeMethods=executeMethods, **params)
-        # Propagate base attributes if required
-        if keepBase:
+        # Copy p_other's history events as-is if requested
+        if keepHistory: r.history.setEvents(other.history)
+        # Propagate base attributes if required (which is useless if history has
+        # already been cloned in its entirety).
+        if not keepHistory and keepBase:
             history = r.history
             # Keep the same creator as p_other
             history[-1].login = other.creator
@@ -978,15 +986,21 @@ class Base:
                 if field.translations:
                     return field.translations.get(label, language, mapping)
             label = getattr(field, '%sId' % label)
-        # Get the appropriate Translation object, or "en" if not found
+        # Get the appropriate Translation object, or, if not found, get the one
+        # corresponding to the defined fallback language.
         store = handler.dbConnection.root.objects
-        translation = store.get(language) or store.get('en')
-        # Get the label from this translation object, or from "en" if not found
+        translation = store.get(language) or \
+                      store.get(self.config.ui.fallbackLanguage)
+        # Get the label from this translation object, or from the fallback
+        # language if not found.
         if translation:
             r = translation.get(label, mapping)
-            if not r and ('en' in store):
-                # "en" may not be among app's languages
-                r = store.get('en').get(label, mapping)
+            if not r:
+                # Try to get a translation in the fallback language
+                fallback = self.config.ui.fallbackLanguage
+                if fallback in store:
+                    # The fallback language may not be among app's languages
+                    r = store.get(fallback).get(label, mapping)
         else:
             r = ''
         # If still no result, put a nice name derived from the label instead of
@@ -1080,10 +1094,11 @@ class Base:
              page=o.getCurrentPage(req, layout);
              page,grouped,css,js,phases=o.getGroupedFields(page, layout)">
       <x>::ui.Includer.getSpecific(tool, css, js)</x>
-      <x>::ui.Globals.getForms(tool)</x>
+      <x if="not req.notGlobal">::ui.Globals.getForms(tool)</x>
       <x var="tagId='pageLayout'; tagName=''; tagCss='';
               layoutTarget=o">:table.pxRender</x>
-      <x var="x=o.callOnView()">::ui.Globals.getScripts(tool, q, layout)</x>
+      <x if="not req.notGlobal"
+         var="x=o.callOnView()">::ui.Globals.getScripts(tool, q, layout)</x>
      </x>''', template=Template.px, hook='content')
 
     # The default element computed and returned when an object is reached by a
@@ -1179,7 +1194,7 @@ class Base:
           return warn_leave_form;
           }
         }
-      }''', template=Template.px, hook='content')
+      }''', template=Template.px, hook='content', name='edit')
 
     def getInitiator(self, search=False):
         '''Gets, from the request, information about a potential initiator'''
@@ -1277,8 +1292,11 @@ class Base:
         # or if the logged user has lost the right to view it, redirect him to
         # its home page.
         if (text is None) or not guard.mayView(self):
-            return self.goto(tool.computeHomePage(), message=text) \
-                   if not popup else Iframe.goBack(tool, initiator)
+            if popup:
+                if text: self.say(text)
+                return Iframe.goBack(tool, initiator)
+            else:
+                return self.goto(tool.computeHomePage(), message=text)
         # If we are here, action is "save" and p_self has been updated. Redirect
         # the user to the appropriate page.
         self.say(text)
@@ -1303,7 +1321,7 @@ class Base:
         req = self.req
         className = req.className
         if not className:
-            self.log(MISSING_CLASS, type='error')
+            self.log(MISS_CLS, type='error')
             self.raiseUnauthorized()
         # This requires a database commit
         handler = self.H()
@@ -1321,7 +1339,7 @@ class Base:
             g = req._get_
             if g: params['_get_'] = g
             # Is the creation of the new object via the initiator allowed ?
-            initiator.checkAllowed()
+            initiator.checkAllowed(log=True)
             # Let the initiator complete URL parameters when relevant
             initiator.updateParameters(params)
         # Create a temp object
@@ -1350,7 +1368,7 @@ class Base:
             text = self.translate('action_done')
             # If we were consulting the object itself, return to the home page.
             # Else, return to the referer page (it is the default).
-            if handler.getPublishedObject() == self:
+            if handler.isPublished(self):
                 back = self.tool.computeHomePage()
         # If we are called from an Ajax request, simply return the message
         if handler.isAjax(): return text
