@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -35,11 +35,9 @@ from appy.model.fields.group import Group, UiGroup
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 XSS_DETECTED = 'Detected Javascript in user input.'
-XSS_WARNING = 'Your behaviour is considered a security attack. System ' \
-              'administrator has been warned.'
-SEARCH_TRIGGER = '%s field "%s:%s": executing search "%s"...'
-SEARCH_DONE = 'Fetched %d object(s).'
-UNINDEXED = 'Field "%s": cannot retrieve catalog version of unindexed field.'
+XSS_WARNING  = 'Your behaviour is considered a security attack. System ' \
+               'administrator has been warned.'
+UNINDEXED    = 'Field "%s": cannot retrieve catalog version of unindexed field.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Show:
@@ -117,7 +115,7 @@ class Initiator:
            showing self.field.'''
         return self.o.getUrl(sub='view', page=self.field.pageName, nav='no')
 
-    def checkAllowed(self):
+    def checkAllowed(self, log=False):
         '''Checks that adding an object via this initiator is allowed'''
         return True
 
@@ -194,10 +192,6 @@ class Field:
     # In most cases, outer fields store persistent lists
     outerType = PersistentList
 
-    # Parameters needed to re-trigger a search. Some fields applying on sets of
-    # objects may need to re-trigger a search from these request parameters.
-    searchParams = ('className', 'search', 'sortKey', 'sortOrder', 'filters')
-
     # The initiator class related to the Field
     initiator = Initiator
 
@@ -237,13 +231,15 @@ class Field:
       layoutTarget=field">:field.getPx(minimal, \
                                        hostLayout or layout, _ctx_)</x>''')
 
-    def doRender(self, layout, o, name=None, minimal=False, tagCss=None):
+    def doRender(self, layout, o, name=None, minimal=False, tagCss=None,
+                 specific=None):
         '''Allows to call pxRender from code, to display the content of this
            field on p_o in some specific context, for example in a Computed
            field.'''
         context = O(layout=layout, field=self, minimal=minimal,
                     name=name or self.name, o=o)
         if tagCss: context.tagCss = tagCss
+        if specific: context.update(specific)
         # Get the current PX context, to copy some entries from it. If no such
         # context exists, create one.
         ctx = o.traversal.context or o.traversal.createContext(layout=layout)
@@ -287,10 +283,10 @@ class Field:
        <x if="hasattr(o, 'getSupTitle')">::o.getSupTitle(navInfo)</x>
        <x>::ui.Title.get(o, mode=mode.titleMode, nav=navInfo, target=target, \
           page=pageName, popup=popup, selectJs=selectJs, highlight=True, \
-          backHook=backHook)</x>
+          backHook=backHook, pageLayout=uiSearch.search.pageLayoutOnView)</x>
        <span if="hasattr(o,'getSubTitle')"
              class=":class_.getCssFor(o,'subTitle')"
-             style=":mode.showSubTitles and 'display:inline' or 'display:none'"
+             style=":'display:inline' if mode.showSubTitles else 'display:none'"
              name="subTitle">::uiSearch.highlight(o.getSubTitle())</span>
 
        <!-- Actions -->
@@ -838,10 +834,11 @@ class Field:
             if rowId == '-1': r = None # It is a template row
             else:
                 # Get the outer value
-                r = o.req[outerName] if fromRequest else o.values.get(outerName)
+                outerField = o.getField(outerName)
+                r = o.req[outerName] if fromRequest else outerField.getValue(o)
                 # Access the inner value
                 if r:
-                    if o.getField(outerName).outerType == PersistentList:
+                    if outerField.outerType == PersistentList:
                         # The row ID is the index if an element from a list
                         rowId = int(rowId)
                         if rowId < len(r):
@@ -891,8 +888,9 @@ class Field:
            suffix of the "main" HTML element.'''
         return ''
 
-    def getValue(self, o, name=None, layout=None):
+    def getValue(self, o, name=None, layout=None, single=None):
         '''Gets, on p_o(bject), the value for this field (p_self)'''
+
         # Possible values for parameters are described hereafter.
         # ----------------------------------------------------------------------
         # name    | p_name can be different from self.name if p_self is a
@@ -900,11 +898,19 @@ class Field:
         #         | number.
         # ----------------------------------------------------------------------
         # layout  | In most cases, we don't care about the layout for getting
-        #         | the value of p_self on p_obj. One exception is that a
-        #         | default value can be defined specifically on layout "edit".
-        #         | In that case, p_layout must be available in order to
-        #         | determine the possible specific default value for this
-        #         | layout.
+        #         | the value of p_self on p_o. One exception is that a default
+        #         | value can be defined specifically on layout "edit". In that
+        #         | case, p_layout must be available in order to determine the
+        #         | possible specific default value for this layout.
+        # ----------------------------------------------------------------------
+        # single  | This attribute, although defined here at the most abstract
+        #         | level, has only sense for "multi-values", like values from
+        #         | the Ref field. Such values are always stored as lists or
+        #         | tuples but, when field multiplicity is (x, 1), it is
+        #         | sometimes convenient to retrieve the single stored value.
+        #         | This can be achieved with p_single being True. If p_single
+        #         | is None or False, the retrieved value will be the whole
+        #         | list or tuple.
         # ----------------------------------------------------------------------
         # Get the value from the database
         value = self.getStoredValue(o, name)
@@ -948,6 +954,22 @@ class Field:
            if this value is mutable, get a copy of it.'''
         return self.getValue(o)
 
+    def getComparableValue(self, o):
+        '''Get the value of this field on p_o, as can be compared with its new
+           version, coming from the validator.'''
+        # For immutable values, it simply returns the currently stored value.
+        # For mutable values, there are 2 cases:
+        # (a) the value as produced by the validator completely overwrites the
+        #     existing value. Thus, the previous and new versions can safely be
+        #     compared, because being completely distinct. In this case, as for
+        #     immutable values, the currently stored value can also be used
+        #     as-is, as "comparable" value ;
+        # (b) the value as produced by the validator updates the stored value.
+        #     In this case, comparing the previous and new versions would lead
+        #     to comparing the same value. Here, the "comparable" value must be
+        #     a deep copy of the previous value and not the stored value itself.
+        return self.getValue(o, single=False)
+
     def getFormattedValue(self, o, value, layout='view', showChanges=False,
                           language=None):
         '''p_value is a real p_o(bject) value from a field from this type. This
@@ -986,10 +1008,10 @@ class Field:
            p_language is specified, p_value is the p_language-specific part of a
            multilingual field. p_value is in p_o's history, at entry whose index
            is p_i.'''
-        if language:
-            r = field.getUniFormattedValue(o, value, contentLanguage=language)
+        if language is None:
+            r = self.getFormattedValue(o, value)
         else:
-            r = field.getFormattedValue(o, value)
+            r = self.getUniFormattedValue(o, value, contentLanguage=language)
         # If p_r is a list, transform it into a HTML "ul" tag
         if type(r) in utils.sequenceTypes:
             r = '<ul>%s</ul>' % ''.join(['<li>%s</li>' % v for v in r])
@@ -1055,6 +1077,11 @@ class Field:
             r = '<span class="editable" %s>%s</span>' % (onClick, value)
         return r
 
+    def getFakeObject(self, value, req):
+        '''Returns a fake object, to mimic an object storing this p_value for
+           field p_self.'''
+        return O(values={self.name: value}, req=req, id=0, iid=0)
+
     def getIndexType(self, class_=False):
         '''Returns the name of the class corresponding to the index used to
            store values of this field in a database catalog, or the class itself
@@ -1137,6 +1164,12 @@ class Field:
         if self.isEmptyValue(o, value): return
         return value
 
+    def getFilterValue(self, value):
+        '''p_value is a valid value for p_self, as produced by
+           m_getStorableValue. This method applies a potential transform on it
+           in order to be used as filter value for a search.'''
+        return value
+
     def getInputValue(self, inRequest, requestValue, value):
         '''Gets the value that must be filled in the "input" widget
            corresponding to this field.'''
@@ -1222,69 +1255,6 @@ class Field:
            as complete without the "hour, minute" part.'''
         return not self.isEmptyValue(o, value)
 
-    def getCheckedInfo(self, req):
-        '''Gets the status of checked elements from the p_req(uest)'''
-        # It returns a tuple (ids, unchecked):
-        # ----------------------------------------------------------------------
-        #   "ids"     | is a dict of object IIDs;
-        # "unchecked" | is True if these objects are unchecked, or False if
-        #             | they are checked.
-        # ----------------------------------------------------------------------
-        
-        ids = {int(v) for v in req.checkedIds.split(',')} if req.checkedIds \
-                                                          else {}
-        return ids, req.checkedSem == 'unchecked'
-
-    def getSearchParams(self, req, layout):
-        '''Encodes in a string all the params in the request that are required
-           for re-triggering a search.'''
-        # If key "search" is present in the request, but empty, it represents
-        # the special search named "allSearch".
-        if ('search' in req) and not req.search:
-            req.search = 'allSearch'
-        if not req.search or (layout == 'cell'): return ''
-        return ':'.join([(req[key] or '') for key in Field.searchParams])
-
-    def getSearchResults(self, tool, searchParams):
-        '''Execute the search whose parameters (p_searchParams) are returned by
-           m_getSearchParams and return its results. Log the operation.'''
-        # Get search parameters as an object
-        params = O()
-        i = 0
-        for part in searchParams.split(':', len(Field.searchParams) - 1):
-            setattr(params, Field.searchParams[i], part)
-            i += 1
-        # Retrieve a Ref object when relevant
-        Search = tool.Search
-        refObject, refField = Search.getRefInfo(tool)
-        # Get the Search instance
-        class_ = tool.model.classes.get(params.className)
-        search = Search.get(params.search, tool, class_, None, ui=False)
-        # Executing the search may take some time and potentially slow down
-        # the system: log this action.
-        tool.log(SEARCH_TRIGGER % (self.type, class_.name, self.name,
-                                   search.name))
-        # (Re-)execute the search, but without restricting the number of results
-        r = search.run(tool.H(), batch=False,
-          sortBy=params.sortKey, sortOrder=params.sortOrder,
-          filters=sutils.getDictFrom(params.filters),
-          refObject=refObject, refField=refField)
-        tool.log(SEARCH_DONE % len(r))
-        return r
-
-    def keepCheckedResults(self, req, objects):
-        '''Among p_objects as retrieved via m_getSearchResults, keep only those
-           being checked, according to m_getCheckedInfo.'''
-        ids, unchecked = self.getCheckedInfo(req)
-        i = len(objects) - 1
-        # Remove, from search results, unchecked objects
-        while i >= 0:
-            if unchecked: remove = objects[i].iid in ids
-            else:         remove = objects[i].iid not in ids
-            if remove:
-                del objects[i]
-            i -= 1
-
     def validateValue(self, o, value):
         '''This method may be overridden by child classes and will be called at
            the right moment by m_validate defined below for triggering
@@ -1426,8 +1396,6 @@ class Field:
                 o.log('method %s:\n%s' % (method.__name__, tb), type='error')
                 # Raise the initial error
                 raise te
-        except Exception as e:
-            raise e
 
     def getAttribute(self, o, name, cache=True):
         '''Gets the value of attribute p_name on p_self, which can be a simple
@@ -1491,6 +1459,22 @@ class Field:
         # height via a attribute "style", not "size".
         return ''
 
+    def getInputSize(self, sizeAttr=True):
+        '''Returns the width of the input field, either in the "size" attribute,
+           (p_sizeAttr is True) or as a CSS "width" attribute (p_sizeAttr is
+           False).'''
+        # If the field width is expressed as a percentage, the width must be
+        # specified via the CSS "width" attribute. In all other cases, it is
+        # specified via the "size" tag attribute.
+        width = self.width
+        isPercentage = width and isinstance(width, str) and width.endswith('%')
+        if sizeAttr:
+            # Tag attribute "size"
+            return '' if isPercentage else width
+        else:
+            # CSS attribute "width"
+            return 'width:%s' % width if isPercentage else ''
+
     def getSelectStyle(self, isSearch, isMultiple):
         '''If this field must be rendered as a HTML "select" field, get the
            value of its "style" attribute. p_isSearch is True if the field is
@@ -1514,6 +1498,20 @@ class Field:
             # attribute.
         return ';'.join(res)
 
+    def getRadioStyle(self, isSearch=False):
+        '''If this field must be rendered as radio buttons or checkboxes, and
+           its height is specified as a string, restrict the visible part of the
+           widget to this height, with a scrollbar when appropriate.'''
+        prefix = 's' if isSearch else ''
+        r = []
+        height = getattr(self, '%sheight' % prefix)
+        if isinstance(height, str):
+            r.append('height:%s;overflow-y:auto' % height)
+        width = getattr(self, '%swidth' % prefix)
+        if isinstance(width, str):
+            r.append('width:%s;overflow-x:auto' % width)
+        return ';'.join(r)
+
     def getWidthInChars(self, isSearch):
         '''If attribute "width" contains an integer value, it contains the
            number of chars shown in this field (a kind of "logical" width). If
