@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,6 +21,7 @@ from appy.px import Px
 from appy import utils
 from appy.ui import LinkTarget
 from appy.ui.layout import Layouts
+from appy.model.searches import Search
 from appy.model.fields import Field, Initiator, Show
 
 # Constants  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -113,7 +114,7 @@ class Action(Field):
       <x if="multi">
        <input type="hidden" name="multi" value="True"/>
        <input type="hidden" name="searchParams"
-              value=":field.getSearchParams(req, layout)"/>
+              value=":tool.Search.encodeForReplay(req, layout)"/>
        <input type="hidden" name="checkedIds"/>
        <input type="hidden" name="checkedSem"/>
        <!-- The parameter starting with a star indicates to collect search
@@ -128,8 +129,8 @@ class Action(Field):
          title=":isFake if isFake else ''"
          onclick=":field.getOnClick(o, name, req, layout, q, multi) \
                    if not isFake else ''">
-       <img src=":url(field.icon, base=field.iconBase)" class=":picto"/><br/>
-       <x>::inputTitle</x>
+       <img src=":url(field.icon, base=field.iconBase)" class=":picto"/>
+       <div style=":'display:%s' % config.ui.pageDisplay">::inputTitle</div>
       </a>
 
       <!-- As button -->
@@ -392,13 +393,13 @@ class Action(Field):
         if options: args['options'] = options
         if self.confirm == 'comment':
             args['comment'] = o.req.popupComment
-        # Is that a multi-action? A multi-action is an action to perform on a
+        # Is that a multi-action ? A multi-action is an action to perform on a
         # list of objects instead of a single object.
         if req.multi:
-            # Re-trigger the search to get the list of objects
-            objects = self.getSearchResults(o.tool, req.searchParams)
+            # Replay the search to get the list of objects
+            objects = Search.replay(o.tool, req.searchParams)
             # Remove those not being checked in the UI
-            self.keepCheckedResults(req, objects)
+            Search.keepCheckedResults(req, objects)
             args['objects'] = objects
         # Call method(s) in self.action
         if type(self.action) in utils.sequenceTypes:
@@ -432,7 +433,7 @@ class Action(Field):
                               comment=args.get('comment'))
         return r
 
-    def getValue(self, o, name=None, layout=None):
+    def getValue(self, o, name=None, layout=None, single=None):
         '''Call the action and return the result'''
         return self(o)
 
