@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -135,12 +135,11 @@ class Boolean(Field):
           persist, inlineEdit, view, cell, edit, xml, translations)
         self.pythonType = bool
 
-    def getValue(self, obj, name=None, layout=None):
+    def getValue(self, o, name=None, layout=None, single=None):
         '''Never returns "None". Returns always "True" or "False", even if
            "None" is stored in the DB.'''
-        value = Field.getValue(self, obj, name, layout)
-        if value is None: return False
-        return value
+        value = Field.getValue(self, o, name, layout, single)
+        return False if value is None else value
 
     def getValueLabel(self, value):
         '''Returns the label for p_value (True or False): if self.render is
