@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -58,7 +58,7 @@ class Date(Field):
     pxHour = Hour.edit
 
     edit = Px('''
-     <x var="years=field.getSelectableYears()">
+     <x var="years=field.getSelectableYears(o)">
       <!-- Day -->
       <select if="field.showDay" var2="days=range(1,32); part='%s_day' % name"
               name=":part" id=":part">
@@ -98,7 +98,7 @@ class Date(Field):
      </x>''')
 
     search = Px('''
-     <table var="years=range(field.startYear, field.endYear+1);
+     <table var="years=field.getSelectableYears(o);
                  dstart,dend=field.getDefaultSearchValues(o)">
        <!-- From -->
        <tr var="fromName='%s_from' % name;
@@ -119,7 +119,7 @@ class Date(Field):
          </select> / 
          <select id=":widgetName" name=":widgetName">
           <option value="">--</option>
-          <option for="value in range(field.startYear, field.endYear+1)"
+          <option for="value in years"
                   value=":value" selected=":value == dstart[0]">:value</option>
          </select>
          <!-- The icon for displaying the calendar popup -->
@@ -156,7 +156,7 @@ class Date(Field):
          </select> / 
          <select id=":yearToName" name=":yearToName">
           <option value="">--</option>
-          <option for="value in range(field.startYear, field.endYear+1)"
+          <option for="value in years"
                   value=":value" selected=":value == dend[0]">:value</option>
          </select>
          <!-- The icon for displaying the calendar popup -->
@@ -228,11 +228,12 @@ class Date(Field):
         '''Javascript files are only required if the calendar must be shown'''
         if self.calendar: Field.getJs(self, layout, r, config)
 
-    def getSelectableYears(self):
-        '''Gets the list of years one may select for this field.'''
-        res = range(self.startYear, self.endYear + 1)
-        if self.reverseYears: res.reverse()
-        return res
+    def getSelectableYears(self, o):
+        '''Gets the list of years one may select for this field'''
+        startYear = self.getAttribute(o, 'startYear')
+        r = range(startYear, self.endYear + 1)
+        if self.reverseYears: r.reverse()
+        return r
 
     def validateValue(self, obj, value):
         try:
