@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,11 +16,13 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+import copy
 from persistent.mapping import PersistentMapping
 
 from appy.px import Px
 from appy.ui.layout import Layout
 from appy.model.utils import Object
+from appy.utils.diff import HtmlDiff
 from appy.model.fields.list import List
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -69,7 +71,8 @@ class Dict(List):
      <table var="isEdit=layout == 'edit'" if="isEdit or value"
             id=":'list_%s' % name" class="grid" width=":field.width"
             var2="keys=field.keys(o);
-                  subFields=field.getSubFields(o, layout)">
+                  subFields=field.getSubFields(o, layout);
+                  o=alto|o">
       <!-- Header -->
       <tr valign="bottom">
        <th width=":field.widths[0]"></th>
@@ -118,6 +121,13 @@ class Dict(List):
             r[k] = self.getStorableRowValue(o, v)
         return r
 
+    def getComparableValue(self, o):
+        '''Return a (deep) copy of field value on p_o'''
+        # Indeed, because a new value does not overwrite but updates the stored
+        # value, the comparable value must be a deep copy of the stored value.
+        r = getattr(o, self.name, None)
+        if r: return copy.deepcopy(r)
+
     def remove(self, o, key):
         '''Remove entry corresponding to p_key on the value stored on p_o'''
         if self.name not in o.values: return
@@ -142,6 +152,8 @@ class Dict(List):
                     dbValue[key] = data
                 else:
                     dbValue[key].update(data)
+                    # Force the mapping to take the change into account
+                    dbValue[key] = dbValue[key]
 
     def subValidate(self, o, value, errors):
         '''Validates inner fields'''
@@ -150,4 +162,63 @@ class Dict(List):
                 message = subField.validate(o, getattr(row, name, None))
                 if message:
                     setattr(errors, '%s*%s' % (subField.name, key), message)
+
+    def getDiffSubValue(self, old, new, texts):
+        '''Produce a diff between this p_old sub-value and its p_new version.
+           p_texts contain precomputed translated texts about the user that
+           performed the change.'''
+        if old:
+            r = HtmlDiff.templateDiv % (HtmlDiff.deleteStyle, texts[1], old)
+        else:
+            r = ''
+        if new:
+            r = '%s%s' % (r, HtmlDiff.templateDiv % (HtmlDiff.insertStyle,
+                                                     texts[0], new))
+        return r
+
+    def getDiffValue(self, o, old, new, texts):
+        '''Return a XHTML representation of dict value p_new incorporating the
+           differences with this p_old version.'''
+        # Return the p_old value as-is, if the p_new version is None
+        if not new:
+            fake = self.getFakeObject(old, o.req)
+            return self.doRender('view', fake)
+        # Update the old value by replacing every modified sub-value (w.r.t
+        # p_new) with a diff between the old and the new sub-value.
+        diff = copy.deepcopy(old)
+        for key, sub in old.items():
+            # Get the corresponding entry in p_new
+            newSub = new.get(key)
+            diffSub = diff[key]
+            if newSub:
+                # Browse sub-values on this row of data
+                for name, val in sub.items():
+                    # Get the new version of sub-value
+                    newVal = getattr(newSub, name, None)
+                    if newVal == val: continue
+                    # Values are different, produce a diff
+                    diffVal = self.getDiffSubValue(val, newVal, texts)
+                    setattr(diffSub, name, diffVal)
+            else:
+                # This complete row has been removed from p_new
+                for name, val in sub.items():
+                    diffVal = self.getDiffSubValue(val, None, texts)
+                    setattr(diffSub, name, diffVal)
+        # Produce a fake object containing the diff
+        fake = self.getFakeObject(diff, o.req)
+        # Render the diff
+        return self.doRender('view', o, minimal=True, specific={'alto': fake})
+
+    def getHistoryValue(self, o, value, i, language=None, empty='-'):
+        '''For a Dict field, instead of showing the previous p_value in p_o's
+           history, show a XHTML diff between p_value, as found at index p_i,
+           and a newer version of it in p_o's history before p_i, or, if not
+           found, the value as currently stored on p_o.'''
+        # Return an empty value if the p_value is inexistent
+        if not value: return empty
+        # Find the newer version of p_value
+        history = o.history
+        newer = history.getNewer(self, value, i-1)
+        diffTexts = history[i].getDiffTexts(o)
+        return self.getDiffValue(o, value, newer, diffTexts)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
