@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -30,6 +30,7 @@ from appy.model.utils import Object
 from appy.server.static import Static
 from appy.utils import path as putils
 from appy.utils import string as sutils
+from appy.pod.converter import Converter
 from appy.xml.unmarshaller import UnmarshalledFile
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -171,10 +172,10 @@ class FileInfo:
             # An in-database file for which we do not have the full path
             raise Exception(PATH_KO % self.fsName)
         # Produce the file via the Static class
-        Static.write(handler, path, None, response=handler.resp, fileInfo=self,
+        Static.write(handler, path, None, fileInfo=self,
                      disposition=disposition, enableCache=cache)
 
-    def writeFile(self, name, value, folder):
+    def writeFile(self, name, value, folder, config=None):
         '''Writes a file to the filesystem, from p_value that can be:
            - an Object instance (coming from a HTTP post);
            - another ("not-in-DB") FileInfo instance;
@@ -204,7 +205,13 @@ class FileInfo:
             self.size = len(content)
             f.write(content)
         elif isinstance(value, FileInfo):
-            src = open(value.fsPath, 'rb')
+            fsPath = value.fsPath
+            if not fsPath.startswith('/'):
+                # The p_value is a "in-db" FileInfo instance. If p_config is
+                # there, we can reconstitute its absolute path.
+                fsPath = config.database.binariesFolder / fsPath / value.fsName
+                fsPath = str(fsPath)
+            src = open(fsPath, 'rb')
             self.size = self.replicateFile(src, f)
             src.close()
         else:
@@ -223,6 +230,7 @@ class FileInfo:
         '''Copies the "external" file stored at p_filePath in the db-controlled
            file system, for storing a value for p_fieldName.'''
         # Set names for the file
+        filePath = str(filePath) if isinstance(filePath, Path) else filePath
         name = self.normalizeFileName(filePath)
         self.uploadName = name
         self.fsName = '%s%s' % (fieldName, os.path.splitext(name)[1])
@@ -250,7 +258,7 @@ class FileInfo:
         shutil.copyfile(self.getFilePath(o), filePath)
         if format:
             # Convert the dumped file using LibreOffice
-            out, err = o.tool.convert(filePath, format)
+            out, err = Converter(filePath, format).run()
             # Even if we have an "error" message, it could be a simple warning.
             # So we will continue here and, as a subsequent check for knowing if
             # an error occurred or not, we will test the existence of the
@@ -345,7 +353,7 @@ class File(Field):
       sdefault='', scolspan=1, swidth=None, sheight=None, view=None, cell=None,
       edit=None, xml=None, translations=None, render=None, icon='paperclip',
       disposition='attachment', nameStorer=None, cache=False, resize=False,
-      thumbnail=None):
+      thumbnail=None, viewWidth=None):
         # This boolean is True if the file is an image
         self.isImage = isImage
         # "downloadAction" can be a method called every time the file is
@@ -371,7 +379,7 @@ class File(Field):
         # By default, caching is disabled for this field: the browser will not
         # be able to cache the file stored in it. This is the default, for
         # security reasons. If you decide the browser can cache the file stored
-        # here, set the foolowing attribute to True. Attribute p_cache can also
+        # here, set the following attribute to True. Attribute p_cache can also
         # store a method that must return a boolean value.
         self.cache = cache
         # Attribute "width" is used to specify the width of the image or
@@ -393,6 +401,11 @@ class File(Field):
         # in p_self, resized accordingly (provided you have defined it as
         # "resizable").
         self.thumbnail = thumbnail
+        # When the uploaded file is an image, on "view" and "cell" layouts, you
+        # may want to display it smaller than its actual size. In order to do
+        # that, specify a width (as a string, ie '700px') in the following
+        # attribute. The attribute may also hold a method returning the value.
+        self.viewWidth = viewWidth
         # Call the base constructor
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
           show, page, group, layouts, move, False, True, None, None, False,
@@ -456,7 +469,8 @@ class File(Field):
         # For images, display them directly
         if self.isImage:
             # Define a max width when relevant
-            css = ' style="max-width: %s"' % self.width if self.width else ''
+            viewWidth = self.getAttribute(o, 'viewWidth')
+            css = ' style="max-width: %s"' % viewWidth if viewWidth else ''
             return '<img src="%s" title="%s"%s/>' % (url, title, css)
         # For non-images, display a link for downloading it, as an icon when
         # relevant.
@@ -504,8 +518,8 @@ class File(Field):
         #    | the same attributes, representing a file unmarshalled from XML
         #    | content.
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        #  b | a string. In this case, the string represents the path of a file
-        #    | on disk;
+        #  b | a string or pathlib.Path instance. In this case, it represents
+        #    | the path of a file on disk;
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  c | a 2-tuple (name, content) where:
         #    | - "name"     is the name of the file (ie "myFile.odt")
@@ -516,8 +530,7 @@ class File(Field):
         #    | - "name" and "content" have the same meaning than above;
         #    | - "mimeType" is the MIME type of the file.
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        #  e | a FileInfo instance, that must be "not-in-DB", ie, with an
-        #    | absolute path in attribute fsPath.
+        #  e | a FileInfo instance, be it "in-DB" or not.
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if value:
             # There is a new value to store. Get the folder on disk where to
@@ -531,8 +544,8 @@ class File(Field):
             if isinstance(value, Object) or isinstance(value, FileInfo) or \
                isinstance(value, UnmarshalledFile):
                 # Cases (a) and (e)
-                info.writeFile(self.name, value, folder)
-            elif isinstance(value, str):
+                info.writeFile(self.name, value, folder, config=o.config)
+            elif isinstance(value, str) or isinstance(value, Path):
                 # Case (b)
                 info.copyFile(self.name, value, folder)
             else:
@@ -549,7 +562,9 @@ class File(Field):
             # Store the FileInfo instance in the database
             o.values[self.name] = info
             # Resize the image when relevant
-            if self.resize and self.width: info.resize(folder, self.width, o)
+            if self.resize and self.width:
+                width = self.getAttribute(o, 'width')
+                info.resize(folder, width, o)
             # Update the thumbnail if defined
             fileName = str(folder / info.fsName)
             if self.thumbnail: o.getField(self.thumbnail).store(o, fileName)
@@ -577,15 +592,15 @@ class File(Field):
             if not path.is_file():
                 Static.notFound(handler, config)
                 return
-            # Send the file to the browser
-            info.writeResponse(handler, str(path),
-                               disposition=self.getAttribute(o, 'disposition'),
-                               cache=self.getAttribute(o, 'cache'))
             # Call the "download action" if specified
             if self.downloadAction:
                 self.downloadAction(o, info)
                 # In that case, a commit is required
                 o.H().commit = True
+            # Send the file to the browser
+            info.writeResponse(handler, str(path),
+                               disposition=self.getAttribute(o, 'disposition'),
+                               cache=self.getAttribute(o, 'cache'))
         else:
             # Return a 404
             Static.notFound(handler, config)
