@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -94,6 +94,9 @@ class Integer(Field):
           sheight, persist, inlineEdit, view, cell, edit, xml, translations)
         # Define the corresponding Python type for values stored for this type
         self.pythonType = int
+        # Define the filter PX when appropriate
+        if self.indexed:
+            self.filterPx = 'pxFilterText'
 
     def getFormattedValue(self, o, value, layout='view', showChanges=False,
                           language=None):
