@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -191,7 +191,7 @@ class Ogone(Field):
         r = (passphrase.join(r) + passphrase).encode()
         return hashlib.sha512(r).hexdigest()
 
-    def getValue(self, o, name=None, layout=None):
+    def getValue(self, o, name=None, layout=None, single=None):
         '''The "value" of the Ogone field is a dict that collects all the
            necessary info for making the payment.'''
         tool = o.tool
