@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -55,14 +55,14 @@ class Password(Field):
         var2="placeholder=field.getPlaceholder(o, confirm);
               inputId=confirm and ('%s_confirm' % name) or name">
       <input type="password" id=":inputId" name=":inputId"
-             style=":'width: %s' % field.width"
+             size=":field.getInputSize()" style=":field.getInputSize(False)"
              maxlength=":field.maxChars" placeholder=":placeholder"/>
       <br if="not confirm"/>
      </x>''')
 
     def __init__(self, validator=None, multiplicity=(0,1), show=True,
       page='main', group=None, layouts=None, move=0, readPermission='read',
-      writePermission='write', width='15em', height=None, maxChars=None,
+      writePermission='write', width='25em', height=None, maxChars=None,
       colspan=1, master=None, masterValue=None, focus=False, historized=False,
       mapping=None, generateLabel=None, label=None, sdefault='', scolspan=1,
       swidth=None, sheight=None, persist=True, placeholder=None, view=None,
@@ -147,7 +147,7 @@ class Password(Field):
            not given, a password will be generated, made of at most p_maxLength
            chars. This method returns the generated password (or simply
            p_password if no generation occurred).'''
-        if newPassword is None:
+        if password is None:
             # Generate one
             password = self.generate(maxLength)
             msgPart = 'generated'
