@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,12 +21,6 @@ from appy.px import Px
 from appy.utils import iconParts
 from appy.model.utils import Object
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NO_CUR_PAGE = 'There is no current page to show. The default page for this ' \
-              'object is probably invisible. Please define method ' \
-              'getDefaultViewPage and / or getDefaultEditPage To overcome ' \
-              'the problem.'
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Page:
     '''Used for describing a page, its related phase, show condition, etc.'''
@@ -197,7 +191,7 @@ class UiPage:
             # The icon may not be available yet. For example, we may be creating
             # an object having a field that will be used to store the page icon,
             # but no image has been uploaded yet.
-            base = 'appy'
+            base = None
             icon = 'page.svg'
         else:
             base, icon = page.iconBase, page.icon
@@ -249,19 +243,12 @@ class UiPhase:
        <div var="viewUrl=o.getUrl(sub='view',page=pag.name,popup=popup,nav=nav)\
                   if pag.showOnView else None;
                  pictoUrl=pag.iconUrl">
-        <!-- The edit or lock icon -->
-        <x var="locked=o.Lock.isSet(o, user, pag.name);
-               editable=mayEdit and mayAct and pag.showOnEdit and pag.showEdit">
-         <a if="editable and not locked"
-            href=":o.getUrl(sub='edit', page=pag.name, popup=popup, nav=nav)">
-          <img src=":url('edit.svg')" class="pictoT shake"
-               title=":_('object_edit')"/></a>
-         <x var="page=pag.name; lockStyle='iconM pictoT'">::o.Lock.px</x>
-        </x>
 
         <!-- Main icon -->
-        <div><a if="viewUrl" href=":viewUrl">
-              <img src=":pictoUrl" class=":picto"/></a>
+        <div>
+         <a if="viewUrl" href=":viewUrl">
+          <img src=":pictoUrl" class=":picto"/>
+         </a>
          <img if="not viewUrl" src=":pictoUrl" class=":picto"/>
         </div>
 
@@ -270,6 +257,16 @@ class UiPhase:
          <a if="viewUrl" href=":viewUrl">::label</a>
          <x if="not viewUrl">::label</x>
         </x>
+
+        <!-- The edit or lock icon -->
+        <x var="locked=o.Lock.isSet(o, user, pag.name);
+               editable=mayEdit and mayAct and pag.showOnEdit and pag.showEdit">
+         <a if="editable and not locked"
+            href=":o.getUrl(sub='edit', page=pag.name, popup=popup, nav=nav)">
+          <img src=":url('editPage.svg')" class="pictoT shake"
+               title=":_('object_edit')"/></a>
+         <x var="page=pag.name; lockStyle='iconM pictoT'">::o.Lock.px</x>
+        </x>
        </div>
       </div>
 
@@ -279,37 +276,46 @@ class UiPhase:
 
      # Class names ending with "C" are used when controls are *C*ompact
      css='''
-      .phase { margin: 0 0 30px -30px; display: flex; flex-wrap: wrap;
-               width: fit-content }
-      .phase > div { height: 115px; width: 125px; text-align: center;
-        text-transform: uppercase; font-weight: bold; font-size: 90%;
-        display: flex; align-items: center; justify-content: center;
-        background-color: white; position: relative; padding: 0 5px;
-        border-right: 1px solid |linkColor|; border-top: 1px solid |linkColor|;
-        border-bottom: 1px solid |linkColor| }
+      .phase { margin:0 0 30px -30px; display:flex; flex-wrap:wrap;
+               width:fit-content }
+      .phase div, .phase form { display:|pageDisplay| }
 
-      .phaseC { margin: 0 0 20px 0; border-left: 1px solid |linkColor| }
-      .phaseC > div { height: 80px; width: 110px; font-size: 9pt }
+      .phase > div { height:|pageHeight|; width:|pageWidth|; text-align:center;
+        text-transform:uppercase; font-weight:bold; font-size:90%;
+        display:flex !important; align-items:center; justify-content:center;
+        padding:|pagePadding|; background-color:|phaseBgColor|;
+        position:relative; padding:0 5px; color:|phaseColor|;
+        border-right:1px solid |phaseBorderColor|;
+        border-top:1px solid |phaseBorderColor|;
+        border-bottom:1px solid |phaseBorderColor| }
+      
+      .phase a { color:|phaseColor| }
 
-      .picto { width: 55px; height: 55px; margin-bottom: 6px }
-      .pictoT { position: absolute; top: 8px; right: 8px; width: 22px }
-      .pictoC, .pictoB { width: 35px; height: 35px; margin-bottom: 4px }
-      .inverted { filter: invert(66%) }
+      .phaseC { margin:0 0 20px 0; border-left:1px solid |phaseBorderColor| }
+      .phaseC > div { height:|pageCHeight|; width:|pageCWidth|; font-size:9pt;
+                      margin:|pageMargin|; padding:|pageCPadding| }
 
-      .currentPage { background-color: |headerBgColor| !important;
-                     color: |headerColor| }
-      .currentPage img { filter: invert(66%) }
-      .currentPage a { color: |headerColor| }
+      .picto { width:|pictoSize|; height:|pictoSize|; margin:|pictoMargin| }
+      .pictoT { position:|epictoPosition|; top:8px; right:8px; width:22px;
+                padding: |epictoPadding| }
+      .pictoC, .pictoB { width:|pictoCSize|; height:|pictoCSize|;
+                         margin:|pictoCMargin| }
+      .inverted { filter:invert(66%) }
+
+      .currentPage { background-color:|phaseBgcColor| !important;
+                     color:|phaseCcolor| }
+      .currentPage img { filter:invert(66%) }
+      .currentPage a, .currentPage div { color:|brightColor| }
 
       // Animation for the "edit" icon
       @keyframes shakeImage {
-        0% { transform: rotate(0deg); filter: sepia(0) }
-        95% { transform: rotate(0deg); filter: sepia(0) }
-        97% { transform: rotate(10deg); filter: sepia(1) }
-        99% { transform: rotate(-10deg); filter: sepia(0.6) }
-        100% { transform: rotate(0deg); filter: sepia(0.3) }
+        0% { transform:rotate(0deg); filter:sepia(0) }
+        95% { transform:rotate(0deg); filter:sepia(0) }
+        97% { transform:rotate(10deg); filter:sepia(1) }
+        99% { transform:rotate(-10deg); filter:sepia(0.6) }
+        100% { transform:rotate(0deg); filter:sepia(0.3) }
       }
-      .shake { animation: shakeImage 10s infinite }''')
+      .shake { animation:shakeImage 10s infinite }''')
 
     def __init__(self, name, phases):
         # The name of the phase
@@ -496,13 +502,28 @@ class UiPhases:
     def finalize(self):
         '''Finalize phases, ie, by removing phases without any visible page'''
         # Remove phases without page
+        toDelete = None
         for name in self.all.keys():
             if not self.all[name].pages:
+                if toDelete is None: toDelete = []
+                toDelete.append(name)
+        if toDelete is not None:
+            for name in toDelete:
                 del(self.all[name])
-        # If p_self.currentPage is None, it will crash. Set, as current page,
-        # the first visible page, just to avoid a crash.
         if not self.currentPage:
-            raise Exception(NO_CUR_PAGE)
+            # We can't continue. Possibly, the page mentioned in the URL does
+            # not exist, or the user may not be allowed to consult it anymore.
+            # Typically, the user has logged out in browser tab A, and he tries
+            # to reload a page in tab B, that became invisible when consulted as
+            # anonymous user.
+            # ~~~
+            # Another explanation may be that there is no current page to show
+            # according to conditions defined on pages for self.o's class. While
+            # this can be a problem of conception in the app, there is a
+            # possible solution to overcome it: defining, on this class, methods
+            # m_getDefaultViewPage and / or m_getDefaultEditPage.
+            o = self.o
+            o.raiseMessage(o.translate('page_not_found'))
 
     def getStickyIndex(self, table):
         '''Returns the index of the element, within the page layout represented
