@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -30,29 +30,28 @@ from appy.model.utils import Object as O
 from appy.model.fields import Field, Show
 from appy.ui.layout import Layouts, Layout
 from appy.model.fields.file import FileInfo
-from appy.pod import PodError, styles_manager
+from appy.pod import PodError, styles_manager, formatsByTemplate
 
 # Messages - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 POD_ERROR = 'An error occurred while generating the document. Please contact ' \
-  'the system administrator.'
-NO_TEMPLATE = 'Please specify a pod template in field "template".'
-UNAUTHORIZED = 'You are not allowed to perform this action.'
-TEMPLATE_NOT_FOUND = 'Template not found at %s.'
-FREEZE_ERROR = 'Error while trying to freeze a "%s" file in pod field "%s" ' \
-  '(%s).'
-FREEZE_FATAL_ERROR = 'Server error. Please contact the administrator.'
-RENDERING = "rendering pod %s :: %s (in %s)..."
-RENDERED = "pod %s:%s (%s) rendered in %.2fsec."
-MAIL_NO_RECIPIENT = 'mailing "%s" contains no recipient.'
-MAIL_NO_USER = 'mailing "%s": inexistent user or no email for "%s".'
-MAIL_NO_RECIPIENT_FINALLY = 'mailing "%s" contains no recipient (after ' \
-  'removing wrong entries, see above).'
-OVERWRITING = '%s: overwriting %s...'
-FREEZING_ODT = 'freezing the ODT version...'
-FREEZING_OVERWRITING = 'freeze: overwriting %s...'
-FROZEN_OK = 'frozen at %s.'
-UNFROZEN_OK = 'removed (unfrozen) %s.'
-UPLOADED_OK = 'uploaded at %s.'
+            'the system administrator.'
+NO_TPL    = 'Please specify a pod template in field "template".'
+UNAUTH    = 'You are not allowed to perform this action.'
+TPL_INEX  = 'Template not found at %s.'
+FREEZ_ERR = 'Error while trying to freeze a "%s" file in pod field "%s" (%s).'
+FREEZ_FER = 'Server error. Please contact the administrator.'
+RENDERING = "Rendering pod %s :: %s (in %s)..."
+RENDERED  = "%s:%s (%s) rendered in %.2fsec."
+MAIL_NO_R = 'Mailing "%s" contains no recipient.'
+MAIL_NO_U = 'Mailing "%s": inexistent user or no email for "%s".'
+MAIL_NO_F = 'Mailing "%s" contains no recipient (after removing wrong ' \
+            'entries, see above).'
+OVERWRIT  = '%s: overwriting %s...'
+FREEZ_ODT = 'Freezing the ODT version...'
+FREEZ_OV  = 'Freeze: overwriting %s...'
+FROZEN_OK = 'Frozen at %s.'
+UNFROZ_OK = 'Removed (unfrozen) %s.'
+UPLOAD_OK = 'Uploaded at %s.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -168,8 +167,7 @@ class Pod(Field):
     defaultConfig = Config()
 
     # Default output formats for POD templates, by template type
-    allFormats = {'.odt': ('pdf', 'doc', 'docx', 'odt'),
-                  '.ods': ('xls', 'xlsx', 'ods')}
+    allFormats = formatsByTemplate
 
     # Getting a pod value is something special: disable the standard Appy
     # machinery for this.
@@ -179,9 +177,9 @@ class Pod(Field):
         '''Pod-specific layouts'''
         # Right-aligned layouts, convenient for pod fields exporting search
         # results or multi-template pod fields.
-        r = Layouts(view=Layout('fl!', css_class='podTable')) # "r"ight
+        r = Layouts(view=Layout('lf!', css_class='podTable')) # "r"ight
         rf= Layouts(view=Layout('f!', css_class='podTable')) # "r"ight, no label
-        l = Layouts(view= Layout('fl;', css_class='podTable')) # "l"eft
+        l = Layouts(view= Layout('lf;', css_class='podTable')) # "l"eft
         # "r"ight "m"ulti-template (where the global field label is not used)
         rm = Layouts(view=Layout('f!', css_class='podTable'))
         # Inline layout
@@ -195,7 +193,7 @@ class Pod(Field):
              linkGetter='getNode(%s)' % q(linkId) if confirm else 'this';
              js='generatePod(%s,%s,%s,%s,%s,%s,null,%s)' % \
                  (linkGetter, q(o.url), q(name), q(info.template), q(fmt), \
-                 q(field.getSearchParams(req, layout)), gc)"
+                 q(tool.Search.encodeForReplay(req, layout)), gc)"
         onclick=":confirm and 'askConfirm(%s,%s,%s)' % (q('script'), \
                  q(js,False), field.labelId) or js"
         title=":field.getIconTitle(o, fmt, frozen)" class="clickable"
@@ -213,6 +211,9 @@ class Pod(Field):
         var2="mailings=field.getVisibleMailings(o, info.template);
               lineBreak=((loop.info.nb + 1) % field.maxPerRow) == 0;
               confirm=field.getConfirm(o, info.template)">
+      <!-- Show the specific template name when relevant -->
+      <span if="not onCell and field.useTemplateName"
+            class="pod smaller">:field.getTemplateName(o, info.template)</span>
       <x for="fmt in info.formats"
          var2="freezeAllowed=(fmt in info.freezeFormats) and \
                              (field.show != 'result');
@@ -263,8 +264,9 @@ class Pod(Field):
                                                None, forUi=True)">
            <td colspan="2">
             <a var="js='generatePod(this,%s,%s,%s,%s,%s,null,%s,%s,comment)' % \
-                       (q(id), q(name), q(info.template), q(fmt), \
-                        q(field.getSearchParams(req, layout)), gc, q(mailing))"
+                        (q(id), q(name), q(info.template), q(fmt), \
+                         q(tool.Search.encodeForReplay(req, layout)), gc, \
+                         q(mailing))"
                onclick=":'askConfirm(%s,%s,null,true,400,%s,%s,6)' % \
                           (q('script'), q(js,False), \
                            q(mInfo.getConfirmLabel(_)), q(mInfo.body))"
@@ -277,10 +279,6 @@ class Pod(Field):
         </table>
        </span>
       </x>
-      <!-- Show the specific template name when relevant -->
-      <span if="not onCell and field.useTemplateName"
-        class=":'pod smaller' if not loop.info.last and not lineBreak else \
-            'podLast smaller'">:field.getTemplateName(o, info.template)</span>
       <br if="lineBreak"/>
      </x>''',
 
@@ -304,16 +302,8 @@ class Pod(Field):
         for (var key in data) node.setAttribute(key, data[key]);
       }
 
-      // Function that allows to generate a document from a pod template
-      generatePod = function(node, url, fieldName, template, podFormat,
-                       queryData, customParams, checkHook, mailing, mailText) {
-        var f = document.getElementById('podForm');
-        f.action = url + '/' + fieldName + '/generate';
-        f.template.value = template;
-        f.podFormat.value = podFormat;
-        f.queryData.value = queryData;
-        if (customParams) { f.customParams.value = customParams; }
-        else { f.customParams.value = ''; }
+      // Manage query data, if the POD to produce is tied to a search
+      manageQueryData = function(f, queryData, checkHook) {
         if (queryData) {
           /* If "queryData" specifies a custom search, get criteria from the
              browser's session storage. */
@@ -322,13 +312,36 @@ class Pod(Field):
             f.criteria.value = sessionStorage.getItem(elems[0]);
           }
         }
+        else if (checkHook) {
+          /* If this hook refers to a Search within a Ref field, get search
+             params and build query data from it. */
+          var ssel = document.getElementById(checkHook + '_ssel');
+          if (ssel && ssel.value) {
+            // Get the search params corresponding to this search
+            var p = document.getElementById('searchResults')['ajax'].params,
+                data=[p['className'], p['search'], p['sortKey'], p['sortOrder'],
+                      stringFromDict(p['filters'])].join(':');
+            f.queryData.value = data;
+          }
+        }
+      }
+
+      // Generate a document from a POD template
+      generatePod = function(node, url, fieldName, template, podFormat,
+                       queryData, customParams, checkHook, mailing, mailText) {
+        var f = document.getElementById('podForm');
+        f.action = url + '/' + fieldName + '/generate';
+        f.template.value = template;
+        f.podFormat.value = podFormat;
+        f.queryData.value = queryData;
+        f.customParams.value = customParams || '';
+        manageQueryData(f, queryData, checkHook);
         if (mailing) {
           f.mailing.value = mailing;
           if (mailText) f.mailText.value = mailText;
         }
         // Transmit value of cookie "showSubTitles"
         f.showSubTitles.value = readCookie('showSubTitles') || 'True';
-        f.action.value = 'generate';
         // Initialise the status of checkboxes
         setChecked(f, checkHook);
         // Disable the link to prevent double-clicks
@@ -386,7 +399,7 @@ class Pod(Field):
         # application. A pod template name Test.odt that is stored at the root
         # of your app will be referred as "Test.odt" in self.template. If it is
         # stored within sub-folder "pod", it will be referred as "pod/Test.odt".
-        if not template: raise Exception(NO_TEMPLATE)
+        if not template: raise Exception(NO_TPL)
         if isinstance(template, str):
             self.template = [template]
         elif isinstance(template, tuple):
@@ -675,22 +688,23 @@ class Pod(Field):
         # Compute the absolute path
         r = putils.resolvePath(os.path.join(baseFolder, template))
         if not os.path.isfile(r):
-            raise Exception(TEMPLATE_NOT_FOUND % template)
+            raise Exception(TPL_INEX % template)
         # Unwrap the path if the file is simply a pointer to another one
         elems = os.path.splitext(r)
         if elems[1] not in Pod.allFormats:
             r = elems[0]
         return r
 
-    def getDownloadName(self, o, template, format, queryRelated):
+    def getDownloadName(self, o, template, format=None, queryRelated=False,
+                        bypassMethod=False):
         '''Gets the name of the pod result as will be seen by the user that will
            download it. Ensure the returned name is not too long for the OS that
            will store the downloaded file with this name.'''
         # Use method self.downloadName if present and if it returns something
         # for p_template.
-        if self.downloadName:
+        if not bypassMethod and self.downloadName:
             name = self.downloadName(o, template)
-            if name: return '%s.%s' % (name, format)
+            if name: return '%s.%s' % (name, format) if format else name
         # Compute the default download name
         fileName = Normalize.fileName(self.getTemplateName(o, template))[:40]
         if not queryRelated:
@@ -698,7 +712,7 @@ class Pod(Field):
             # the object title.
             title = o.getShownValue('title')
             fileName = '%s-%s' % (Normalize.fileName(title)[:40], fileName)
-        return fileName + '.' + format
+        return '%s.%s' % (fileName, format) if format else fileName
 
     def getVisibleTemplates(self, o):
         '''Returns, among self.template, the template(s) that can be shown'''
@@ -793,7 +807,7 @@ class Pod(Field):
         '''Sends the emails for m_mailing'''
         info = self.getMailingInfo(o, template, mailing, mailText)
         if not info.logins:
-            o.log(MAIL_NO_RECIPIENT % mailing)
+            o.log(MAIL_NO_R % mailing)
             return 'action_ko'
         # Collect logins corresponding to inexistent users and recipients
         missing = []
@@ -810,9 +824,9 @@ class Pod(Field):
                 else:
                     recipients.append(recipient)
         if missing:
-            o.log(MAIL_NO_USER % (mailing, str(missing)))
+            o.log(MAIL_NO_U % (mailing, str(missing)))
         if not recipients:
-            o.log(MAIL_NO_RECIPIENT_FINALLY % mailing)
+            o.log(MAIL_NO_F % mailing)
             msg = 'action_ko'
         else:
             o.tool.sendMail(recipients, info.subject, info.body, [attachment])
@@ -821,7 +835,7 @@ class Pod(Field):
 
     def getValue(self, o, name=None, layout=None, template=None, format=None,
                  result=None, queryData=None, computeCustomContext=None,
-                 secure=True, executeAction=True):
+                 secure=True, executeAction=True, single=None):
         '''For a pod field, getting its value means computing a pod document or
            returning a frozen one.'''
         # A pod field differs from other field types because there can be
@@ -848,13 +862,13 @@ class Pod(Field):
         # Security check
         if secure and not queryData:
             if self.showTemplate and not self.showTemplate(o, template):
-                raise Exception(UNAUTHORIZED)
+                raise Exception(UNAUTH)
         # Return the possibly frozen document (not applicable for query-related
         # pods).
         if not queryData:
             frozen = self.isFrozen(o, template, format)
             if frozen:
-                fileName = self.getDownloadName(o, template, format, False)
+                fileName = self.getDownloadName(o, template, format)
                 return FileInfo(frozen, inDb=False, uploadName=fileName)
         # We must call pod to compute a pod document from "template"
         appPath = o.appPath
@@ -872,17 +886,17 @@ class Pod(Field):
         podContext = {'tool': tool, 'user': o.user, 'self': o, 'field': self,
                       'now': DateTime(), '_': o.translate, 'appPath': appPath,
                       'template': template, 'req': req, 'config': tool.config}
-        # If the pod document is related to a search, re-trigger it and put the
+        # If the pod document is related to a search, replay it and put the
         # result in the pod context.
         if queryData:
-            podContext['objects'] = self.getSearchResults(tool, queryData)
+            podContext['objects'] = Search.replay(tool, queryData)
             podContext['queryData'] = queryData.split(':')
             podContext['_ref'] = Search.getRefInfo(tool, nameOnly=False)
         # Add the field-specific context if present
         if specificContext: podContext.update(specificContext)
         # Add the custom context when required
         if computeCustomContext:
-            self.setCustomContext(podContext, o, req)
+            self.setCustomContext(podContext, o, req, queryData)
         # Variable "_checked" can be expected by a template but absent (ie,
         # when generating frozen documents).
         if '_checked' not in podContext: podContext['_checked'] = O()
@@ -981,7 +995,7 @@ class Pod(Field):
 
         # Security check
         if secure and (format not in self.getFreezeFormats(o, template)):
-            raise Exception(UNAUTHORIZED)
+            raise Exception(UNAUTH)
         # Compute the absolute path where to store the frozen document in the
         # database.
         folder = o.getFolder(create=True)
@@ -989,7 +1003,7 @@ class Pod(Field):
         result = folder / fileName
         if result.is_file():
             prefix = 'freeze (upload)' if upload else 'freeze'
-            o.log(OVERWRITING % (prefix, result))
+            o.log(OVERWRIT % (prefix, result))
             result.unlink()
         if not upload:
             # Generate the document
@@ -997,22 +1011,22 @@ class Pod(Field):
                                 result=str(result))
             if isinstance(doc, str):
                 # An error occurred, the document was not generated
-                o.log(FREEZE_ERROR % (format, self.name, doc), type='error')
+                o.log(FREEZ_ERR % (format, self.name, doc), type='error')
                 if not freezeOdtOnError or (format == 'odt'):
-                    raise Exception(FREEZE_FATAL_ERROR)
-                o.log(FREEZING_ODT)
+                    raise Exception(FREEZ_FER)
+                o.log(FREEZ_ODT)
                 # Freeze the ODT version of the document, which does not require
                 # to call LibreOffice: the risk of error is smaller.
                 fileName = self.getFreezeName(template, 'odt')
                 result = folder / fileName
                 if result.is_file():
-                    o.log(FREEZING_OVERWRITING % result)
+                    o.log(FREEZ_OV % result)
                 doc = self.getValue(o, template=template, format='odt',
                                     result=result)
                 if isinstance(doc, str):
-                    self.log(FREEZE_ERROR % ('odt', self.name, doc),
+                    self.log(FREEZ_ERR % ('odt', self.name, doc),
                              type='error')
-                    raise Exception(FREEZE_FATAL_ERROR)
+                    raise Exception(FREEZ_FER)
                 o.log(FROZEN_OK % result)
         else:
             # Store the uploaded file in the database
@@ -1021,7 +1035,7 @@ class Pod(Field):
             doc = FileInfo(result, inDb=False)
             doc.replicateFile(upload.value, f)
             f.close()
-            o.log(UPLOADED_OK % result)
+            o.log(UPLOAD_OK % result)
         return doc
 
     def unfreeze(self, o, template=None, format='pdf', secure=False):
@@ -1029,7 +1043,7 @@ class Pod(Field):
            in p_format.'''
         # Security check
         if secure and (format not in self.getFreezeFormats(o, template)):
-            raise Exception(UNAUTHORIZED)
+            raise Exception(UNAUTH)
         # Compute the absolute path to the frozen doc
         folder = o.getFolder()
         fileName = self.getFreezeName(template, format)
@@ -1037,7 +1051,7 @@ class Pod(Field):
         if frozen.is_file():
             frozen.unlink()
             putils.FolderDeleter.deleteEmpty(str(folder))
-            o.log(UNFROZEN_OK % str(frozen))
+            o.log(UNFROZ_OK % str(frozen))
 
     def getFreezeFormats(self, o, template=None):
         '''What are the formats into which the current user may freeze
@@ -1063,7 +1077,7 @@ class Pod(Field):
             r += ' (%s)' % r.translate('frozen')
         return r
 
-    def setCustomContext(self, context, o, req):
+    def setCustomContext(self, context, o, req, queryData):
         '''Before calling pod to compute a result, if specific elements must be
            added to the p_context, compute it here. This request-dependent
            method is not called when computing a pod field for freezing it into
@@ -1077,21 +1091,29 @@ class Pod(Field):
         gc = self.getChecked
         if not gc: return
         # Manage a Ref field
-        if gc != 'search':
-            # Get the IDs specified in the request
-            ids, unchecked = self.getCheckedInfo(req)
-            objects = []
-            tool = o.tool
-            for tied in getattr(o, gc):
-                if unchecked: condition = tied.iid not in ids
-                else:         condition = tied.iid in ids
-                if condition and tied.allows('read'):
-                    objects.append(tied)
+        if gc == 'search':
+            # Manage a search. All search results are in p_context['objects'].
+            Search.keepCheckedResults(req, context['objects'])
+        else:
+            # Manage a Ref field
+            if queryData:
+                # We are in the presence of results from one of the searches
+                # defined in Ref.searches. Search results are already in
+                # p_context['objects'].
+                Search.keepCheckedResults(req, context['objects'])
+                objects = context['objects']
+            else:
+                # The IDs of the tied objects are specified in the request
+                ids, unchecked = Search.getCheckedInfo(req)
+                objects = []
+                tool = o.tool
+                for tied in getattr(o, gc):
+                    if unchecked: condition = tied.iid not in ids
+                    else:         condition = tied.iid in ids
+                    if condition and tied.allows('read'):
+                        objects.append(tied)
             context['_checked'] = O()
             setattr(context['_checked'], gc, objects)
-        # Manage a search. All search results are in p_context['objects'].
-        else:
-            self.keepCheckedResults(req, context['objects'])
 
     def setRequestValue(self, o):
         '''For a Pod field there is not possibility to carry its value in the
