@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -26,19 +26,21 @@ from appy.database.operators import Operator
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 emptyList = []
-WRONG_RADIOS = 'You cannot render this field as radio buttons if it may ' \
-  'contain several values (max multiplicity is higher than 1).'
-WRONG_CHECKBOXES = 'You cannot render this field as checkboxes if it can ' \
-  'only contain a single value (max multiplicity is 1).'
-WRONG_INLINEDIT = 'It is currently not possible to inline-edit a multi-' \
-  'valued Select field.'
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+RADIOS_KO  = 'You cannot render this field as radio buttons if it may ' \
+             'contain several values (max multiplicity is higher than 1).'
+CBS_KO     = 'You cannot render this field as checkboxes if it can ' \
+             'only contain a single value (max multiplicity is 1).'
+IEDIT_KO   = 'It is currently not possible to inline-edit a multi-' \
+             'valued Select field.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Selection:
     '''If you want to have dynamically computed possible values for a Select
        field, use a Selection instance.'''
 
-    def __init__(self, method):
+    def __init__(self, method, single=None):
         # p_method must be a method that will be called every time Appy will
         # need to get the list of possible values for the related field.
         #  - p_method can be the method object in itself, or the method name.
@@ -46,6 +48,7 @@ class Selection:
         #    defining the related field; it can also refer to a method defined
         #    on the tool. In this latter case, you are forced to define a method
         #    name (not a method object), prefixed with "tool:".
+
         # m_method, in most cases, accepts no arg and return a list (or tuple)
         # of pairs (lists or tuples): (id, text), where:
         #   - "id" is one of the possible values for the field;
@@ -60,12 +63,23 @@ class Selection:
         # (m_translate) to produce an i18n version of "text".
         self.method = method
 
+        # While p_method is called on edit and filter layouts to list the
+        # possible values to select, the method you specify in p_single allows
+        # to produce the translated text for a single value. It accepts a single
+        # arg, being one of the values as stored by the select field, and must
+        # return its translated text. If you don't specify a p_single method,
+        # p_method will be used instead, but useless processing will take place,
+        # because p_method produces texts for the complete list of possible
+        # values. Moreover, a stored select value may not be among the list of
+        # values as returned by p_method anymore.
+        self.single = single
+
     def getText(self, o, value, field, language=None):
         '''Gets the text that corresponds to p_value'''
-        if language:
-            withTranslations = language
-        else:
-            withTranslations = True
+        # Use method p_single if specified
+        if self.single: return self.single(o, value)
+        # Use p_self.method
+        withTranslations = language if language else True
         vals = field.getPossibleValues(o, ignoreMasterValues=True,
                                        withTranslations=withTranslations)
         for v, text in vals:
@@ -156,11 +170,12 @@ class Select(Field):
               title=":text">:Px.truncateValue(text, charsWidth)</option>
      </select>
      <x if="not isSelect">
-      <div for="val, text in possibleValues">
-       <input type=":field.render" name=":name" id=":val" value=":val"
+      <div for="val, text in possibleValues"
+           var2="vid='%s_%s' % (name, val)">
+       <input type=":field.render" name=":name" id=":vid" value=":val"
               class=":masterCss" onchange=":field.getOnChange(o, layout)"
               checked=":field.isSelected(o, name, val, rawValue)"/>
-       <label lfor=":val" class="subLabel">:text</label>
+       <label lfor=":vid" class="subLabel">:text</label>
       </div>
      </x>
      <script if="hostLayout" var2="x=o.Lock.set(o, user, field=field)">:\
@@ -204,8 +219,8 @@ class Select(Field):
       <option for="val, text in field.getPossibleValues(tool, \
                 withTranslations=True, withBlankValue='forced', \
                 blankLabel='everything', className=className)"
-       selected=":(name in mode.filters) and (mode.filters[name] == val)"
-       value=":val" title=":text">:Px.truncateValue(text, charsWidth)</option>
+              selected=":mode.inFilter(name, val)" value=":val"
+              title=":text">:Px.truncateValue(text, charsWidth)</option>
      </select>''')
 
     def __init__(self, validator=None, multiplicity=(0,1), default=None,
@@ -272,12 +287,12 @@ class Select(Field):
         # Valid render modes depend on multiplicities
         multiple = self.isMultiValued()
         if multiple and (self.render == 'radio'):
-            raise Exception(WRONG_RADIOS)
+            raise Exception(RADIOS_KO)
         if not multiple and (self.render == 'checkbox'):
-            raise Exception(WRONG_CHECKBOXES)
+            raise Exception(CBS_KO)
         # It is currently not possible to inline-edit a multi-valued field
         if multiple and self.inlineEdit:
-            raise Exception(WRONG_INLINEDIT)
+            raise Exception(IEDIT_KO)
 
     def isSelected(self, o, name, possibleValue, dbValue):
         '''When displaying a Select field, must the p_possibleValue appear as
@@ -294,8 +309,8 @@ class Select(Field):
             return possibleValue in compValue
         return possibleValue == compValue
 
-    def getValue(self, o, name=None, layout=None, single=False):
-        value = Field.getValue(self, o, name, layout)
+    def getValue(self, o, name=None, layout=None, single=None):
+        value = Field.getValue(self, o, name, layout, single)
         if not value:
             return emptyList if self.isMultiValued() else value
         if isinstance(value, str) and self.isMultiValued():
@@ -304,27 +319,37 @@ class Select(Field):
             value = list(value)
         return value
 
+    def getTranslatedValue(self, o, value, language=None):
+        '''Get the translated text for p_value, when p_value is one of p_self's
+           authorized "static" values according to its p_self.validator (being a
+           list or tuple o values).'''
+        if self.translations:
+            # Translations are available via a FieldTranslations instance
+            language = language or o.guard.userLanguage
+            r = self.translations.get('value', language, value=value)
+        else:
+            r = o.translate('%s_list_%s' % (self.labelId, value))
+        return r
+
     def getFormattedValue(self, o, value, layout='view', showChanges=False,
                           language=None):
         '''Select-specific value formatting'''
         # Return an empty string if there is no p_value
         if Field.isEmptyValue(self, o, value) and not showChanges: return ''
-        r = value
         if isinstance(self.validator, Selection):
             # Value(s) come from a dynamic vocabulary
             val = self.validator
             if self.isMultiValued():
-                return [val.getText(o, v, self, language) for v in value]
+                r = [val.getText(o, v, self, language) for v in value]
             else:
-                return val.getText(o, value, self, language)
+                r = val.getText(o, value, self, language)
         else:
             # Values come from a fixed vocabulary whose texts are in i18n files
-            _ = o.translate
+            _ = self.getTranslatedValue
             if self.isMultiValued():
-                r = [_('%s_list_%s' % (self.labelId, v), language=language) \
-                     for v in value]
+                r = [_(o, v, language=language) for v in value]
             else:
-                r = _('%s_list_%s' % (self.labelId, value), language=language)
+                r = _(o, value, language=language)
         return r
 
     def getSearchValue(self, req, value=None):
@@ -378,7 +403,10 @@ class Select(Field):
             else:
                 res = []
                 for v in values:
-                    res.append((v, self.getFormattedValue(o,v,language=lg)))
+                    # Translations may already have been included
+                    if not isinstance(v ,tuple):
+                        v = (v, self.getFormattedValue(o,v,language=lg))
+                    res.append(v)
         else:
             # Get the possible values from attribute "[s]validator"
             validator = self.svalidator if className else self.validator
@@ -387,13 +415,12 @@ class Select(Field):
                 if not withTranslations: res = [v[0] for v in res]
                 elif isinstance(res, list): res = res[:]
             else:
-                # The list of (static) values is directly given in
-                # self.validator.
+                # The list of (static) values is in self.validator
                 res = []
                 for value in validator:
-                    label = '%s_list_%s' % (self.labelId, value)
                     if withTranslations:
-                        res.append( (value, o.translate(label, language=lg)) )
+                        text = self.getTranslatedValue(o, value, language=lg)
+                        res.append((value, text))
                     else:
                         res.append(value)
         if (withBlankValue == 'forced') or \
@@ -414,6 +441,10 @@ class Select(Field):
     def validateValue(self, o, value):
         '''Ensure p_value is among possible values'''
         possibleValues = self.getPossibleValues(o, ignoreMasterValues=True)
+        # In the context of a slave whose possible values depend on the selected
+        # master, while the object is temp, possible values could not have been
+        # computed yet.
+        if not possibleValues: return
         if isinstance(value, str):
             error = value not in possibleValues
         else:
