@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -24,6 +24,7 @@ from appy.px import Px
 from appy.xml import Parser
 from appy.utils import flipDict
 from appy.ui.layout import Layouts
+from appy.xml.escape import Escape
 from appy.model.fields import Field
 from appy.utils import string as sutils
 from appy.ui.layout import Layouts, Layout
@@ -748,10 +749,10 @@ class Text(Multilingual, Field):
         if Field.isEmptyValue(self, o, value) and not showChanges: return ''
         r = value
         if layout in ('view', 'cell'):
-            r = sutils.formatText(r, format='html_from_text')
+            r = Escape.xhtml(r, p=True)
         # If value starts with a carriage return, add a space; else, it will
         # be ignored.
-        if isinstance(r, str) and \
+        elif isinstance(r, str) and \
            (r.startswith('\n') or r.startswith('\r\n')): r = ' ' + r
         return r
 
@@ -765,6 +766,12 @@ class Text(Multilingual, Field):
         if max and (len(value) > max): value = value[:max]
         return value
 
+    def validateUniValue(self, o, value): return
+
+    def getFilterValue(self, value):
+        '''Ensure the value is suffixed with a star'''
+        return value if value.endswith('*') else '%s*' % value
+
     @classmethod
     def computeSearchValue(class_, field, req, value=None):
         '''Converts text encoded in a search form into a range search or into
