@@ -1,7 +1,7 @@
 '''Meta-class for a Appy class'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -25,6 +25,7 @@ from persistent.mapping import PersistentMapping
 
 from appy.px import Px
 from appy.model import Model
+
 import appy.model.meta.class_
 from appy.model.meta import Meta
 from appy.model.base import Base
@@ -32,9 +33,11 @@ from appy.model.page import Page
 from appy.model.user import User
 from appy.model.tool import Tool
 from appy.model.group import Group
+from appy.ui.layout import Layouts
 from appy.model.fields import Field
 from appy.model.workflow import Role
 from appy.model.searches import Search
+from appy.utils import string as sutils
 from appy.model.document import Document
 from appy.model.utils import Object as O
 from appy.model.fields.phase import Phase
@@ -365,6 +368,13 @@ class Class(Meta):
             r = o.getShownValue('title')
         return r
 
+    def getPageLayouts(self, o):
+        '''Gets the page layouts being applicable to p_o, being an instance of
+           p_self.'''
+        r = getattr(self.python, 'layouts', None)
+        if callable(r): r = r(o)
+        return r or Layouts.Page.defaults
+
     def produceUniqueLinks(self):
         '''When producing links ("a" tags) to instances of this class, must
            these links always be unique ?'''
@@ -481,6 +491,18 @@ class Class(Meta):
             for search in r: search.init(self)
         return r
 
+    def getSearch(self, name, tool=None):
+        '''Returns static or dynamic search whose name is p_name'''
+        # Search among static searches
+        r = self.searches.get(name)
+        if r or (tool is None): return r
+        # Search among dynamic searches
+        dyn = self.getDynamicSearches(tool)
+        if dyn:
+            for search in dyn:
+                if search.name == name:
+                    return search
+
     def getGroupedSearches(self, tool, ctx):
         '''Returns an object with 2 attributes:
            * "searches" stores the searches defined for this class, as instances
@@ -589,6 +611,23 @@ class Class(Meta):
         r = getattr(self.python, 'listColumns', ('title',))
         return r if not callable(r) else r(tool)
 
+    def getFilters(self, tool, filters=None):
+        '''Extracts, from the request (or from p_filters if passed), filters
+           defined on fields belonging to this class.'''
+        r = sutils.getDictFrom(filters or tool.req.filters)
+        # Apply a potential transform on every filter value
+        for name in list(r.keys()):
+            # Get the corresponding field
+            field = self.fields.get(name)
+            if field:
+                try:
+                    value = field.getStorableValue(None, r[name])
+                    r[name] = field.getFilterValue(value)
+                except Exception:
+                    # The encoded value is invalid. Ignore it.
+                    del(r[name])
+        return r
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -603,11 +642,25 @@ class Class(Meta):
          onclick="openPopup('iframePopup')" style=":url('add.svg', bg=True)"/>
      </a>''')
 
+    # Style characteristics to apply to the "add" button, depending on its type
+    addStyles = {
+      # The "add" button as it appears in the portlet
+      'portlet': O(css='buttonPortlet button', bgIcon='portletAdd.svg',
+                   bgSize='24px 24px'),
+      # The "add" button as it may appear elsewhere (ie, on search results)
+      'small'  : O(css='buttonSmall button', bgIcon='add.svg',
+                   bgSize='18px 18px'),
+      # When using pxAdd from a custom PX, let this latter manage button style
+      None     : O(css='', bgIcon=None, bgSize=None)
+    }
+
     # Form for creating instances of this class from the portlet or a search
     pxAdd = Px('''
-     <form var="create=class_.getCreateVia(tool)"
+     <form var="create=class_.getCreateVia(tool); className=class_.name"
            if="create" class="addForm" name=":'%s_add' % className"
-           var2="target=ui.LinkTarget(class_.python); label=_('object_add');"
+           var2="styles=class_.addStyles[buttonType];
+                 target=ui.LinkTarget(class_.python, popup=viaPopup);
+                 text=_(label or 'object_add')"
            action=":'%s/new' % tool.url" target=":target.target">
       <input type="hidden" name="className" value=":className"/>
       <input type="hidden" name="template" value=""/>
@@ -615,12 +668,12 @@ class Class(Meta):
       <input type="hidden" name="nav" value=":nav"/>
       <input type="hidden" name="popup"
            value=":'True' if (popup or (target.target!='_self')) else 'False'"/>
+
       <!-- Create from an empty form -->
-      <input var="suffix='Small' if asButton else 'Portlet';
-                  size='18px 18px' if asButton else '24px 24px'"
-             class=":'button%s button' % suffix" style=":url('add.svg',bg=size)"
-             value=":label" title=":label" type="submit"
-             onclick=":target.getOnClick('searchResults')"/>
+      <input class=":styles.css" value=":text" title=":text" type="submit"
+        style=":url(styles.bgIcon, bg=styles.bgSize) if styles.bgIcon else ''"
+        onclick=":target.getOnClick('searchResults')"/>
+
       <!-- Create from a pre-filled form when relevant -->
       <div if="create != 'form'" class="addFrom"
          var2="fromRef=False; sourceField=None;
