@@ -2,7 +2,7 @@
    searches. A searched is always attached to a given class from the model.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -26,12 +26,15 @@ from appy.model.batch import Batch
 from appy.ui.criteria import Criteria
 from appy.ui.template import Template
 from appy.utils import string as sutils
+from appy.model.utils import Object as O
 from appy.model.fields.group import Group
 from appy.model.searches.modes import Mode
 from appy.model.searches import initiators
 
 # Error messages - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 WRONG_FIELD = 'Field "%s" does not exist on %s.'
+REPLAY_S    = 'Class "%s": replaying search "%s"...'
+REPLAY_OK   = 'Fetched %d object(s).'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class UiSearch:
@@ -54,6 +57,7 @@ class UiSearch:
      <x var="layout='view';
              batch=mode.batch;
              empty=mode.empty;
+             search=uiSearch.search;
              showNewSearch=showNewSearch|True;
              showHeaders=showHeaders|True;
              specific=uiSearch.getResultsTop(mode, ajax)">
@@ -66,7 +70,7 @@ class UiSearch:
       <script>::mode.getAjaxData()</script>
 
       <!-- Pod templates -->
-      <x if="not empty and not popup">
+      <x if="not empty and not popup and search.showPods">
        <table var="fields=class_.getListPods()"
               if="fields and mode.objects" align=":dright">
         <tr>
@@ -78,31 +82,34 @@ class UiSearch:
       </x>
 
       <!-- Title -->
-      <div if="not popup" class="pageTitle topSpaceS">
+      <div if="not popup and search.showTitle" class="pageTitle topSpaceS">
        <x>::uiSearch.translated</x>
        <x if="mode.batch and not empty">
         <img src=":url('arrowsA.svg')" class="iconS"
              style="transform: rotate(270deg)"/>
         <x>:mode.batch.total</x>
        </x>
-       <x if="uiSearch.search.add and guard.mayInstantiate(class_)"
-          var2="asButton=True;
+       <x if="search.add and guard.mayInstantiate(class_)"
+          var2="buttonType='small'; viaPopup=search.viaPopup; label=None;
                 nav=mode.getNavInfo(batch.total)">:class_.pxAdd</x>
       </div>
       <table width="100%">
        <tr valign="top">
         <!-- Search description -->
         <td>
-         <div class="discreet"
-              if="uiSearch.translatedDescr">:uiSearch.translatedDescr</div>
-         <!-- Perform a new search -->
-         <a if="showNewSearch and mode.newSearchUrl"
-            href=":mode.newSearchUrl">:_('search_new')</a>
-         <x if="mode.fromRef">&nbsp;&mdash;&nbsp; 
-          <a href=":mode.getRefUrl()">:_('goto_source')</a></x>
+         <x if="search.showTitle">
+          <div class="discreet"
+               if="uiSearch.translatedDescr">:uiSearch.translatedDescr</div>
+          <!-- Perform a new search -->
+          <a if="showNewSearch and mode.newSearchUrl"
+             href=":mode.newSearchUrl">:_('search_new')</a>
+          <x if="mode.fromRef and not mode.inField">&nbsp;&mdash;&nbsp; 
+           <a href=":mode.getRefUrl()">:_('goto_source')</a></x>
+         </x>
         </td>
         <!-- (Top) navigation -->
-        <td if="mode.batch" align=":dright">:mode.batch.pxNavigate</td>
+        <td if="mode.batch and uiSearch.showTopNav"
+            align=":search.navAlign">:mode.batch.pxNavigate</td>
        </tr>
       </table>
 
@@ -110,7 +117,8 @@ class UiSearch:
       <x if="not empty" var2="currentNumber=0">:mode.px</x>
 
       <!-- (Bottom) navigation -->
-      <div if="mode.batch" align=":dright">:mode.batch.pxNavigate</div>
+      <div if="mode.batch and uiSearch.showBottomNav"
+           align=":search.navAlign">:mode.batch.pxNavigate</div>
 
       <!-- No result -->
       <x if="empty">
@@ -134,6 +142,8 @@ class UiSearch:
         self.colspan = search.colspan
         self.showActions = search.showActions
         self.actionsDisplay = search.actionsDisplay
+        self.showTopNav = search.showNav in ('top', 'both')
+        self.showBottomNav = search.showNav in ('bottom', 'both')
         className = self.container.name
         if search.translated:
             self.translated = search.translated
@@ -199,11 +209,16 @@ class UiSearch:
 
     def getTitleMode(self, popup):
         '''How titles to search results, being instances of p_class_, must be
-           rendered ? r_ is:
-           * "link" : as links allowing to go to instances' view pages;
-           * "select": as objects that can be selected from a popup;
-           * "text": as simple, unclickable text.
-        '''
+           rendered ?'''
+        # Possible r_return values are the following.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # If...    | Every search result's title is rendered as ...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # "link"   | links allowing to go to instances' view pages;
+        # "dlink"  | links, as for "link", but the link is wrapped in a "div";
+        # "select" | objects, that can be selected from a popup;
+        # "text"   | simple, unclickable text.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if popup: return 'select'
         # Check if the title mode is specified on the container class
         mode = self.container.getTitleMode()
@@ -222,13 +237,6 @@ class UiSearch:
         '''Highlight search results within p_text'''
         return Criteria.highlight(self.tool.H(), text)
 
-    def parseFilters(self, req):
-        '''Parses the filters from the p_req(uest) or return None if not
-           found.'''
-        r = req.filters
-        if r is None: return {}
-        return sutils.getDictFrom(r)
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Search:
     '''Used for specifying a search for a given class'''
@@ -238,12 +246,17 @@ class Search:
     # has nothing to do with popup-oriented initiator as defined hereafter.
     initiator = initiators.SearchInitiator
 
+    # Parameters needed to replay a search (see m_replay)
+    replayParams = ('className', 'search', 'sortKey', 'sortOrder', 'filters')
+
     def __init__(self, name=None, group=None, sortBy='title', sortOrder='asc',
-                 maxPerPage=30, default=False, colspan=1, translated=None,
+                 maxPerPage=30, default=False, colspan=1, viaPopup=None,
                  show=True, showActions='all', actionsDisplay='block',
+                 showPods=True, showTitle=True, showFilters=True,
+                 showNav='both', navAlign='right', translated=None,
                  translatedDescr=None, checkboxes=False, checkboxesDefault=True,
                  container=None, add=False, resultModes=None, shownInfo=None,
-                 actions=None, rowAlign='top', **fields):
+                 actions=None, rowAlign='top', pageLayoutOnView=None, **fields):
         # "name" is mandatory, excepted in some special cases (ie, when used as
         # "select" param for a Ref field).
         self.name = name
@@ -256,6 +269,9 @@ class Search:
         # on main link.
         self.default = default
         self.colspan = colspan
+        # Attribute "viaPopup" has the same semantics as its homonym attribute
+        # on class appy.model.fields.Ref.
+        self.viaPopup = viaPopup
         # If a translated name or description is already given here, we will
         # use it instead of trying to translate from labels.
         self.translated = translated
@@ -266,6 +282,12 @@ class Search:
         # homonyms on the Ref class.
         self.showActions = showActions
         self.actionsDisplay = actionsDisplay
+        # Various parts of search results may be shown or not
+        self.showPods = showPods
+        self.showTitle = showTitle # The title, description, results count...
+        self.showFilters = showFilters # Filters, in grid mode
+        self.showNav = showNav # May be 'top', 'bottom', 'both' or None
+        self.navAlign = navAlign # Can be 'left', 'center' or 'right'
         # In dict "fields", keys are names of indexed fields and values are
         # simple search values or terms built with database operators.
         self.fields = fields
@@ -293,6 +315,9 @@ class Search:
         # When search resulrs are rendered as a list, this attribute specifies
         # each row's vertical alignment.
         self.rowAlign = rowAlign
+        # You may override the page layout used when accessing, on the "view"
+        # layout, every search result, by using the following attribute.
+        self.pageLayoutOnView = pageLayoutOnView
 
     def init(self, class_):
         '''Lazy search initialisation'''
@@ -390,9 +415,14 @@ class Search:
                 value = field.getSearchValue(None, value=value)
                 if value: params[name] = value
         # Manage a potential Ref field
-        if refObject:
-            back = refField.back
-            if back: params['cid'] = '%d_%s' % (refObject.iid, back.name)
+        if refObject and refField.back:
+            backName = refField.back.name
+            if refField.composite:
+                # Use the "cid" index (object containment)
+                params['cid'] = '%d_%s' % (refObject.iid, backName)
+            else:
+                # We suppose the Ref field is indexed
+                params[backName] = refObject.iid
         # Perform the search in the corresponding catalog, getting IIDs of
         # matched objects.
         database = handler.server.database
@@ -470,21 +500,21 @@ class Search:
             o = tool.getObject(id)
             field = o.getField(fieldName)
             if field.type == 'Ref':
-                initiator = initiators.RefInitiator(o, field, fieldName, mode)
-                r = field.getSelect(o)
+                if mode.startswith('search*'):
+                    # A search from Ref attribute "searches"
+                    r = field.getSearch(o, mode[7:])
+                else:
+                    initiator = initiators.RefInitiator(o, field,
+                                                        fieldName, mode)
+                    r = field.getSelect(o)
             elif field.type == 'Computed':
                 r = field.getSearch(o)
+        elif name.isdigit():
+            # The name of the search refers to a Query instance
+            r = tool.getObject(name).getSearch()
         else:
-            # Search among static searches
-            r = modelClass.searches.get(name)
-            if not r:
-                # Search among dynamic searches
-                dyn = modelClass.getDynamicSearches(tool)
-                if dyn:
-                    for search in dyn:
-                        if search.name == name:
-                            r = search
-                            break
+            # Search among static or dynamic searches
+            r = modelClass.getSearch(name, tool)
         # The search may not exist
         if not r: tool.raiseUnauthorized(tool.translate('search_broken'))
         # Return a UiSearch if required
@@ -500,14 +530,105 @@ class Search:
         # 2. A search is executed in the context of a Ref field with
         #    link="dropdown".
         if not info:
-            criteria = Criteria.readFromRequest(tool.H())
-            if criteria and ('_ref' in criteria): info = criteria['_ref']
+            # Try first to get the info from key "ref" in the request
+            info = tool.req.ref
+            if not info:
+                # Try to get it from the "_ref" key within search criteria
+                criteria = Criteria.readFromRequest(tool.H())
+                if criteria and ('_ref' in criteria): info = criteria['_ref']
         if not info or (sep not in info): return None, None
         id, field = info.split(sep)
         o = tool.getObject(id)
         field = field if nameOnly else o.getField(field)
         return o, field
 
+    @classmethod
+    def encodeForReplay(class_, req, layout):
+        '''Encodes in a string all the params in the p_req(uest) being required
+           for re-playing a search.'''
+        # If key "search" is present in the request, but empty, it represents
+        # the special search named "allSearch".
+        if ('search' in req) and not req.search:
+            req.search = 'allSearch'
+        if not req.search or (layout == 'cell'): return ''
+        return ':'.join([(req[key] or '') for key in class_.replayParams])
+
+    @classmethod
+    def replay(class_, tool, searchParams):
+        '''Replays the search whose parameters are in p_searchParams'''
+
+        # In some contexts, a Search needs to be "replayed". For example,
+        # imagine you run a search from the UI. While search results are shown
+        # in the browser, a POD field is also available, to get results in a PDF
+        # file. When the user asks to generate this file, the current search
+        # must be replayed in order to deliver the resulting objects to the POD
+        # field (in its context).
+        #
+        # Search parameters (p_searchParams) are in the format produced by
+        # m_encodeForReplay. This method return resulting objects. The operation
+        # is logged.
+        # ~~~
+        # Get search parameters as an object
+        params = O()
+        i = 0
+        for part in searchParams.split(':', len(class_.replayParams) - 1):
+            setattr(params, class_.replayParams[i], part)
+            i += 1
+        # Retrieve a Ref object when relevant
+        if 'search*' in params.search:
+            # Ref info may not be in the request, but may de deduced from the
+            # search name.
+            parts = params.search.split(',')
+            info = '%s:%s' % (parts[0], parts[1])
+        else:
+            info = None
+        refObject, refField = class_.getRefInfo(tool, info=info, nameOnly=False)
+        # Get the Search instance
+        modelClass = tool.model.classes.get(params.className)
+        search = class_.get(params.search, tool, modelClass, None, ui=False)
+        # Executing the search may take some time and potentially slow down
+        # the system: log this action.
+        tool.log(REPLAY_S % (modelClass.name, params.search))
+        # Replay the search, but without restricting the number of results
+        r = search.run(tool.H(), batch=False,
+          sortBy=params.sortKey, sortOrder=params.sortOrder,
+          filters=modelClass.getFilters(tool, params.filters),
+          refObject=refObject, refField=refField)
+        tool.log(REPLAY_OK % len(r))
+        return r
+
+    @classmethod
+    def getCheckedInfo(class_, req):
+        '''Gets the status of checked elements from the p_req(uest)'''
+
+        # Those elements do not necessarily refer to search results: they could
+        # also denote referred objects.
+        # 
+        # The method returns a tuple (ids, unchecked):
+        # ----------------------------------------------------------------------
+        #   "ids"     | is a dict of object IIDs;
+        # "unchecked" | is True if these objects are unchecked, or False if
+        #             | they are checked.
+        # ----------------------------------------------------------------------
+
+        ids = {int(v) for v in req.checkedIds.split(',')} if req.checkedIds \
+                                                          else {}
+        return ids, req.checkedSem == 'unchecked'
+
+    @classmethod
+    def keepCheckedResults(class_, req, objects):
+        '''Among p_objects as retrieved via m_replay, keep only those being
+           checked, according to m_getCheckedInfo.'''
+        ids, unchecked = class_.getCheckedInfo(req)
+        i = len(objects) - 1
+        # Remove, from search results, unchecked objects
+        while i >= 0:
+            if unchecked: remove = objects[i].iid in ids
+            else:         remove = objects[i].iid not in ids
+            if remove:
+                del objects[i]
+            i -= 1
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  PXs
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -516,7 +637,7 @@ class Search:
     results = Px('''
      <div var="hook=uiSearch.getRootHook()" id=":hook">
       <script>:uiSearch.getCbJsInit(hook)</script>
-      <x>::ui.Globals.getForms(tool)</x>
+      <x if="not mode.inField">::ui.Globals.getForms(tool)</x>
       <div align=":dright" if="len(resultModes) &gt; 1">
        <select name="px"
                onchange=":'switchResultMode(this, %s)' % q('searchResults')">
@@ -569,15 +690,16 @@ class Search:
     # Display results of a live search
     liveResults = Px('''
      <div var="search=uiSearch.search;
-               filters=uiSearch.parseFilters(req);
+               filters=search.container.getFilters(tool);
                objects=search.run(handler,filters=filters).objects;
                io,ifield=search.getRefInfo(tool, pre, nameOnly=False, sep='_')"
           id=":'%s_LSResults' % pre">
       <p if="not objects">::_('query_no_result')</p>
       <div for="o in objects" style="padding: 3px 5px">
        <a href=":search.onSelectLiveResult(o, io, ifield)"
-          var="content=Px.truncateValue(o.title, width=80)"
-          title=":o.title">:content</a>
+          var="title=o.getShownValue();
+               content=Px.truncateValue(title, width=80)"
+          title=":title">:content</a>
       </div>
       <!-- Go to the page showing all results -->
       <div if="objects and not io" align=":dright" style="padding: 3px">
@@ -672,7 +794,11 @@ class Search:
 
      css='''.lsSelected { background-color: #d9d7d9 }
             .lsRef { display: flex }
-            .lsSearch { display: flex; margin-top: 2px }
+            .lsSearch { display: flex; margin-top: 5px; padding: |lsPadding|;
+                        background-color: |lsBgColor| }
+            .lsSearch input[type=text] { background-color: transparent;
+                        border-bottom: |lsBottom| |lsBottomColor| }
+            .lsSearch .dropdown { background-color: |lsrBgColor| }
             .lsAll { font-size: 80%; font-style: italic }''',
 
      # Live-search-related functions (LS)
