@@ -2,7 +2,7 @@
    calendar, etc.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -24,6 +24,7 @@ from appy.model.batch import Batch
 from appy.utils import string as sutils
 from appy.model.utils import Object as O
 from appy.ui import LinkTarget, Columns, Title
+from appy.model.searches.gridder import Gridder
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Mode:
@@ -87,22 +88,44 @@ class Mode:
         # A Batch instance, when only a sub-set of the result set is shown at
         # once.
         self.batch = None
-        # Are we sure the result is empty ? (ie, objects could be empty but
-        # matched objects could be absent due to filters).
+        # Determine result's "emptiness". If a search produces results without
+        # any filter, it is considered not being empty. Consequently, a search
+        # that would produce results without filters, but for which there is no
+        # result, due to current filters, is not considered being empty.
         self.empty = True
         # URL for triggering a new search
         self.newSearchUrl = None
         # Is the search triggered from a Ref field ?
         self.fromRef = False
+        # Is the search integrated into another field ?
+        self.inField = False
         # The target for "a" tags
-        self.target = LinkTarget(self.class_)
+        self.target = LinkTarget(self.class_, popup=uiSearch.search.viaPopup)
         # How to render links to result objects ?
         self.titleMode = uiSearch.getTitleMode(self.popup)
 
     def init(self):
-        '''Every concrete class may have a specific initialization part'''
+        '''Lazy mode initialisation. Can be completed by sub-classes.'''
         # Store criteria for custom searches
-        self.criteria = self.tool.req.criteria
+        tool = self.tool
+        req = tool.req
+        self.criteria = req.criteria
+        ui = self.uiSearch
+        search = ui.search
+        # The search may be triggered via a Ref field
+        io, ifield = search.getRefInfo(tool, nameOnly=False)
+        if io:
+            self.refObject = io
+            self.refField = ifield
+        else:
+            self.refObject, self.refField = None, None
+        # Build the URL allowing to trigger a new search
+        if search.name == 'customSearch':
+            part = '&ref=%s:%s' % (io.iid, ifield.name) if io else ''
+            self.newSearchUrl = '%s/Search/advanced?className=%s%s' % \
+                                (tool.url, search.container.name, part)
+        self.fromRef = bool(ifield)
+        self.inField = ifield and ('search*' in req.search)
 
     def getAjaxData(self):
         '''Initializes an AjaxData object on the DOM node corresponding to the
@@ -118,6 +141,10 @@ class Mode:
         # Add custom search criteria
         if self.criteria:
             params['criteria'] = self.criteria
+        # Add the "ref" key if present
+        ref = self.tool.req.ref
+        if ref:
+            params['ref'] = ref
         # Concrete classes may add more parameters
         self.updateAjaxParameters(params)
         # Convert params into a JS dict
@@ -166,12 +193,12 @@ class List(Mode):
            <img if="(mode.sortKey != field.name) or (mode.sortOrder == 'asc')"
                 onclick=":'askBunchSorted(%s, %s, %s)' % \
                           (q(mode.hook), q(field.name), q('desc'))"
-                src=":url('desc.svg')" class="clickable iconS"/>
+                src=":url('asc.svg')" class="clickable iconS"
+                style="transform:rotate(180deg)"/>
           </x>
           <!-- Filter widget -->
           <x var="field,filterPx=field.getFilterPx()"
-             if="filterPx and ((mode.batch.total &gt; 1) or \
-                 mode.filters)">:filterPx</x>
+             if="filterPx and mode.showFilters">:filterPx</x>
           <x if="ui.Title.showSub(class_, field)">:ui.Title.pxSub</x>
          </x>
         </x>
@@ -213,37 +240,32 @@ class List(Mode):
         ui = self.uiSearch
         search = ui.search
         tool = self.tool
-        # [Custom searches only]
-        if ui.name == 'customSearch':
-            # The search may be triggered via a Ref field
-            self.refObject, self.refField = search.getRefInfo(tool,
-                                                              nameOnly=False)
-            # Build the URL allowing to trigger a new search
-            o = self.refObject
-            part = '&ref=%s:%s' % (o.iid, self.refField.name) if o else ''
-            self.newSearchUrl = '%s/Search/advanced?className=%s%s' % \
-                                (tool.url, search.container.name, part)
-        else:
-            self.refObject, self.refField = None, None
-        self.fromRef = bool(self.refField)
-        # Build search parameters (start number, sort and filter)
         req = tool.req
+        # Build search parameters (start number, sort and filter)
         start = int(req.start or '0')
         self.sortKey = req.sortKey
         self.sortOrder = req.sortOrder or 'asc'
-        self.filters = ui.parseFilters(req)
+        self.filters = self.class_.getFilters(tool)
         # Run the search
         self.batch = search.run(tool.H(), start=start, sortBy=self.sortKey,
           sortOrder=self.sortOrder, filters=self.filters,
           refObject=self.refObject, refField=self.refField)
         self.batch.hook = self.hook
         self.objects = self.batch.objects
-        # Show sub-titles ?
+        # Show sub-titles ? Show filters ?
         self.showSubTitles = eval(req.showSubTitles or 'False')
+        self.showFilters = self.filters or (self.batch.total > 1)
         # Every matched object may be selected via a checkbox
         self.rootHook = ui.getRootHook()
         self.checkboxes = ui.checkboxes
-        self.checkboxesId = self.rootHook + '_objs'
+        if self.fromRef and self.inField:
+            # The search is a simple view of objects from a Ref field:
+            # checkboxes must have IDs similar to the standard view of these
+            # objects via the Ref field.
+            prefix = '%s_%s' % (self.refObject.iid, self.refField.name)
+        else:
+            prefix = self.rootHook
+        self.checkboxesId = prefix + '_objs'
         self.cbShown = ui.showCheckboxes()
         self.cbClass = '' if self.cbShown else 'hide'
         # Determine result emptiness
@@ -296,6 +318,17 @@ class List(Mode):
             r.append('%s:%s' % (k, v))
         return ','.join(r)
 
+    def inFilter(self, name, value):
+        '''Returns True if this p_value, for field named p_name, is among the
+           currently set p_self.filters.'''
+        values = self.filters.get(name)
+        if not values: return
+        if isinstance(values, str):
+            r = value == values
+        else:
+            r = value in values
+        return r
+
     def getNavInfo(self, number):
         '''Gets the navigation string corresponding to the element having this
            p_number within the list of results.'''
@@ -312,25 +345,47 @@ class Grid(List):
     '''Displays search results as a table containing one cell per object'''
 
     px = Px('''
+     <!-- Filters -->
+     <div class="gridFilters" if="uiSearch.search.showFilters">
+      <x for="column in [c for c in mode.columns if not c.special]"
+         var2="field,filterPx=column.field.getFilterPx()">
+       <div if="filterPx and mode.showFilters">
+        <div><label>:_(field.labelId)</label></div><x>:filterPx</x>
+       </div>
+      </x>
+     </div>
+
+     <!-- The grid itself -->
      <div style=":mode.gridder.getContainerStyle()">
-      <div for="zobj in mode.objects" class="thumbnail"
-           var2="obj=zobj.appy(); mayView=zobj.mayView()">
+      <div for="o in mode.objects" class="thumbnail"
+           var2="mayView=guard.mayView(o)">
        <table class="thumbtable">
         <tr var="@currentNumber=currentNumber + 1" valign="top"
             for="column in mode.columns"
             var2="field=column.field; backHook='searchResults'">
-         <td if="field.name=='title'" colspan="2">:field.pxRenderAsResult</td>
-         <x if="field.name!='title'">
+         <td if="field.name == 'title'" colspan="2">:field.pxResult</td>
+         <x if="(field.name != 'title') and mode.gridder.showFields">
           <td><label lfor=":field.name">::_('label', field=field)</label></td>
-          <td>:field.pxRenderAsResult</td>
+          <td>:field.pxResult</td>
          </x>
         </tr>
        </table>
-       <p class="thumbmore"><img src=":url('more')" class="clickable"
-          onclick="followTitleLink(this)"/></p>
+       <div class="thumbmore">
+        <img src=":url('more')" class="clickable"
+             onclick="followTitleLink(this)"/>
+       </div>
       </div>
+      <!-- Store object IDs in the session storage -->
+      <script>:mode.store</script>
+
+      <!-- Show a message if there is no visible object, due to filters -->
+      <div if="not mode.objects">::_('query_no_filtered_result')</div>
      </div>''',
 
+     css='''.gridFilters { display: flex; flex-wrap: wrap;
+                           justify-content: center; margin-top: 20px }
+            .gridFilters > div { margin: 10px 20px }''',
+
      js='''
       followTitleLink = function(img) {
         var parent = img.parentNode.parentNode,
@@ -342,7 +397,15 @@ class Grid(List):
     def __init__(self, *args):
         List.__init__(self, *args)
         # Extract the gridder defined on p_self.class_ or create a default one
-        self.gridder = getattr(self.class_, 'gridder', None) or Gridder()
+        gridder = getattr(self.class_.python, 'gridder', None)
+        if callable(gridder): gridder = gridder(self.tool)
+        self.gridder = gridder or Gridder()
+
+    def init(self):
+        '''Lazy initialisation'''
+        List.init(self)
+        # Disable the possibility to show/hide sub-titles
+        self.showSubTitles = True
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Calendar(Mode):
@@ -404,7 +467,7 @@ class Calendar(Mode):
         # Format for representing hours in the UI
         self.hourFormat = '%H:%M'
         # If filters are defined from a list mode, get it
-        self.filters = self.uiSearch.parseFilters(req)
+        self.filters = self.class_.getFilters(tool)
 
     def updateAjaxParameters(self, params):
         '''Grid-specific ajax parameters'''
