@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,8 +16,8 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import re, urllib.parse
 from pathlib import Path
+import re, urllib.parse, subprocess
 
 import appy.ui
 from appy.tr import po
@@ -30,14 +30,17 @@ from appy.data import nativeNames
 from appy.utils.dates import Date
 from appy.model.group import Group
 from appy.model.fields import Show
+from appy.model.query import Query
 from appy.database import Database
 from appy.ui.layout import Layouts
 from appy.utils.mail import sendMail
 from appy.ui.template import Template
 from appy.model.fields.ref import Ref
+from appy.server.backup import Backup
 from appy.model.searches import Search
 from appy.model.fields.rich import Rich
 from appy.model.fields import Initiator
+from appy.model.carousel import Carousel
 from appy.model.fields.phase import Page
 from appy.model.page import Page as OPage
 from appy.database.catalog import Catalog
@@ -46,6 +49,11 @@ from appy.model.fields.action import Action
 from appy.model.translation import Translation
 from appy.model.fields.calendar import Calendar
 from appy.model.fields.computed import Computed
+from appy.server.context import AuthenticationContext
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+RS_FG_KO   = 'Server cannot be restarted if running in the foreground.'
+RESTART_S  = "Restarting server (wait %d'')..."
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Tool(Base):
@@ -57,14 +65,16 @@ class Tool(Base):
     Date = Date
     OPage = OPage
     Search = Search
-    
+    AuthContext = AuthenticationContext
+
     # While most users have generally the read permission on the tool and
     # translations, they must not be allowed to consult tool/view and sub-pages.
     # Traversing tool/view is thus by default restricted to those having the
     # write permission on the tool.
     traverse = Base.traverse.copy()
     traverse.update({'ui':True, 'guard':True, 'Search':True,
-                     'Database':True, 'Server':True, 'view': 'perm:write'})
+                     'AuthContext': 'Authenticated', 'Database':True,
+                     'Server':True, 'view': 'perm:write'})
 
     Initiator = Initiator
 
@@ -161,6 +171,9 @@ class Tool(Base):
         if peers and peers.appy0:
             peers.appy0.pump(self)
 
+        # Define the home page
+        Tool.default = getattr(Tool, config.ui.home)
+
         # Call method "onInstall" on the tool when available. This hook allows
         # an app to execute code on server initialisation.
         if hasattr(self, 'onInstall'): self.onInstall()
@@ -287,23 +300,21 @@ class Tool(Base):
     #  Page "pages"
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    def showRootPages(self):
-        '''Show root pages if the logged user is allowed to write the tool'''
+    def showPagePages(self):
+        '''Show page "pages" if the logged user is allowed to write the tool'''
         # The page may be edited only if there is at least one page defined (in
         # order to select it as default page).
         if not self.allows('write'): return
         return 'view' if self.isEmpty('pages') else True
 
-    pp = Page('pages', show=showRootPages, label='Tool_page_pages',
+    pp = Page('pages', show=showPagePages, label='Tool_page_pages',
               icon='pagePages.svg')
 
     pages = Ref(OPage, multiplicity=(0,None), add=True, link=False,
                 composite=True, show=Show.VX, actionsDisplay='inline', page=pp,
                 back=Ref(attribute='toTool', show=False, layouts='f'),
-                showHeaders=True, **ta)
-
-    # Return the visible root pages
-    def getRootPages(self): return OPage.getRoot(self)
+                numbered=True, showHeaders=True, shownInfo=OPage.listColumns,
+                **ta)
 
     # The page, among "pages", being defined as the default one
     defaultPage = Ref(OPage, add=False, link=True, render='links', page=pp,
@@ -311,13 +322,38 @@ class Tool(Base):
                       show=lambda o: not o.isEmpty('pages'),
                       select=lambda o: o.pages, **ta)
 
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+    #  Page "carousels"
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+
+    pc = Page('carousels', show=lambda o: 'view' if o.allows('write') else None,
+              label='Tool_page_carousels')
+
+    carousels = Ref(Carousel, multiplicity=(0,None), add=True, link=False,
+                    composite=True, show=Show.VX, actionsDisplay='inline',
+                    back=Ref(attribute='toTool', show=False, layouts=Layouts.f),
+                    layouts=Layouts.dv, page=pc, showHeaders=True, **ta)
+
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+    #  Page "queries"
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+
+    pq = Page('queries', show=lambda o: 'view' if o.allows('write') else None,
+              label='Tool_page_queries')
+
+    queries = Ref(Query, multiplicity=(0,None), add=True, link=False,
+                  composite=True, show=Show.VX, actionsDisplay='inline',
+                  back=Ref(attribute='toTool', show=False, layouts=Layouts.f),
+                  layouts=Layouts.dv, page=pq, showHeaders=True,
+                  shownInfo=Query.listColumns, **ta)
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  "admin" zone
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
     def forAdmin(self):
         '''Returns page visibility for pages from the admin zone'''
-        return 'view' if self.user.login == 'admin' else None
+        return 'view' if self.user.hasRole('Manager') else None
 
     onlineUsers = Computed(method=lambda o: User.getOnline(o), layouts='f',
       show='view', page=Page('online', phase='admin', show=forAdmin,
@@ -379,25 +415,6 @@ class Tool(Base):
             # no default home object.
             if user.hasRole('Manager'): return self
 
-    def getPageTitle(self, o):
-        '''Returns content for tag html > head > title'''
-        # Return the app name in some cases
-        if not o or (o == self): return self.translate('app_name')
-        # The page title is based on p_o's title
-        r = o.getShownValue()
-        # Ensure this is a string
-        r = r if isinstance(r, str) else str(r)
-        # Remove <br/> tags
-        if '<br/>' in r: r = r[:r.index('<br/>')]
-        # Add the page title if found
-        if 'page' in self.req:
-            label = '%s_page_%s' % (obj.class_.name, self.req.page)
-            text = self.translate(label, blankOnError=True)
-            if text:
-                if r: r += ' :: '
-                r += text
-        return Px.truncateValue(res, width=100)
-
     def getGoogleAnalyticsCode(self, handler, config):
         '''If the config defines a Google Analytics ID, this method returns the
            Javascript code to be included in every page, allowing Google
@@ -428,27 +445,56 @@ class Tool(Base):
         if not date: return
         return self.config.ui.formatDate(self, date, format, withHour, language)
 
+    def restart(self, wait=5):
+        '''Restarts the currently running Appy server (S), by forking another
+           process that will wait p_wait seconds before restarting S.'''
+        # This cannot be done for servers running in the foreground
+        if self.H().server.mode == 'fg':
+            self.log(RS_FG_KO, type='warning')
+            return
+        # Fork a process that will launch a "./site restart" command. Waiting
+        # p_wait seconds may give enough time to the Appy server for committing
+        # ongoing transactions.
+        sitePath = self.config.server.sitePath
+        args = [str(sitePath / 'bin' / 'site'), 'restart']
+        if wait:
+            args.append('-w')
+            args.append(str(wait))
+        self.log(RESTART_S % wait)
+        subprocess.Popen(args, stdout=None, stdin=None, stderr=None)
+
+    def backup(self):
+        '''Method allowing to trigger the backup of a Appy site'''
+        return Backup(self).run()
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  PXs
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
-    # Hooks for defining a PX that proposes additional links or icons, before
-    # and after the links/icons corresponding to top-level pages and icons.
-    pxLinks = Px('')
-    pxLinksAfter = Px('')
-
     # PX "home" is the default page shown for a standard Appy site, excepted
     # when discreet logins are enabled. In this latter case, PX "public",
     # hereafter, is more appropriate.
     home = Px('''
      <!-- Redirect the user when appropriate -->
-     <x if="not isAnon or cfg.discreetLogin"
+     <x if="(not isAnon or cfg.discreetLogin) and not req.stay"
         var2="x=tool.goto(tool.computeHomePage())"></x>
 
+     <!-- Links -->
+     <div class=":'%sTop' % _px_.name">:Template.pxTemplateLinks</div>
+
      <!-- The home logo and text -->
-     <div if="isAnon and not cfg.discreetLogin" class="homeText">
-       <img if="cfg.homeLogo" src=":cfg.getImageUrl(siteUrl, 'homeLogo')"/>
-       <x if="not tool.isEmpty('defaultPage')">::tool.defaultPage.content</x>
+     <div if="isAnon and _px_.name == 'home'" class="homeText">
+       <img src=":url('homeLogo')"/>
+       <!-- The home text must not be shown here, if shown on a public page -->
+       <x if="not tool.isEmpty('defaultPage') and \
+              not cfg.discreetLogin">::tool.defaultPage.content</x>
+     </div>
+
+     <!-- The backgrounds for px "homes" -->
+     <div if="_px_.name == 'homes'" class="homes">
+      <div for="fileName, info in cfg.homesBackgrounds.items()"
+           var2="base,width=info; bg=tool.buildUrl(fileName, base=base,bg=True)"
+           style=":'%s;width:%s' % (bg, width)"></div>
      </div>''',
 
      css='''
@@ -457,20 +503,45 @@ class Tool(Base):
       .homeText h2 { font-size: 200% }
       .homeText p { font-size: 120%; margin: 0 }
       .homeText a, .homeText a:visited { color: #ffd8eb }
+      .homeTop { position:fixed; top:20px; right:20px; color:|headerColor| }
      ''',
 
      template=Template.px, hook='content', name='home')
 
-    default = home
+    # A variant of page "home", allowing to define several backgrounds, rendered
+    # one besides each other.
+    homes = Px(home.content, template=Template.px, hook='content', name='homes',
+     css='''.homes { display:flex; width:100%; height:100% }
+            .homes div { background-size: cover }
+            .homesTop { position:absolute; text-align:center; left:0; right:0;
+                        top:15px; z-index:10 }''')
 
-    # The "public" page is an alternative to "home" for anonymous users, when
-    # discreet logins are enabled.
+    # PX "public" shows one of the root pages (rp), or the default one if none
+    # of them is specified. When discreet logins are enabled, the public page
+    # is shown, loaded with the default page.
     public = Px('''
      <!-- The home text -->
-     <div var="default=tool.defaultPage" class="xhtml">
-      <x if="default">::default.content</x>
-      <x if="not default">::_('no_default_page')</x>
+     <div class="public"
+          var="layout='view';
+               root=tool.getObject(req.rp) if req.rp else tool.defaultPage">
+      <x if="root">
+       <!-- Display links on top of the 1st page, if the header is not shown -->
+       <div if="not showHeader"
+            class="publicTop">:Template.pxTemplateLinks</div>
+
+       <!-- Display the root page and all pages chained to it -->
+       <x for="o in root.getChain()"
+          var2="x=o.initialiseLocalPage(req)">:o.view</x>
+      </x>
+      <x if="not root">::_('no_default_page')</x>
      </div>''',
+
+     css='''
+      .publicTop { position:fixed; top:20px; right:20px; color:|headerColor| }
+      .publicTop .img { width:27px }
+      .publicTop .topIcons { display:inline }
+     ''',
+
      template=Template.px, hook='content', name='public')
 
     # Disable tool removal
