@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -78,6 +78,13 @@ class Object:
            exist.'''
         return
 
+    def __eq__(self, other):
+        '''Equality between objects is, like standard Python dicts, based on
+           equality of all their attributes and values.'''
+        if isinstance(other, Object):
+            return self.__dict__ == other.__dict__
+        return False
+
     def update(self, other):
         '''Set information from p_other (another Object instance or a dict) into
            p_self.'''
@@ -90,7 +97,7 @@ class Object:
         r.update(self)
         return r
 
-    # Allow our highly manipulated Object class to be picklable
+    # Allow this highly manipulated Object class to be picklable
     def __getstate__(self): return self.__dict__
     def __setstate__(self, state): self.__dict__.update(state)
 
@@ -184,5 +191,8 @@ def notFromPortlet(tool):
     # This is practical if you want to get the facilities provided by the
     # "portlet zone" for a class (ie, all search facilities) but without
     # allowing people people to create such classes at the root level.
-    return 'form' if not tool.traversal.context.rootClasses else None
+    try:
+        return 'form' if not tool.traversal.context.rootClasses else None
+    except AttributeError:
+        pass
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
