@@ -1,7 +1,7 @@
 '''This module defines classes allowing to store an object's history'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -30,6 +30,7 @@ from persistent.mapping import PersistentMapping
 
 from appy.px import Px
 from appy.model.batch import Batch
+from appy.xml.escape import Escape
 from appy.utils import string as sutils
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -132,6 +133,14 @@ class Event(persistent.Persistent):
         # A textual optional comment for the event
         self.comment = comment
 
+    def clone(self):
+        '''Create an return a clone from myself'''
+        params = self.__dict__.copy()
+        login = params.pop('login')
+        state = params.pop('state')
+        date = params.pop('date')
+        return self.__class__(login, state, date, **params)
+
     def completeComment(self, comment, sep='<br/><br/>'):
         '''Appends p_comment to the existing p_self.comment'''
         if not self.comment:
@@ -152,7 +161,7 @@ class Event(persistent.Persistent):
         '''Returns the formatted version of p_self.comment'''
         comment = self.comment
         if not comment: return empty
-        return sutils.formatText(comment)
+        Escape.xhtml(comment, p=True)
 
     def mayEditComment(self, config, user, isManager):
         '''May p_user edit the comment on this event ?'''
@@ -225,16 +234,17 @@ class Change(Event):
     pxDetail = Px('''
      <table class="changes" width="100%" if="event.changes">
       <tr>
-       <th align=":dleft" width="30%">:_('modified_field')</th>
-       <th align=":dleft" width="70%">:_('previous_value')</th>
+       <th align=":dleft">:_('modified_field')</th>
+       <th align=":dleft">:_('previous_value')</th>
       </tr>
       <tr for="name, value in event.changes.items()" valign="top"
           var2="fname=name if isinstance(name, str) else name[0];
                 lg=None if isinstance(name, str) else name[1];
                 field=o.getField(fname)">
-       <td><x>::_(field.labelId)</x>
+       <td><x>::_(field.labelId) if field else fname</x>
            <x if="lg">:' (%s)' % ui.Language.getName(lg)</x></td>
-       <td>::value</td>
+       <td>::field.getHistoryValue(o, value, history.data.index(event), lg) \
+             if field else value</td>
       </tr>
      </table>''')
 
@@ -283,17 +293,33 @@ class Change(Event):
             r.append('%s: %s' % (date, msg))
         return r
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class Custom(Event):
+    '''Represents a custom event, for which the label is chosen by the
+       developer.'''
+
+    def __init__(self, login, state, date, **params):
+        # Extract the label to use to name the event
+        self.label = params.pop('label')
+        Event.__init__(self, login, state, date, **params)
+
+    def getLabel(self, o):
+        '''Returns the translated label for a data change'''
+        return self.label
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Link(Event):
     '''Represents a link, via a Ref field, between 2 objects. The event is
        stored in the source object's history.'''
+
     # Although it may seem similar to a Change, it does not inherit from it.
     # Indeed, the complete list of previously linked objects is not stored:
     # instead, the title of the newly linked object is set as comment in this
     # Link event.
+
     def __init__(self, login, state, date, **params):
         # We may have linked an existing object or a newly created one
-        self.addition = params['addition']
+        self.addition = params.pop('addition')
         Event.__init__(self, login, state, date, **params)
 
     def getLabel(self, o):
@@ -302,9 +328,10 @@ class Link(Event):
 
 class Unlink(Event):
     '''Represents an object being unlinked from another one via a Ref field'''
+
     def __init__(self, login, state, date, **params):
         # We may have simply unlinked an object or it may have been deleted
-        self.deletion = params['deletion']
+        self.deletion = params.pop('deletion')
         Event.__init__(self, login, state, date, **params)
 
     def getLabel(self, o):
@@ -353,7 +380,8 @@ class History(PersistentList):
 
     view = Px('''
      <div if="not o.isTemp()"
-       var2="history=o.history; hasHistory=not history.isEmpty()">
+       var2="history=o.history; hasHistory=not history.isEmpty();
+             createComment=history[-1].comment">
       <table width="100%" class="header" cellpadding="0" cellspacing="0">
        <tr>
         <td colspan="2" class="by">
@@ -374,6 +402,9 @@ class History(PersistentList):
          <!-- State -->
          <x> &mdash; <x>:_('Base_state')</x> : 
             <b>:_(o.getLabel(o.state, field=False))</b></x>
+
+         <!-- Initial comment -->
+         <div if="createComment" class="topSpace">::createComment</div>
         </td>
        </tr>
 
@@ -383,14 +414,15 @@ class History(PersistentList):
      </div>''',
 
      css='''
-      .header { margin-bottom: 5px; background-color: #f3f3f7;
-                border: 3px solid white }
-      .by { padding: 7px }
-      .history td { padding: 8px }
-      .history th { font-style: italic; text-align: left; padding: 10px 5px;
-                    background-color: white }
+      .header { margin-bottom:5px; background-color:#f3f3f7;
+                border:3px solid white }
+      .by { padding:7px }
+      .history>tbody>tr>td { padding:8px }
+      .history>tbody>tr>th { font-style:italic; text-align:left;
+                             padding:10px 5px; background-color:white }
       .changes { margin: 4px 0 }
-      .changes td, .changes th { border: 1px solid #e0e0e0 }
+      .changes>tbody>tr>td, .changes>tbody>tr>th { border:1px solid #e0e0e0;
+                                                   padding:5px }
      ''')
 
     def __init__(self, o):
@@ -460,9 +492,28 @@ class History(PersistentList):
         # Browse fields
         for field in fields:
             if not field.getAttribute(o, 'historized'): continue
-            r[field.name] = field.getValue(o)
+            r[field.name] = field.getComparableValue(o)
         return r
 
+    def getNewer(self, field, value, i, language=None):
+        '''Find the newer version of this p_field p_value in p_self, starting at
+           index p_i, or, if not found, take the value as currently stored on
+           p_self.o.'''
+        # p_value was found in this history, at index p_i+1
+        found = False
+        name = field.name
+        for event in self.iter(eventType='Change', chronological=True, i=i):
+            if event.hasField(name, language=language):
+                # A newer version exists in this history
+                newer = event.getValue(name, language)
+                found = True
+                break
+        # If no newer version was found in p_o's history, take the value being
+        # currently stored on p_o for p_self.
+        if not found:
+            newer = getattr(self.o, name, None)
+        return newer
+
     def historize(self, previous):
         '''Records, in self.o's history, potential changes on historized fields.
            p_previous contains the values, before an update, of the historized
@@ -473,11 +524,11 @@ class History(PersistentList):
         for name in list(previous.keys()):
             prev = previous[name]
             field = o.getField(name)
-            curr = field.getValue(o)
+            curr = field.getValue(o, single=False)
             if (prev == curr) or ((prev is None) and (curr == '')) or \
                ((prev == '') and (curr is None)):
                 del(previous[name])
-                continue            
+                continue
             # The previous value may need to be formatted
             field.updateHistoryValue(o, previous)
         # Add the entry in the history (if not empty)
@@ -492,6 +543,14 @@ class History(PersistentList):
         return [event for event in self.iter(eventType=type, condition=cond, \
                                              context={'notBefore':notBefore})]
 
+    def setEvents(self, other):
+        '''Replace p_self's events with clones from p_other's events'''
+        # Empty p_self's history first
+        while len(self): del(self[0])
+        # Create clones from p_other's events and insert them into p_self
+        for event in other:
+            self.append(event.clone())
+
     def getBatch(self, o, size):
         '''Returns the Batch instance allowing to navigate within p_o's
            history.'''
@@ -509,4 +568,14 @@ class History(PersistentList):
         hook = batch.hook
         return "new AjaxData('%s/history/events','GET',%s,'%s')" % \
                (self.o.url, params, hook)
+
+    def replaceLogin(self, old, new):
+        '''Replace, in all events, login p_old with login p_new. Returns the
+           number of replacements done.'''
+        r = 0
+        for event in self:
+            if event.login == old:
+                event.login = new
+                r += 1
+        return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
