@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -404,8 +404,8 @@ class UiTransition:
                     (q(formId), q(transition.confirm), back) \
                     if mayTrigger else ''">
        <img src=":url(transition.icon, base=transition.iconBase)"
-            class=":picto"/><br/>
-       <x>:label</x>
+            class=":picto"/>
+       <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
       </a>
 
       <!-- Real button -->
