@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -19,61 +19,63 @@
 import re, sys
 from xml.sax.saxutils import quoteattr
 
-from appy.pod import actions
 from appy.pod import PodError
 from appy.pod.elements import *
 from appy.utils import Traceback
 from appy.xml import xmlPrologue
 from appy.xml.escape import Escape
+from appy.pod import actions, getUuid
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class ParsingError(Exception): pass
 
 # ParsingError-related constants - - - - - - - - - - - - - - - - - - - - - - - -
-ELEMENT = 'identifies the part of the document that will be impacted ' \
-  'by the command. It must be one of %s.' % str(PodElement.POD_ELEMS)
-FOR_EXPRESSION = 'must be of the form: {name} in {expression}. {name} must be '\
-  'a Python variable name. It is the name of the iteration variable. ' \
-  '{expression} is a Python expression that, when evaluated, produces a ' \
-  'Python sequence (tuple, string, list, etc).'
-POD_STATEMENT = 'A Pod statement has the form: do {element} ' \
-  '[{command} {expression}]. {element} ' + ELEMENT + ' Optional {command} ' \
-  'can be "if" (conditional inclusion of the element) or "for" (multiple ' \
-  'inclusion of the element). For an "if" command, {expression} is any ' \
-  'Python expression. For a "for" command, {expression} '+ FOR_EXPRESSION
-FROM_CLAUSE = 'A "from" clause has the form: from[+] {expression}, where ' \
-  '{expression} is a Python expression that, when evaluated, produces a valid '\
-  'chunk of odt content that will be inserted instead of the element that is ' \
-  'the target of the note. If the "+" is specified, the root target element ' \
-  'will be kept and its content will be replaced with the expression result.'
-EMPTY_NOTE = 'This note is empty. It must at least contain a Pod statement. '+ \
-  POD_STATEMENT
-BAD_STATEMENT = 'Syntax error for statement "%s". ' + POD_STATEMENT
-BAD_SUB_STATEMENT = 'Wrong sub-statement "%s".'
-BAD_ELEMENT = 'Bad element "%s". An element ' + ELEMENT
-BAD_MINUS = "The '-' operator can't be used with element '%s'. It can only be "\
-  "specified for elements among %s."
-ELEMENT_NOT_FOUND = 'Action specified element "%s" but available elements ' \
-  'in this part of the document are %s.'
-BAD_FROM_CLAUSE = 'Syntax error in "from" clause "%s". ' + FROM_CLAUSE
-DUPLICATE_NAMED_IF = 'An "if" statement with the same name already exists.'
-ELSE_NOT_MAIN = 'An "else" clause can only be defined as a main statement.'
-ELSE_WITHOUT_IF = 'No previous "if" statement could be found for this "else" ' \
-  'statement.'
-ELSE_WITHOUT_NAMED_IF = 'I could not find an "if" statement named "%s".'
-BAD_FOR_EXPRESSION = 'Bad "for" expression "%s". A "for" expression ' + \
-  FOR_EXPRESSION
-BAD_VAR_EXPRESSION = 'Bad variable definition "%s". A variable definition ' \
-  'must have the form: {name[,name2,...} = {expression}, or * = {expression}. '\
-  'Every name must be a Python-compliant variable name. {expression} is a ' \
-  'Python expression. When encountering such a statement, pod will define, ' \
-  'in the specified part of the document, variables {name}, {name2}, etc, ' \
-  'whose values will be initialized by evaluating {expression}.'
-BAD_VAR_STAR = 'A multi-variable definition must be of the form: * = {expr}'
-BAD_VAR_NAME = 'Bad variable name "%s".'
-EVAL_EXPR_ERROR = 'Error while evaluating expression "%s". %s'
-BAD_META_CONDITION = 'Wrong meta-condition "%s". A meta-condition must be a ' \
-  'Python expression surrounded by single or double quotes.'
+ELEMENT  = 'identifies the part of the document that will be impacted by the ' \
+           'command. It must be one of %s.' % str(PodElement.POD_ELEMS)
+FOR_EXPR = 'must be of the form: {name} in {expression}. {name} must be a ' \
+           'Python variable name. It is the name of the iteration variable. ' \
+           '{expression} is a Python expression that, when evaluated, ' \
+           'produces a Python sequence (tuple, string, list, etc).'
+POD_ST   = 'A Pod statement has the form: do {element} ' \
+           '[{command} {expression}]. {element} ' + ELEMENT + \
+           ' Optional {command} can be "if" (conditional inclusion of the ' \
+           'element) or "for" (multiple inclusion of the element). For an ' \
+           '"if" command, {expression} is any Python expression. For a "for" ' \
+           'command, {expression} '+ FOR_EXPR
+FROM_CL  = 'A "from" clause has the form: from[+] {expression}, where ' \
+           '{expression} is a Python expression that, when evaluated, ' \
+           'produces a valid chunk of odt content that will be inserted ' \
+           'instead of the element that is the target of the note. If the ' \
+           '"+" is specified, the root target element will be kept and its ' \
+           'content will be replaced with the expression result.'
+EMP_NOTE = 'This note is empty. It must at least contain a Pod statement. ' + \
+           POD_ST
+BAD_ST   = 'Syntax error for statement "%s". ' + POD_ST
+BAD_S_ST = 'Wrong sub-statement "%s".'
+BAD_ELT  = 'Bad element "%s". An element ' + ELEMENT
+BAD_MNS  = "The '-' operator can't be used with element '%s'. It can only be " \
+           "specified for elements among %s."
+ELT_N_F  = 'Action specified element "%s" but available elements in this ' \
+           'part of the document are %s.'
+FR_CL_KO = 'Syntax error in "from" clause "%s". ' + FROM_CL
+DUP_N_IF = 'An "if" statement with the same name already exists.'
+ELSE_N_M = 'An "else" clause can only be defined as a main statement.'
+ELSE_N_I = 'No previous "if" statement could be found for this "else" ' \
+           'statement.'
+ELSE_N_N = 'I could not find an "if" statement named "%s".'
+FOR_KO   = 'Bad "for" expression "%s". A "for" expression ' + FOR_EXPR
+VAR_KO   = 'Bad variable definition "%s". A variable definition must have ' \
+           'the form: {name[,name2,...} = {expression}, or * = {expression}. '\
+           'Every name must be a Python-compliant variable name. ' \
+           '{expression} is a Python expression. When encountering such a ' \
+           'statement, pod will define, in the specified part of the ' \
+           'document, variables {name}, {name2}, etc, whose values will be ' \
+           'initialized by evaluating {expression}.'
+VAR_S_KO = 'A multi-variable definition must be of the form: * = {expr}'
+VAR_N_KO = 'Bad variable name "%s".'
+EVAL_KO  = 'Error while evaluating expression "%s". %s'
+MTA_C_KO = 'Wrong meta-condition "%s". A meta-condition must be a Python ' \
+           'expression surrounded by single or double quotes.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class BufferIterator:
@@ -115,7 +117,6 @@ class BufferIterator:
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Buffer:
     '''Abstract class representing any buffer used during rendering'''
-    elementRex = re.compile('([\w-]+:[\w-]+)\s*(.*?)>', re.S)
 
     def __init__(self, env, parent):
         self.parent = parent
@@ -141,6 +142,16 @@ class Buffer:
 
     def getLength(self): pass # To be overridden
 
+    def patchProtected(self, elem, attrs):
+        '''Add or patch the attribute allowing to protect this p_elem (a cell or
+           section), by evaluating variable "cellProtected" or
+           "sectionProtected".'''
+        type = elem[:elem.index(':')]
+        name = '%s:protected' % type
+        var = 'cell' if type == 'table' else 'section'
+        value = attrs.get(name, "false").strip() or "false"
+        attrs[name] = ':str(%sProtected).lower()|"%s"' % (var, value)
+
     def patchElement(self, elem, attrs):
         '''Modifies p_elem's attributes when relevant and make them dependent of
            some variable definitions.'''
@@ -150,7 +161,8 @@ class Buffer:
             # ie, for naming calc sheets (which are implemented as tables).
             attrs = attrs._attrs
             name = 'table:name'
-            attrs[name] = ':tableName|"%s"' % attrs[name]
+            currentName = attrs.get(name) or getUuid(removeDots=True,prefix='T')
+            attrs[name] = ':tableName|"%s"' % currentName
         elif elem == 'table:table-column':
             # Convert attribute "number-columns-repeated" of every table column
             # (or add it if it does not exist) to let the user define how he
@@ -160,18 +172,25 @@ class Buffer:
             columnNumber = self.env.getTable().nbOfColumns -1
             nb = (key in attrs) and attrs[key] or '1'
             attrs[key] = ':columnsRepeated[%d]|%s' % (columnNumber, nb)
-        elif self.env.parser.caller.protection and \
-             (elem in ('table:table-cell', 'text:section')):
-            # Add or patch the attribute allowing to protect the cell or
-            # section, by evaluating variable "cellProtected" or
-            # "sectionProtected".
+        elif elem == 'table:table-cell':
             attrs = attrs._attrs
-            type = elem[:elem.index(':')]
-            name = '%s:protected' % type
-            var = (type == 'table') and 'cell' or 'section'
-            value = attrs.get(name, "false").strip() or "false"
-            attrs[name] = ':str(%sProtected).lower()|"%s"' % (var, value)
-        elif (elem == 'text:list-item') and ('text:start-value' in attrs):
+            key = 'table:style-name'
+            style = attrs.get(key)
+            if style and style.startswith('S'):
+                # By convention, if a style name starts with letter "S", POD
+                # will allow to replace the name of the cell style with another
+                # one, via function "styleFor" if found in the context. The
+                # function will be passed 2 args: the name of the style that
+                # should normally have been applied, and the context itself. It
+                # must return the name of the style to apply. This mechanism is
+                # restricted to styles starting with "S" for performance
+                # reasons.
+                attrs[key] = ':styleFor("%s",_ctx_)|"%s"' % (style, style)
+            elif self.env.parser.caller.protection:
+                self.patchProtected(elem, attrs)
+        elif elem == 'text:section' and self.env.parser.caller.protection:
+            self.patchProtected(elem, attrs._attrs)
+        elif elem == 'text:list-item' and 'text:start-value' in attrs:
             attrs = attrs._attrs
             val = int(attrs['text:start-value'])
             attrs['text:start-value'] = ':loop._all_[-1].nb+%d|%d' % (val, val)
@@ -240,7 +259,7 @@ class FileBuffer(Buffer):
     def __init__(self, env, result):
         Buffer.__init__(self, env, None)
         self.result = result
-        self.content = open(result, 'w')
+        self.content = open(result, 'w', encoding='utf-8')
         self.content.write(xmlPrologue)
 
     # getLength is used to manage insertions into sub-buffers. But in the case
@@ -263,10 +282,9 @@ class FileBuffer(Buffer):
             else: self.write(res)
         except Exception as e:
             if not self.env.raiseOnError:
-                PodError.dump(self, EVAL_EXPR_ERROR % (expression, e),
-                              dumpTb=False)
+                PodError.dump(self, EVAL_KO % (expression, e), dumpTb=False)
             else:
-                raise Exception(EVAL_EXPR_ERROR % (expression, e))
+                raise Exception(EVAL_KO % (expression, e))
 
     def addAttributes(self):
         # Into a FileBuffer, it is not possible to insert Attributes. Every
@@ -491,7 +509,7 @@ class MemoryBuffer(Buffer):
                 metaWrap = metaCondition[0]
                 if (metaWrap != metaCondition[-1]) or \
                    (metaWrap not in expr.metaWraps):
-                    raise ParsingError(BAD_META_CONDITION % metaCondition)
+                    raise ParsingError(MTA_C_KO % metaCondition)
                 expr.metaWrap = metaWrap
                 expr.metaCondition = metaCondition.strip('"\'"')
         if tiedHook: tiedHook.tiedExpression = expr
@@ -525,12 +543,12 @@ class MemoryBuffer(Buffer):
             # Do we have valid variable definitions ?
             match = self.Rex.vars_.match(sub)
             if not match:
-                raise ParsingError(BAD_VAR_EXPRESSION % sub)
+                raise ParsingError(VAR_KO % sub)
             # Extract individual variable definitions
             names = match.group(1).strip()
             if '*' in names:
                 if len(names) > 1:
-                    raise ParsingError(BAD_VAR_STAR % name)
+                    raise ParsingError(VAR_S_KO % name)
             elif ',' in names:
                 # Several variable names
                 names = names.split(',')
@@ -539,7 +557,7 @@ class MemoryBuffer(Buffer):
                 while i < len(names):
                     name = names[i].strip()
                     if not self.Rex.var_.match(name):
-                        raise ParsingError(BAD_VAR_NAME % name)
+                        raise ParsingError(VAR_N_KO % name)
                     names[i] = name
                     i += 1
             r.append((names, match.group(2).strip()))
@@ -565,16 +583,16 @@ class MemoryBuffer(Buffer):
                 if r.name:
                     # We must register this action as a named action
                     if r.name in self.env.namedIfActions:
-                        raise ParsingError(DUPLICATE_NAMED_IF)
+                        raise ParsingError(DUP_N_IF)
                     self.env.namedIfActions[r.name] = r
         elif actionType == 'else':
-            if not main: raise ParsingError(ELSE_NOT_MAIN)
-            if not self.env.ifActions: raise ParsingError(ELSE_WITHOUT_IF)
+            if not main: raise ParsingError(ELSE_N_M)
+            if not self.env.ifActions: raise ParsingError(ELSE_N_I)
             # Does the "else" action reference a named "if" action?
             ifReference = subExpr.strip()
             if ifReference:
                 if ifReference not in self.env.namedIfActions:
-                    raise ParsingError(ELSE_WITHOUT_NAMED_IF % ifReference)
+                    raise ParsingError(ELSE_N_N % ifReference)
                 linkedIfAction = self.env.namedIfActions[ifReference]
                 # This "else" action "consumes" the "if" action: this way,
                 # it is not possible to define two "else" actions related to
@@ -588,7 +606,7 @@ class MemoryBuffer(Buffer):
         elif actionType == 'for':
             forRes = self.Rex.for_.match(subExpr.strip())
             if not forRes:
-                raise ParsingError(BAD_FOR_EXPRESSION % subExpr)
+                raise ParsingError(FOR_KO % subExpr)
             iters, subExpr = forRes.groups()
             iters = self._getForIterators(iters)
             r = actions.For(statementName, self, subExpr, podElem, minus, iters)
@@ -608,21 +626,21 @@ class MemoryBuffer(Buffer):
         r = -1
         try:
             # Check that the statement group is not empty
-            if not statements: raise ParsingError(EMPTY_NOTE)
+            if not statements: raise ParsingError(EMP_NOTE)
             # Get the main statement (starting with "do...") and check it
             main = statements[0]
             aRes = self.Rex.action.match(main)
             if not aRes:
-                raise ParsingError(BAD_STATEMENT % main)
+                raise ParsingError(BAD_ST % main)
             statementName, podElem, minus, actionType, subExpr = aRes.groups()
             if not (podElem in PodElement.POD_ELEMS):
-                raise ParsingError(BAD_ELEMENT % podElem)
+                raise ParsingError(BAD_ELT % podElem)
             if minus and (not podElem in PodElement.MINUS_ELEMS):
-                raise ParsingError(BAD_MINUS % (podElem,PodElement.MINUS_ELEMS))
+                raise ParsingError(BAD_MNS % (podElem,PodElement.MINUS_ELEMS))
             # Find the target element in the buffer
             i = self.getIndex(podElem)
             if i == -1:
-                raise ParsingError(ELEMENT_NOT_FOUND % (podElem, str([
+                raise ParsingError(ELT_N_F % (podElem, str([
                         e.__class__.__name__.lower() \
                         for e in self.elements.values()])))
             podElem = self.elements[i]
@@ -638,13 +656,13 @@ class MemoryBuffer(Buffer):
                    statement.startswith('from+'):
                     fromInfo = self.Rex.from_.match(statement)
                     if not fromInfo:
-                        raise ParsingError(BAD_FROM_CLAUSE % fromClause)
+                        raise ParsingError(FR_CL_KO % fromClause)
                     fromClause = fromInfo.groups()
                 # Get any secondary statement
                 else:
                     info = self.Rex.subAction.match(statement)
                     if not info:
-                        raise ParsingError(BAD_SUB_STATEMENT % statement)
+                        raise ParsingError(BAD_S_ST % statement)
                     actionType, subExpr = info.groups()
                     last = self.createPodAction(actionType, statements, '',
                                              subExpr, podElem, None, main=False)
@@ -669,7 +687,7 @@ class MemoryBuffer(Buffer):
         if actionType == 'for':
             forRes = self.Rex.for_.match(statement)
             if not forRes:
-                raise ParsingError(BAD_FOR_EXPRESSION % statement)
+                raise ParsingError(FOR_KO % statement)
             iters, subExpr = forRes.groups()
             iters = self._getForIterators(iters)
             action = actions.For('for', self, subExpr, elem, False, iters)
@@ -858,10 +876,9 @@ class MemoryBuffer(Buffer):
                         if self.env.raiseOnError: raise e
                     except Exception as e:
                         if not self.env.raiseOnError:
-                            PodError.dump(result, EVAL_EXPR_ERROR % (
-                                          evalEntry.expr, e))
+                            PodError.dump(result, EVAL_KO % (evalEntry.expr, e))
                         else:
-                            raise actions.EvaluationError(e, EVAL_EXPR_ERROR % \
+                            raise actions.EvaluationError(e, EVAL_KO % \
                                         (evalEntry.expr, '\n'+Traceback.get(5)))
                 elif isinstance(evalEntry, Attributes) or \
                      isinstance(evalEntry, Attribute):
