@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -19,6 +19,7 @@
 import sys, os, os.path, re, time
 from optparse import OptionParser
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 htmlFilters = {'odt': 'HTML (StarWriter)',
                'ods': 'HTML (StarCalc)',
                'odp': 'impress_html_Export'}
@@ -59,8 +60,10 @@ configProvider = 'com.sun.star.configuration.ConfigurationProvider'
 configAccess = 'com.sun.star.configuration.ConfigurationAccess'
 configUpdateAccess = 'com.sun.star.configuration.ConfigurationUpdateAccess'
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class ConverterError(Exception): pass
+# Default options when exporting to CSV
+defaultCsvOptions = '59,34,76,1'
+# Defaut options when reading a CSV file
+defaultCsvLoadOptions = defaultCsvOptions
 
 # Constants and messages - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DOC_NOT_FOUND = '"%s" not found.'
@@ -125,6 +128,18 @@ HELP_PDF = 'If the output format is PDF, you can define here conversion ' \
   'options, as a series of comma-separated key=value pairs, as in ' \
   '"ExportNotes=True,PageRange=1-20". Available options are documented in ' \
   '%s.' % HELP_PDF_URL
+HELP_CSV_URL = 'https://wiki.openoffice.org/wiki/Documentation/DevGuide/' \
+  'Spreadsheets/Filter_Options#Filter_Options_for_the_CSV_Filter'
+HELP_CSV = 'If the ouput format is CSV, you can define here conversion ' \
+  'options, as a comma-separated list of values. Default options are: %s.' \
+  'Values correspond to ASCII codes. The first one represents the field ' \
+  'separator. The most frequent values are: 59 (the semi-colon ;), 44 (the ' \
+  'comma ,) and 9 (a tab). The second value represents the text delimiter. ' \
+  'The most frequent values are: 34 (double quotes), 39 (single quotes) or ' \
+  'no value at all (as in 59,,76,1). The third one is the file encoding. The ' \
+  'most frequent values are 76 (UTF-8) and 12 (ISO-8859-1). Complete ' \
+  'documentation about CSV options can be found at %s.' % \
+  (defaultCsvOptions, HELP_CSV_URL)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Classes defined here are used to contact LibreOffice via input/output streams
@@ -265,13 +280,15 @@ class PdfOptions:
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Converter:
     '''Converts a document readable by LibreOffice into pdf, doc, txt, rtf...'''
-    Error = ConverterError
+
+    # Converter-specific error class
+    class Error(Exception): pass
 
     def __init__(self, docPath, result, server=DEFAULT_SERVER,
                  port=DEFAULT_PORT, templatePath=None, optimalColumnWidths=None,
                  distributeColumns=None, script=None, resolveFields=False,
-                 pdfOptions=None, ppp=False, stream='auto', pageStart=1,
-                 verbose=False):
+                 pdfOptions=None, csvOptions=defaultCsvOptions, ppp=False,
+                 stream='auto', pageStart=1, verbose=False):
         # The server and port where LibreOffice listens
         self.server = server
         self.port = port
@@ -314,6 +331,8 @@ class Converter:
             self.functions = {}
         # Options for conversion to PDF
         self.pdfOptions = PdfOptions().get(pdfOptions)
+        # Options for conversion to CSV
+        self.csvOptions = csvOptions
         # Verbosity
         self.verbose = verbose
         # Enable the PPP (POD post-processor) ?
@@ -335,7 +354,7 @@ class Converter:
     def getInputType(self, docPath):
         '''Extracts the input type from the p_docPath'''
         res = os.path.splitext(docPath)[1][1:].lower()
-        if res not in FILE_TYPES: raise ConverterError(INPUT_TYPE_ERROR % res)
+        if res not in FILE_TYPES: raise self.Error(INPUT_TYPE_ERROR % res)
         return res
 
     def getResultType(self, result):
@@ -369,7 +388,7 @@ class Converter:
            and the absolute path as the second element.''' 
         import unohelper
         if not os.path.exists(filePath) and not os.path.isfile(filePath):
-            raise ConverterError(DOC_NOT_FOUND % filePath)
+            raise self.Error(DOC_NOT_FOUND % filePath)
         docAbsPath = os.path.abspath(filePath)
         # Return one path for OO, one path for me
         return unohelper.systemPathToFileUrl(docAbsPath), docAbsPath
@@ -382,8 +401,8 @@ class Converter:
             if isinstance(res, dict):
                 res = res[self.inputType]
         else:
-            raise ConverterError(BAD_RESULT_TYPE % (self.resultType,
-                                                    FILE_TYPES.keys()))
+            raise self.Error(BAD_RESULT_TYPE % (self.resultType,
+                                                FILE_TYPES.keys()))
         return res
 
     def getResultUrl(self, result):
@@ -418,9 +437,8 @@ class Converter:
             f.close()
             os.remove(r)
             return unohelper.systemPathToFileUrl(r)
-        except (OSError, IOError):
-            e = sys.exc_info()[1]
-            raise ConverterError(CANNOT_WRITE_RESULT % (r, e))
+        except (OSError, IOError) as e:
+            raise self.Error(CANNOT_WRITE_RESULT % (r, e))
 
     def props(self, properties):
         '''Create a UNO-compliant tuple of properties, from tuple p_properties
@@ -480,9 +498,8 @@ class Converter:
                 helper = create('com.sun.star.frame.DispatchHelper', ctx)
                 self.dispatchHelper = helper
             self.log(' done.')
-        except NoConnectException:
-            e = sys.exc_info()[1]
-            raise ConverterError(CONNECT_ERROR % (self.port, e))
+        except NoConnectException as e:
+            raise self.Error(CONNECT_ERROR % (self.port, e))
 
     def getColumnModifiers(self):
         '''Returns the elements allowing to know if we must optimize or
@@ -599,9 +616,14 @@ class Converter:
         # Getting some base LO objects is required
         controller = self.doc.getCurrentController()
         viewCursor = controller.getViewCursor()
-        # Force LO to load big documents in their entirery
-        viewCursor.jumpToLastPage()
-        viewCursor.jumpToEndOfPage()
+        # The following lines of code were a trial to force LO to load big
+        # documents in their entirery: because, with some early LO 6.x versions,
+        # it could happen that resulting PDF results were truncated. But in some
+        # rare cases, those lines caused LO to run 100% CPU in an infinite loop.
+        # ~~~
+        # viewCursor.jumpToLastPage()
+        # viewCursor.jumpToEndOfPage()
+        # ~~~
         # 1) Update indexes
         for index in LoIter(self.doc.getDocumentIndexes(), log):
             if index.ServiceName == 'com.sun.star.text.ContentIndex':
@@ -693,8 +715,7 @@ class Converter:
             props = [('Hidden', True)]
             if self.inputType == 'csv':
                 # Give some additional params if we need to open a CSV file
-                props.append(('FilterFlags', '59,34,76,1'))
-                #props.append(('FilterData', 'Any'))
+                props.append(('FilterFlags', defaultCsvLoadOptions))
             self.doc = self.loadDocument(self.docUrl, self.props(props))
             self.log(' done.')
             # [ODT] Perform additional tasks
@@ -706,9 +727,8 @@ class Converter:
                 pass
             # [ODT] After every change has been applied, refresh the TOC
             if isOdt: self.updateToc()
-        except IllegalArgumentException:
-            e = sys.exc_info()[1]
-            raise ConverterError(URL_NOT_FOUND % (self.docPath, e))
+        except IllegalArgumentException as e:
+            raise self.Error(URL_NOT_FOUND % (self.docPath, e))
 
     def setPdfOptions(self):
         '''If p_self.doc must be converted to PDF, and PDF options need to be
@@ -732,8 +752,9 @@ class Converter:
            type.'''
         self.log('Saving the result in %s...' % self.resultUrl, cr=False)
         props = [('FilterName', self.resultFilter), ('UpdateDocMode', 3)]
-        if self.resultType == 'csv': # Add options for CSV export
-            props.append(('FilterOptions', '59,34,76,1')) # 59=; 34=" 
+        if self.resultType == 'csv':
+            # Add options for CSV export
+            props.append(('FilterOptions', self.csvOptions))
         elif self.resultType == 'pdf':
             # Get optional PDF options
             options = self.setPdfOptions()
@@ -814,6 +835,8 @@ class ConverterScript:
         add('-v', '--verbose', action='store_true', help=HELP_VERBOSE)
         add('-f', '--pdf', dest='pdf', default=None, metavar='PDF_OPTIONS',
             type='string', help=HELP_PDF)
+        add('-i', '--csv', dest='csv', default=defaultCsvOptions,
+            metavar='CSV_OPTIONS', type='string', help=HELP_CSV)
         add('-c', '--ppp', action='store_true', help=HELP_PPP)
         add('-a', '--stream', dest='stream', default='auto',
             metavar='STREAM', type='string', help=HELP_STREAM)
@@ -836,12 +859,12 @@ class ConverterScript:
         if stream in ('True', 'False'): stream = eval(stream)
         converter = Converter(args[0], args[1], options.server, options.port,
           options.template, optimize, distribute, options.script, resolveFields,
-          options.pdf, options.ppp, stream, options.pageStart, options.verbose)
+          options.pdf, options.csv, options.ppp, stream, options.pageStart,
+          options.verbose)
         try:
             converter.run()
-        except ConverterError:
-            e = sys.exc_info()[1]
-            sys.stderr.write(str(e))
+        except Converter.Error as err:
+            sys.stderr.write(str(err))
             sys.stderr.write('\n')
             optParser.print_help()
             sys.exit(ERROR_CODE)
