@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,11 +16,11 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import os,os.path,stat,time,shutil,struct,random,urllib.parse,imghdr,uuid,base64
+import os, os.path, re, stat, shutil, struct, urllib.parse, imghdr, base64
 
 import appy.pod
 from appy import utils
-from appy.pod import PodError
+from appy.pod import PodError, getUuid
 from appy.utils.client import Resource
 from appy.model.utils import Object as O
 from appy.ui.css import CssStyles, px2cm
@@ -34,14 +34,6 @@ PDF_TO_IMG_ERROR = 'A PDF file could not be converted into images. Please ' \
                    'system and the "gs" program is in the path.'
 TO_PDF_ERROR     = 'ConvertImporter error while converting a doc to PDF: %s.'
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def getUuid(removeDots=False):
-    '''Get a unique id (ie, for images/documents to be imported into an ODT
-       document).'''
-    r = uuid.uuid4().hex
-    if removeDots: r = r.replace('.', '')
-    return r
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class DocImporter:
     '''Base class used for importing external content into a pod template (an
@@ -102,11 +94,11 @@ class DocImporter:
         '''Gets the path name of the file to dump on disk (within the ODT for
            images, in a temp folder for docs).'''
         format = self.format
-        if not format or (format == 'image'):
+        if not format or format == 'image':
             at = self.at
-            if at.startswith('http') or not os.path.exists(at):
-                # We will know it only after the HTTP GET (or never if the file
-                # does not exist)
+            if not at or at.startswith('http') or not os.path.exists(at):
+                # It is not possible yet to determine the format (will be done
+                # later).
                 self.format = ''
             else:
                 self.format = os.path.splitext(at)[1][1:]
@@ -159,10 +151,11 @@ class OdtImporter(DocImporter):
         # Insert a page break before importing the doc if needed
         if self.pageBreakBefore: self.res += self.pageBreaks.before
         # Import the external odt document
-        self.res += '<%s:section %s:name="PodImportSection%f">' \
+        name = getUuid(removeDots=True, prefix='PodSect')
+        self.res += '<%s:section %s:name="%s">' \
                     '<%s:section-source %s:href="file://%s" ' \
                     '%s:filter-name="writer8"/></%s:section>' % (
-                        self.textNs, self.textNs, time.time(), self.textNs,
+                        self.textNs, self.textNs, name, self.textNs,
                         self.linkNs, self.importPath, self.textNs, self.textNs)
         # Insert (a) page break(s) after importing the doc if needed
         if self.pageBreakAfter:
@@ -348,7 +341,7 @@ class Image:
 
     def __init__(self, path, format, unit='cm'):
         self.path = path # The image absolute path on disk
-        self.format = format
+        self.format = format or imghdr.what(path)
         # Determine image size in pixels (again, by reading its first bytes)
         self.width, self.height = self.getSizeIn(unit)
 
@@ -412,6 +405,9 @@ class ImageImporter(DocImporter):
     imageNotFound = os.path.join(os.path.dirname(appy.pod.__file__),
                                  'imageNotFound.jpg')
 
+    # Regular expression for finding orientation within EXIF meta-data
+    orientationRex = re.compile('exif:Orientation\s*=\s*(\d+)')
+
     def isImage(self, response):
         '''Cheks that the p_response received after a HTTP GET supposed to
            retrieve an image actually contains an image.'''
@@ -450,14 +446,19 @@ class ImageImporter(DocImporter):
             # the "src" attribute of a HTML "img" tag of the form:
             #              "data:<mimeType>;base64,<base64 content>"
             mimeType = at[5:at.index(';')]
-            self.format = mimeTypesExts[mimeType]
+            # The MIME type may be "image/*". In that case, set "img" as virtual
+            # format; the real format will be detected once the file will be
+            # dumped to disk.
+            self.format = utils.mimeTypesExts.get(mimeType) or 'img'
             content = at[at.index(',')+1:]
             # Decode the base64 content and store it in a temp file on disk
             content = base64.b64decode(content)
-            fileName = sutils.getTempFileName(extension=self.format)
+            fileName = getTempFileName(extension=self.format)
             f = open(fileName, 'wb')
             f.write(content)
             f.close()
+            if self.format == 'img':
+                self.format = imghdr.what(fileName)
             return fileName
         else:
             if at.startswith('file://'): at = at[7:]
@@ -484,7 +485,7 @@ class ImageImporter(DocImporter):
         # The image has not already been imported: copy it
         if not at.startswith('http'):
             shutil.copy(at, importPath)
-            # Ensure we can modify the image (with imagemagick)
+            # Ensure we can modify the image (with ImageMagick)
             os.chmod(importPath, stat.S_IREAD | stat.S_IWRITE)
             return importPath
         # The image has (maybe) been retrieved from a HTTP GET
@@ -501,6 +502,19 @@ class ImageImporter(DocImporter):
                 f.close()
                 return importPath
 
+    def manageExif(self):
+        '''Read, with ImageMagick, EXIF metadata in the image, and apply a
+           rotation (still with ImageMagick) if required.'''
+        # Indeed, LO will not interpret EXIF metadata when reading the image
+        cmd = ['identify', '-format', '%[EXIF:*]', self.importPath]
+        out, err = utils.executeCommand(cmd)
+        if err: raise Exception(CONVERT_ERROR)
+        if out:
+            match = self.orientationRex.search(out)
+            if match:
+                #code = int(match.group(1)) # "auto-orient" manages all codes
+                utils.convert(self.importPath, '-auto-orient')
+
     def init(self, anchor, wrapInPara, size, sizeUnit, maxWidth, cssAttrs,
              keepRatio, convertOptions):
         '''ImageImporter-specific constructor'''
@@ -524,7 +538,9 @@ class ImageImporter(DocImporter):
             if w and h:
                 self.sizeUnit = w.unit
                 self.size = (w.value, h.value)
-        # Call imagemagick to perform a custom conversion if required
+        # Manage EXIF tags when relevant
+        if self.renderer.rotateImages: self.manageExif()
+        # Call ImageMagick to perform a custom conversion if required
         options = self.convertOptions
         image = None
         transformed = False
@@ -608,10 +624,11 @@ class ImageImporter(DocImporter):
             self.anchor = 'char'
         else:
             styleInfo = ''
+        name = getUuid(prefix='I', removeDots=True)
         image = '<%s:frame %s%s:name="%s" %s:z-index="0" ' \
                 '%s:anchor-type="%s"%s><%s:image %s:type="simple" ' \
                 '%s:show="embed" %s:href="%s" %s:actuate="onLoad"/>' \
-                '</%s:frame>' % (d, styleInfo, d, getUuid(), d, t, self.anchor,
+                '</%s:frame>' % (d, styleInfo, d, name, d, t, self.anchor,
                 self.getImageSize(), d, x, x, x, imagePath, x, d)
         if hasattr(self, 'wrapInPara') and self.wrapInPara:
             style = isinstance(self.wrapInPara, str) and \
