@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,45 +18,43 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import re, os.path, random, string
 from collections import UserDict
+import re, os.path, random, string
+
 from appy import utils
 from appy.pod import *
-from appy.pod.odf_parser import OdfEnvironment, OdfParser
 from appy.pod.doc_importers import px2cm as px2cmRatio
-from appy.pod.doc_importers import getUuid
+from appy.pod.odf_parser import OdfEnvironment, OdfParser
 from appy.ui.css import parseStyleAttribute, CssStyles, CssValue, px2cm
 
-# Possible states for the parser
-READING = 0 # Default state
-PARSING_STYLE = 1 # Parsing a style definition
-PARSING_MASTER_STYLES = 2 # Parsing section "master-styles"
-PARSING_PAGE_LAYOUT = 3 # Parsing a page layout
+# Possible states for the styles parser
+READING          = 0 # Default state
+PARSING_STYLE    = 1 # Parsing a style definition
+PARSING_M_STYLES = 2 # Parsing section "master-styles"
+PARSING_P_LAYOUT = 3 # Parsing a page layout
 
 # Error-related constants  - - - - - - - - - - - - - - - - - - - - - - - - - - -
-MAPPING_NOT_DICT = 'The styles mapping must be a dictionary or a UserDict ' \
-  'instance.'
-MAPPING_KEY_NOT_STRING = "The styles mapping dictionary's keys must be strings."
-MAPPING_ELEM_NOT_STRING = 'The styles mapping value for key "%s" must be a ' \
-  'string.'
-MAPPING_ELEM_EMPTY = 'In your styles mapping, you inserted an empty key ' \
-  'and/or value.'
-MAPPING_WRONG_VALUE_TYPE = 'For key "%s", the value must be of type "%s".'
-UNSTYLABLE_TAG = 'You can\'t associate a style to element "%s". Unstylable ' \
-  'elements are: %s'
-STYLE_NOT_FOUND = 'OpenDocument style "%s" was not found in your template. ' \
-  'Note that the styles names ("Heading 1", "Standard"...) that appear when ' \
-  'opening your template with OpenOffice, for example, are a super-set of ' \
-  'the styles that are really recorded into your document. Indeed, only ' \
-  'styles that are in use within your template are actually recorded into ' \
-  'the document. You may consult the list of available styles ' \
-  'programmatically by calling your pod renderer\'s "getStyles" method.'
-HTML_PARA_ODT_TEXT = 'For XHTML element "%s", you must associate a ' \
-  'paragraph-wide OpenDocument style. "%s" is a "text" style (that applies ' \
-  'to only a chunk of text within a paragraph).'
-HTML_TEXT_ODT_PARA = 'For XHTML element "%s", you must associate an ' \
-  'OpenDocument "text" style (that applies to only a chunk of text within a ' \
-  'paragraph). "%s" is a paragraph-wide style.'
+MAP_KO     = 'The styles mapping must be a dictionary or a UserDict instance.'
+KEY_STR_KO = "The styles mapping dictionary's keys must be strings."
+VAL_STR_KO = 'The styles mapping value for key "%s" must be a string.'
+VAL_EMPTY  = 'In your styles mapping, you inserted an empty key and/or value.'
+VAL_TYP_KO = 'For key "%s", the value must be of type "%s".'
+UNSTYL_TAG = 'You can\'t associate a style to element "%s". Unstylable ' \
+             'elements are: %s'
+STYLE_N_F  = 'OpenDocument style "%s" was not found in your template. Note ' \
+             'that the styles names ("Heading 1", "Standard"...) that appear ' \
+             'when opening your template with OpenOffice, for example, are a ' \
+             'super-set of the styles that are really recorded into your ' \
+             'document. Indeed, only styles that are in use within your ' \
+             'template are actually recorded into the document. You may ' \
+             'consult the list of available styles programmatically by ' \
+             'calling your pod renderer\'s "getStyles" method.'
+P_ODT_TXT  = 'For XHTML element "%s", you must associate a paragraph-wide ' \
+             'OpenDocument style. "%s" is a "text" style (that applies to ' \
+             'only a chunk of text within a paragraph).'
+TXT_ODT_P  = 'For XHTML element "%s", you must associate an OpenDocument ' \
+             '"text" style (that applies to only a chunk of text within a ' \
+             'paragraph). "%s" is a paragraph-wide style.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Properties:
@@ -69,7 +67,7 @@ class Properties:
 class TableProperties(Properties):
     '''In a styles mapping, the value @key "table" must be an instance of this
        class.'''
-    defaultMargins = (0.0, 0.0, 0.0, 0.0)
+    defaultMargins = (0.3, 0.0, 0.3, 0.0)
     columnModifiersPrefixes = {'optimize': 'OCW', 'distribute': 'DC'}
 
     def __init__(self, pageWidth=None, px2cm=px2cm, cellPx2cm=10.0,
@@ -175,14 +173,14 @@ class TableProperties(Properties):
             # Get the value from CSS attributes
             cssValue = getattr(attrs, 'margin%s' % direction, None)
             if cssValue: cssValue = cssValue.cm(formatted=False)
-            # Get the the value as defined on p_self
+            # Get the value as defined on p_self
             tbValue = self.margins[i]
             # Choose the prevailing value
             if not self.prevails:
-                value = cssValue or tbValue
+                value = cssValue if cssValue is not None else tbValue
             else:
-                value = tbValue or cssValue
-            if not value: continue
+                value = tbValue if tbValue is not None else cssValue
+            if value is None: continue
             # Determine the name of the corresponding ODF property
             name = 'fo:margin-%s' % direction
             r += ' %s="%.2fcm"' % (name, value)
@@ -222,7 +220,7 @@ class ListProperties(Properties):
     def __init__(self, levels, formats, delta, firstDelta, space, paraStyle):
         # The number of indentation levels supported
         self.levels = levels
-        # The list of formats for bullets/numbers
+        # The list of format characters for bullets or numbers
         self.formats = formats
         # The number of inches to increment at each level (as a float)
         self.delta = delta
@@ -233,7 +231,7 @@ class ListProperties(Properties):
         self.space = space
         # A specific style to apply to the inner paragraphs
         self.paraStyle = paraStyle
-        # The number of levels can > or < to the number of formats. In those
+        # The number of levels can be > or < to the number of formats. In those
         # cases, formats will be applied partially or cyclically to levels.
 
     def dumpStyle(self, name):
@@ -437,7 +435,7 @@ class PageStyle:
 
     def rename(self):
         '''Give to this style a new, unique name'''
-        self.uniqueName = getUuid(removeDots=True)
+        self.uniqueName = getUuid(prefix='PS', removeDots=True)
         # Update attributes
         for name in PageStyle.nameAttributes:
             if name in self.attrs:
@@ -772,14 +770,14 @@ class StylesParser(OdfParser):
             e.currentStyle = style
 
         elif elem == e.tags['page-layout']:
-            e.state = PARSING_PAGE_LAYOUT
+            e.state = PARSING_P_LAYOUT
             pageLayout = PageLayout(attrs[e.tags['name']])
             # Record this page layout in the environment
             e.pageLayouts[pageLayout.name] = pageLayout
             e.currentPageLayout = pageLayout
 
         elif elem == e.tags['master-styles']:
-            e.state = PARSING_MASTER_STYLES
+            e.state = PARSING_M_STYLES
 
         elif e.state == PARSING_STYLE:
             # Find properties within this style definition
@@ -788,12 +786,12 @@ class StylesParser(OdfParser):
                 if fontSizeAttr in attrs:
                     e.currentStyle.setFontSize(attrs[fontSizeAttr])
 
-        elif e.state == PARSING_PAGE_LAYOUT:
+        elif e.state == PARSING_P_LAYOUT:
             # Find properties within this page layout definition
             if elem == e.tags['page-layout-properties']:
                 e.currentPageLayout.setProperties(e, attrs)
 
-        elif e.state == PARSING_MASTER_STYLES:
+        elif e.state == PARSING_M_STYLES:
             # I am parsing section "master-styles"
             if elem == e.tags['master-page']:
                 plnAttr = e.tags['page-layout-name']
@@ -871,7 +869,7 @@ class Css2odf:
     notNegInTable = 'self.inTable() and (val < 0)'
     ignore = {
       # Ignore negative text-indent and margin-left attributes within tables
-      'textindent': notNegInTable, 'marginleft': notNegInTable
+      'textindent': notNegInTable, 'marginleft': notNegInTable,
     }
 
     def __init__(self):
@@ -899,7 +897,7 @@ class Css2odf:
             return
         # Standardize the attribute for use within an ODF document
         unit = value.unit or ''
-        if unit:
+        if unit and (unit != '%'):
             ratio = self.px2cmRatios.get(name, px2cm)
             try:
                 val = value.cm(value=val, ratio=ratio)
@@ -929,7 +927,7 @@ class Css2odf:
         return (name in self.combined) and (' ' in value.value)
 
     def inTable(self):
-        '''In the XHTML > ODT convertion, are we currently parsing a table ?'''
+        '''In the XHTML > ODT conversion, are we currently parsing a table ?'''
         parser = self.xhtmlParser
         return parser and parser.env.currentTables
 
@@ -1025,6 +1023,8 @@ class StylesGenerator:
         self.stylesManager = stylesManager
         # Allows to convert CSS to ODF attributes
         self.css2odf = Css2odf()
+        # Are we generating an ODT or ODS file ?
+        self.isOds = stylesManager.renderer.templateType == 'ods'
 
     def addStyle(self, style, target):
         '''Adds the style definition dynamic styles'''
@@ -1135,10 +1135,12 @@ class StylesGenerator:
         family = self.getStyleFamily(xhtmlElem)
         style = '<style:style style:name="%s" style:family="%s"%s>%s%s' \
                 '</style:style>' % (styleName,family,parent,textProps,paraProps)
-        # I could have added this style in content.xml. But in some cases it
-        # does not work properly. For example, a percentage value for attribute
+        # Within an ODT file, injecting styles in content.xml does not work
+        # properly in some cases. For example, a percentage value for attribute
         # "fo-font-size" will be ignored if the style is dumped in content.xml.
-        self.addStyle(style, 'styles_base')
+        # Inversely, in an ODS file, it is preferable to dump everything in
+        # content.xml.
+        self.addStyle(style, 'content' if self.isOds else 'styles_base')
         # If p_add is True, also add the style to styleManager's defined styles
         # (as a Style instance). Else, it won't be found when it will need to be
         # applied on an inner paragraph.
@@ -1267,12 +1269,10 @@ class StylesManager:
     def checkStylesAdequation(self, htmlStyle, odtStyle):
         '''Checks that p_odtStyle may be used for style p_htmlStyle'''
         if (htmlStyle in XHTML_PARA_TAGS) and (odtStyle in self.textStyles):
-            raise PodError(
-                HTML_PARA_ODT_TEXT % (htmlStyle, odtStyle.displayName))
+            raise PodError(P_ODT_TXT % (htmlStyle, odtStyle.displayName))
         if (htmlStyle in XHTML_INNER_TAGS) and \
             (odtStyle in self.paragraphStyles):
-            raise PodError(HTML_TEXT_ODT_PARA % (
-                htmlStyle, odtStyle.displayName))
+            raise PodError(TXT_ODT_P % (htmlStyle, odtStyle.displayName))
 
     def addStyleEntry(self, stylesMapping, key, value, cssAttrs):
         '''Adds, in dict p_stylesMapping (the output of m_checkStylesMapping
@@ -1294,12 +1294,14 @@ class StylesManager:
     def checkStylesMapping(self, stylesMapping):
         '''Checks that the given p_stylesMapping is correct, and returns the
            internal representation of it.'''
+
         # p_stylesMapping is a dict.
         # ----------------------------------------------------------------------
         # Every key can be:
         # ----------------------------------------------------------------------
-        # (1) | the name of a XHTML 'paragraph-like' tag (p, h1, h2...) or
-        #     | "para", representing "p", "div", "blockquote" or "address";
+        # (1) | the name of a XHTML 'paragraph-like' tag (p, h1, h2...) or the
+        #     | meta-name "para", representing "p", "div", "blockquote" or
+        #     | "address";
         # (2) | the name of a XHTML 'text-like' tag (span, b, i, em...);
         # (3) | the name of a CSS class;
         # (4) | string 'h*';
@@ -1318,13 +1320,13 @@ class StylesManager:
         #     | with outline-level=3, etc.). Note that this value can also be
         #     | negative;
         # ----------------------------------------------------------------------
-        # (c) | if key is "table", the value must be a TableProperties instance
-        #     | (this class is defined hereabove);
+        # (c) | if key is "table", the value must be an instance of
+        #     | appy.pod.styles_manager.c_TableProperties;
         # ----------------------------------------------------------------------
-        # (d) | if key is "ol", the value must be an instance of the hereabove-
-        #     | defined NumberedProperties class; if key is "ul", the value must
-        #     | be an instance of the hereabove-defined BulletedProperties
-        #     | class.
+        # (d) | if key is "ol", the value must be an instance of
+        #     | appy.pod.styles_manager.c_NumberedProperties;
+        #     | if key is "ul", the value must be an instance of
+        #     | appy.pod.styles_manager.c_BulletedProperties.
         # ----------------------------------------------------------------------
         # Some precisions now about about keys. If key is (1) or (2), parameters
         # can be given between square brackets. Every such parameter represents
@@ -1384,10 +1386,10 @@ class StylesManager:
         res = {}
         if not isinstance(stylesMapping, dict) and \
            not isinstance(stylesMapping, UserDict):
-            raise PodError(MAPPING_NOT_DICT)
+            raise PodError(MAP_KO)
         for xhtmlStyleName, odtStyleName in stylesMapping.items():
             if not isinstance(xhtmlStyleName, str):
-                raise PodError(MAPPING_KEY_NOT_STRING)
+                raise PodError(KEY_STR_KO)
             # Separate CSS attributes if any
             cssAttrs = None
             if '[' in xhtmlStyleName:
@@ -1412,16 +1414,15 @@ class StylesManager:
                 if utils.commercial: raise utils.CommercialError()
                 vType = StylesManager.mappingValueTypes[xhtmlStyleName]
                 if not isinstance(odtStyleName, vType):
-                    raise PodError(MAPPING_WRONG_VALUE_TYPE % \
-                                   (xhtmlStyleName, vType.__name__))
+                    raise PodError(VAL_TYP_KO % (xhtmlStyleName,vType.__name__))
             else:
                 if not isinstance(odtStyleName, str):
-                    raise PodError(MAPPING_ELEM_NOT_STRING % xhtmlStyleName)
+                    raise PodError(VAL_STR_KO % xhtmlStyleName)
                 if not xhtmlStyleName or not odtStyleName:
-                    raise PodError(MAPPING_ELEM_EMPTY)
+                    raise PodError(VAL_EMPTY)
             if xhtmlStyleName in XHTML_UNSTYLABLE_TAGS:
-                raise PodError(UNSTYLABLE_TAG % (xhtmlStyleName,
-                                                 XHTML_UNSTYLABLE_TAGS))
+                raise PodError(UNSTYL_TAG % (xhtmlStyleName,
+                                             XHTML_UNSTYLABLE_TAGS))
             # Add the entry in the result
             if xhtmlStyleName not in StylesManager.mappingValueTypes:
                 odtStyle = self.styles.getStyle(odtStyleName)
@@ -1429,7 +1430,7 @@ class StylesManager:
                     if odtStyleName in self.podSpecificStyles:
                         odtStyle = self.podSpecificStyles[odtStyleName]
                     else:
-                        raise PodError(STYLE_NOT_FOUND % odtStyleName)
+                        raise PodError(STYLE_N_F % odtStyleName)
                 self.checkStylesAdequation(xhtmlStyleName, odtStyle)
                 odtTarget = odtStyle
             else:
