@@ -223,7 +223,9 @@ class KeepWithNext(Visitor):
         # Set attributes "keeprows" on "table1" and "table2"
         i = tbody1.children.index(self.splitRow)
         table1.addAttribute('keeprows', ':%d' % (headerRows+i))
+        table1.addAttribute('style', 'margin-bottom:0px')
         table2.addAttribute('keeprows', '%d:' % i)
+        table2.addAttribute('style', 'margin-top:0px')
         # Insert table2 just after table1
         parent = table1.parent
         table2.setParent(parent, at=parent.children.index(table1)+1)
