@@ -3,7 +3,7 @@
    Python and XML.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -26,6 +26,12 @@ from appy.pod.buffers import MemoryBuffer
 from appy.xml import xmlPrologue, xhtmlPrologue
 from appy.px.parser import PxParser, PxEnvironment
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+SPEC_KO      = 'wrong specifier. A specifier must be of the form ' \
+               '"<objectName>*<pxName>", like in "tool*pxTest".'
+SPEC_NAME_KO = 'name "%s" does not correspond to any object in the context.'
+PX_NAME_KO   = 'attribute "%s.%s" does not exist or is not a Px instance.'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Px:
     '''Represents a (chunk of) PX code'''
@@ -93,7 +99,7 @@ class Px:
         self.css = self.compact(css)
         self.js = self.compact(js)
         # A PX can have a name
-        self.name = name
+        self.name = name or ''
         # A PX can be profiled (see m_profile below)
         self.profiler = None
         # Parse the PX
@@ -167,14 +173,20 @@ class Px:
         '''
         # Developer, forget the following line
         if '_ctx_' not in context: context['_ctx_'] = context
+
         # Include, in the context, variable names for "reserved" PX chars
         if 'PIPE' not in context:
             context['PIPE'] = '|'
             context['SEMICOLON'] = ';'
-        # Also include the PX itself and remember the previously running PX
+
+        # The first PX to be called will be stored as key "_px_", while the
+        # currently called (sub-)PX will be stored at key "_cpx_" ("c" stands
+        # for "current").
         px = context.get('_px_')
         if not isTemplate:
-            context['_px_'] = self
+            if px is None: # This is the first PX to be called
+                context['_px_'] = self
+            context['_cpx_'] = self
 
         # This PX call is probably one among a series of such calls, sharing the
         # same p_context. Within this context, we add a sub-dict at key "rt"
@@ -222,7 +234,7 @@ class Px:
             # Stop profiling when relevant
             if profiler: profiler.leave()
         # Restore the previous PX in the context
-        if px: context['_px_'] = px
+        if px: context['_cpx_'] = px
         # Return the result
         return r
 
@@ -271,13 +283,15 @@ class Px:
         return r
 
     @classmethod
-    def injectRequest(class_, context, req, tool):
-        '''Inject all p_req(uest) values into the PX p_context'''
-        # Inject every (key, value) pair found in the request into the context,
-        # with automatic type conversion for values.
-        for name, value in req.items():
-            setattr(context, name, class_.convertValue(value, tool))
-        # When some keys are found in the request, create some related,
+    def injectRequest(class_, context, req, tool, specialOnly=False):
+        '''Inject all p_req(uest) values into the PX p_context (or only
+           search-related entries if p_searchOnly is True).'''
+        if not specialOnly:
+            # Inject every (key, value) pair found in the request into the
+            # context, with automatic type conversion for values.
+            for name, value in req.items():
+                setattr(context, name, class_.convertValue(value, tool))
+        # When some special keys are found in the request, create related,
         # additional entries in the PX context. These keys correspond to base
         # Appy concepts that are regularly transmitted between the browser and
         # the server.
@@ -338,6 +352,19 @@ class Px:
         for key in class_.contextKeys:
             context[key] = other[key]
 
+    # Names of the PXs being Appy home pages. Key indicates if "public" pages
+    # are taken into account or not.
+    appyHomePXs = {
+      True:  {'home':None, 'homes':None, 'public':None},
+      False: {'home':None, 'homes':None}
+    }
+
+    @classmethod
+    def isHome(class_, ctx, orPublic=True):
+        '''Returns True if the currently shown PX has the name of one of the
+           Appy home PXs.'''
+        return ctx._px_.name in class_.appyHomePXs[orPublic]
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Utility methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -351,11 +378,44 @@ class Px:
         return value
 
     @staticmethod
-    def truncateText(text, width=20, suffix='...'):
+    def truncateText(text, width=20, suffix='...', tag='abbr'):
         '''Truncates p_text to max p_width chars. If the text is longer than
            p_width, the truncated part is put in a "abbr" xhtml tag.'''
         width = width or 20 # p_width could be None
         if len(text) > width:
-            text = '<abbr title="%s">%s%s</abbr>' % (text, text[:width], suffix)
+            text = '<%s title="%s">%s%s</%s>' % \
+                   (tag, text, text[:width], suffix, tag)
         return text
+
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # Call a PX from a "specifier"
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    @classmethod
+    def callFromSpec(class_, specifier, o):
+        '''Calls the PX mentioned in p_specifier, with the context defined on
+           p_o.traversal.context.'''
+
+        # A specifier is of the form <name>*<px>.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # <name> | is a name that must be present in the PX context, pointing
+        #        | to an object, module, class...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        #  <px>  | is the name of the PX instance that must exist as attribute
+        #        | on the object denoted by <name>.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+        # Check if the p_specifier is valid
+        parts = specifier.split('*', 1)
+        if len(parts) != 2: raise class_.Error(SPEC_KO)
+        # Does the named object exist in the context ?
+        name, px = parts
+        ctx = o.traversal.context
+        if name not in ctx: raise class_.Error(SPEC_NAME_KO % name)
+        # Does the named PX exist on the object ?
+        obj = ctx[name]
+        px = getattr(obj, px, None)
+        if (px is None) or not isinstance(px, Px):
+            raise class_.Error(PX_NAME_KO % (name, parts[1]))
+        return px(ctx)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
