@@ -1,7 +1,7 @@
 '''Management of the Appy authentication cookie'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -50,7 +50,7 @@ class Cookie:
         return login, password, context
 
     @classmethod
-    def write(class_, handler, login, password, ctx):
+    def write(class_, handler, login, password, ctx=None):
         '''Encode p_login, p_password and p_ctx into the authentication
            cookie.'''
         r = '%s:%s:%s' % (login, password, ctx or '')
