@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,10 +16,13 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from http import HTTPStatus
+
 from appy.px import Px
 from appy.ui.js import Quote
 from appy.utils import Traceback
 from appy.ui.template import Template
+from appy.utils import MessageException
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Error:
@@ -29,31 +32,53 @@ class Error:
      # Error 404: page not found
      404: {'label': 'not_found', 'px':
        Px('''<div>::msg</div>
-           <div if="not isAnon and not popup">
-            <a href=":tool.computeHomePage()">:_('app_home')</a>
-           </div>''', template=Template.px, hook='content')},
+             <div if="not isAnon and not popup">
+              <a href=":tool.computeHomePage()">:_('app_home')</a>
+             </div>''', template=Template.px, hook='content')},
 
      # Error 403: unauthorized
      403: {'label': 'unauthorized', 'px':
-       Px('''<div><img src=":url('password.svg')" class="iconM"
-                       style="margin-right: 5px"/><x>::msg</x></div>''',
-          template=Template.px, hook='content')},
+       Px('''<div><img src=":url('password.svg')" class="iconERR"/>
+                  <x>::msg</x></div>''', template=Template.px, hook='content')},
 
      # Error 500 server error
      500: {'label': 'server_error', 'px':
-       Px('''<div><img src=":url('warning.svg')" class="iconM"
-                       style="margin-right: 5px"/><x>::msg</x></div>''',
-          template=Template.px, hook='content')},
+       Px('''<div><img src=":url('warning.svg')" class="iconERR"/>
+                  <x>::msg</x></div>''', template=Template.px, hook='content')},
 
      # Error 503: service unavailable. This error code is used by Appy when, due
      # to too many conflict errors (heavy load), the server cannot serve the
      # request.
      503: {'label': 'conflict_error',
-         'px': Px('''<div>::msg</div>''', template=Template.px, hook='content')}
+           'px': Px('''<div>::msg</div>''',
+                    template=Template.px, hook='content')},
+
+     # Return code 200: a logical error rendering a nice translated message to
+     # the user, but not considered being an error at the HTTP level.
+     200: {'px': Px('''<img src=":url('warning.svg')" class="iconERR"/>
+                       <x>::msg</x>''', template=Template.px, hook='content')}
     }
 
     @classmethod
-    def get(class_, resp, traversal):
+    def getContent(class_, traversal, code, error):
+        '''Get the textual content to render in the error page'''
+        # 
+        if traversal.user.hasRole('Manager') and (code != 200):
+            r = Traceback.get(html=True)
+        else:
+            if isinstance(error, MessageException):
+                r = str(error)
+            else:
+                _ = traversal.handler.tool.translate
+                r = _(Error.byCode[code]['label'])
+                if error:
+                    text = str(error).strip()
+                    if text:
+                        r += '<div class="discreet">%s</div>' % text
+        return r
+
+    @classmethod
+    def get(class_, resp, traversal, error=None):
         '''A server error just occurred. Try to return a nice error page. If it
            fails (ie, the error is produced in the main PX template), dump a
            simple traceback.'''
@@ -62,20 +87,21 @@ class Error:
         handler.commit = False
         # Log the error
         code = resp.code if resp.code in Error.byCode else 500
-        handler.log('app', 'error', message='%s on %s' % (code, handler.path))
+        # Message exceptions are special errors being 200 at the HTTP level
+        is200 = code == 200
+        message = '%d on %s' % (code, handler.path)
+        if error and error.args:
+            message = '%s - %s' % (message, str(error))
+        handler.log('app', 'warning' if is200 else 'error', message=message)
         if code in (500, 503):
             handler.log('app', 'error', Traceback.get().strip())
-        # For an admin, show a traceback. For normal humans, show a readable
-        # error message.
-        if traversal.user.hasRole('Manager'):
-            content = Traceback.get(html=True)
-        else:
-            content = handler.tool.translate(Error.byCode[code]['label'])
+        # Compute the textual content that will be shown
+        content = Error.getContent(traversal, code, error)
         # If we are called by an Ajax request, return only the error message,
         # and set the return code to 200; else, browsers will complain.
         context = traversal.context
         if context and context.ajax:
-            resp.code = 200
+            resp.code = HTTPStatus.OK
             return '<p>%s</p>' % content
         # Return the PX corresponding to the error code. For rendering it, get
         # the last PX context, or create a fresh one if there is no context.
@@ -85,6 +111,13 @@ class Error:
             req = handler.req
             if 'search' in req: del(req.search)
             traversal.context = context = traversal.createContext()
+        else:
+            # Reinitialise PX counts. Indeed, we have interrupted a "normal"
+            # page rendering that has probably already executed a series of PXs.
+            # Without reinitialising counts, Appy will believe these PXs were
+            # already executed and will not include CSS and JS code related to
+            # these PXs.
+            context['_rt_'] = {}
         context.msg = content
         try:
             return Error.byCode[code]['px'](context)
