@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -351,9 +351,10 @@ class Config:
         # Remember the logins of encountered LDAP users: it is required to
         # (de)activate local users.
         ldapUsers = {} # ~{s_login: None}~
-        for letter in string.ascii_lowercase:
-            # Get all the users whose login starts with "letter"
-            filter = connector.getFilter(self.getUserFilterValues('%s*'%letter),
+        firstChars = string.ascii_lowercase + string.digits
+        for char in firstChars:
+            # Get all the users whose login starts with "char"
+            filter = connector.getFilter(self.getUserFilterValues('%s*' % char),
                                          self.userFilter)
             for baseDn in self.baseDn:
                 data = connector.search(baseDn, self.scope, filter, params)
