@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,13 +18,14 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from DateTime import DateTime
 
+from appy.database.operators import in_
+
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NON_SSO_USER = 'Non-SSO user "%s" prevents access to the SSO user having the ' \
-  'same login.'
-INVALID_ROLE = 'Role "%s" mentioned in a HTTP header is not among ' \
-  'grantable roles.'
-INVALID_GROUP = 'Group "%s" mentioned in a HTTP header was not found.'
-SSO_REACTIVATED = '%s reactivated after a new visit via the SSO reverse proxy.'
+NON_SSO_U = 'Non-SSO user "%s" prevents access to the SSO user having the ' \
+            'same login.'
+INVALID_R = 'Role "%s" mentioned in a HTTP header is not among grantable roles.'
+INVALID_G = 'Group "%s" mentioned in a HTTP header was not found.'
+SSO_REACT = '%s reactivated after a new visit via the SSO reverse proxy.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -32,12 +33,17 @@ class Config:
        single sign-on (SSO), use this class to tell Appy how and where to
        retrieve, within every HTTP request from the reverse proxy, information
        about the currently logged user: login, groups and roles.'''
+
     ssoAttributes = {'loginKey': None, 'emailKey':'email', 'nameKey': 'name',
                      'firstNameKey': 'firstName', 'fullNameKey': 'title'}
     otherPrerogative = {'role': 'group', 'group': 'role'}
+
     # The connection to a SSO server can't really be tested
     testable = False
 
+    # Empty dict
+    emptyDict = {}
+
     def __init__(self):
         # One can give a name to the SSO reverse proxy that will call us
         self.name = ''
@@ -141,7 +147,11 @@ class Config:
         # appy.server.ldap.Config.
         self.ldap = None
 
-    def init(self, tool): pass
+    def init(self, tool):
+        '''Lazy initialisation'''
+        # Lazy-initialise the sub-ldap config when present
+        if self.ldap:
+            self.ldap.init(tool)
 
     def __repr__(self):
         name = self.name or 'SSO reverse proxy'
@@ -202,8 +212,7 @@ class Config:
                 else:
                     # Force an identification failure
                     if warn:
-                        tool.log(NON_SSO_USER % login, type='warning',
-                                 noUser=True)
+                        tool.log(NON_SSO_U % login, type='warning', noUser=True)
                     return
         return login
 
@@ -273,21 +282,15 @@ class Config:
             if role in grantable:
                 user.addRole(role)
             else:
-                tool.log(INVALID_ROLE % role, type='warning', noUser=True)
+                tool.log(INVALID_R % role, type='warning', noUser=True)
 
     def setGroups(self, tool, user, groups):
         '''Puts p_user into p_groups. Ensure those p_groups exist.'''
         for login in groups:
-            if login.startswith('*'):
-                # "login" is a pattern of groups. Query all groups and link the
-                # user to those matching the pattern.
-                suffix = login[1:]
-                for group in tool.search('Group'):
-                    if group.login.endswith(suffix):
-                        group.link('users', user)
-                        if self.afterLinkGroup:
-                            self.afterLinkGroup(user, group)
-            else:
+            # Every "login" may represent a single group or, if it contains an
+            # asterisc, several groups.
+            i = login.find('*')
+            if i == -1:
                 # "login" is the login of a single group
                 group = tool.search1('Group', login=login)
                 if group:
@@ -295,7 +298,25 @@ class Config:
                     if self.afterLinkGroup:
                         self.afterLinkGroup(user, group)
                 else:
-                    tool.log(INVALID_GROUP % login, type='warning', noUser=True)
+                    tool.log(INVALID_G % login, type='warning', noUser=True)
+            else:
+                # "login" is a pattern of groups. Search all groups matching the
+                # pattern.
+                if i == 0:
+                    # Groups share the same prefix. Get them all as basis.
+                    attrs = self.emptyDict
+                    suffix = login[1:]
+                else:
+                    # Limit the search to groups having the defined prefix
+                    prefix = login[:i]
+                    attrs = {'login': in_(prefix, prefix + 'z')}
+                    suffix = '' if i == (len(login) - 1) else login[i+1:]
+                # Search groups and retain those matching the pattern
+                for group in tool.search('Group', **attrs):
+                    if group.login.endswith(suffix):
+                        group.link('users', user)
+                        if self.afterLinkGroup:
+                            self.afterLinkGroup(user, group)
 
     def extractUserInfo(self, source, isRequest=True):
         '''By default, this method extracts, from HTTP headers present in the
@@ -357,7 +378,7 @@ class Config:
         # Reactivate the user if the local copy was noted as deactivated. We
         # trust SSO.
         if user.state == 'inactive':
-            msg = SSO_REACTIVATED % user.login
+            msg = SSO_REACT % user.login
             user.log(msg, noUser=True)
             user.do('reactivate', doHistory=False, comment=msg)
 
