@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,18 +16,19 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from http import HTTPStatus
 import inspect, pathlib, mimetypes, os.path, email.utils, collections
 
 from DateTime import DateTime
 
 import appy
+from appy.utils.string import Normalize
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MAP_VALUE_NOT_PATH = 'Values from the map must be pathlib.Path objects.'
 PATH_NOT_FOUND     = 'Path "%s" was not found or is not a folder.'
 RAM_ROOT_IN_MAP    = 'Ram root "%s" is also used as key for ' \
                      'appy.server.static.Config.map.'
-BROKEN_PIPE        = 'Broken pipe while serving %s.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -103,62 +104,31 @@ class Config:
 class Static:
     '''Class responsible for serving static content'''
 
-    # We will write in the socket per bunch of BYTES bytes
-    BYTES = 50000
-
     # The dict of RAM resources (see doc in class Config hereabove)
     ram = collections.OrderedDict()
 
-    @classmethod
-    def initResponse(class_, handler, code, size=0, end=False):
-        '''Set code and default headers in the HTTP response'''
-        handler.send_response(code)
-        handler.send_header('Connection', 'close')
-        handler.send_header('Content-Length', size)
-        if end: handler.end_headers()
-
     @classmethod
     def notFound(class_, handler, config):
         '''Raise a HTTP 404 error if the resource defined by p_handler.parts was
            not found.'''
-        code = 404
         path = '/%s/%s' % (config.root, '/'.join(handler.parts))
-        handler.log('app', 'error', '%d@%s' % (code, path))
-        class_.initResponse(handler, code, end=True)
-        return code
+        code = HTTPStatus.NOT_FOUND # 404
+        handler.log('app', 'error', '%d @ %s' % (code.value, path))
+        resp = handler.resp
+        resp.code = code
+        resp.build()
 
     @classmethod
-    def writeToSocket(class_, handler, path, content):
-        '''Write the file content to the socket, either from disk (p_path) or
-           from RAM (p_content).'''
-        if content is not None:
-            handler.wfile.write(content)
-        else:
-            with open(path, 'rb') as f:
-                while True:
-                    chunk = f.read(Static.BYTES)
-                    if not chunk: break
-                    try:
-                        handler.wfile.write(chunk)
-                    except BrokenPipeError:
-                        handler.log('app', 'error', BROKEN_PIPE % path)
-
-    @classmethod
-    def writeUnchanged(class_, handler, response=None):
+    def writeUnchanged(class_, handler):
         '''Returns the ad-hoc response if the file has not changed since the
            last time the browser has downloaded it.'''
-        code = 304
-        class_.initResponse(handler, code)
-        if response:
-            response.build(complete=False)
-            response.code = code
-        handler.end_headers()
-        return code
+        resp = handler.resp
+        resp.code = HTTPStatus.NOT_MODIFIED # 304
+        resp.build()
 
     @classmethod
-    def write(class_, handler, path, modified, content=None, response=None,
-              fileInfo=None, disposition='attachment', downloadName=None,
-              enableCache=True):
+    def write(class_, handler, path, modified, content=None, fileInfo=None,
+              disposition='attachment', downloadName=None, enableCache=True):
         '''Serves, to the browser, the content of the file whose path on disk is
            given in p_path or whose content in RAM is given in p_content.'''
         # ~~~
@@ -166,15 +136,6 @@ class Static:
         # asked it, return an empty response with code 304 "Not Modified". Else,
         # return file content with a code 200 "OK".
         # ~~~
-        # If p_response is there, this method is used by the "dynamic serving"
-        # part of Appy to serve a file, either originating from the database-
-        # controlled filesystem via a File field, or a dynamically-computed file
-        # produced, for instance, by a Pod field. In those cases, we retrieve
-        # the p_response elements collected so far (headers, cookies and
-        # messages) and inject them in the specific response built by this
-        # method. p_response will be flagged as "already built" to prevent the
-        # handler from doing it once more.
-        # ~~~
         # If p_path corresponds to a DB-controlled file, his corresponding
         # p_fileInfo is given. In that case, p_disposition will be taken into
         # account. Else, it will be ignored, unless p_downloadName is specified.
@@ -185,50 +146,34 @@ class Static:
         # ~~~
         browserDate = handler.headers.get('If-Modified-Since')
         modified = fileInfo.modified if fileInfo else modified
-        smodified = email.utils.formatdate(modified) # RFC 822 string
+        smodified = email.utils.formatdate(modified, usegmt=True) # RFC 822
         if not browserDate or (smodified > browserDate):
-            code = 200
-            # Compute content length
-            hasContent = content is not None
-            if hasContent: content = content.encode()
-            if fileInfo:
-                size = fileInfo.size
-            else:
-                size = len(content) if hasContent else os.path.getsize(path)
-            # Initialise response headers
-            class_.initResponse(handler, code, size=size)            
+            resp = handler.resp
+            resp.code = HTTPStatus.OK
             # Identify MIME type
+            set = resp.setHeader
             mimeType, encoding = mimetypes.guess_type(path)
             mimeType = mimeType or 'application/octet-stream'
-            set = handler.send_header
             set('Content-Type', mimeType)
             # Define content disposition
             if fileInfo or downloadName:
                 niceName = downloadName or fileInfo.uploadName
                 disp = '%s;filename="%s"' % (disposition, niceName)
-                try:
-                    set('Content-Disposition', disp)
-                except UnicodeEncodeError:
-                    # BaseHTTPRequestHandler requires latin-1 encoding
-                    set('Content-Disposition', disp.encode().decode('latin-1'))
-            # For now, disable byte serving (value "bytes" instead of "none")
-            set('Accept-Ranges', 'none')
+                set('Content-Disposition', disp)
             # ~~~ Manage caching ~~~
             if enableCache:
                 set('Last-Modified', smodified)
+                # Ensure there is no the cache-related header
+                resp.removeHeader('Cache-Control')
+                resp.removeHeader('Expires')
             else:
                 set('Cache-Control', 'no-cache, no-store, must-revalidate')
                 set('Expires', '0')
-            # Incorporate elements from the current p_response object
-            if response:
-                response.build(complete=False)
-                response.code = code
-            handler.end_headers()
             # Write the file content to the socket
-            class_.writeToSocket(handler, path, content)
+            path = None if content else path
+            resp.build(content, path)
         else:
-            code = class_.writeUnchanged(handler, response)
-        return code
+            class_.writeUnchanged(handler)
 
     @classmethod
     def writeFromDisk(class_, handler, path,
@@ -236,15 +181,16 @@ class Static:
         '''Serve a static file from disk, whose path is p_path'''
         # The string version of p_path
         spath = str(path)
-        return class_.write(handler, spath, os.path.getmtime(spath),
-                            disposition=disposition, downloadName=downloadName)
+        class_.write(handler, spath, os.path.getmtime(spath),
+                     disposition=disposition, downloadName=downloadName)
 
     @classmethod
     def writeFromRam(class_, handler, config):
         '''Serve a static file loaded in RAM, from dict Static.ram'''
         # p_handler.parts contains something starting with ['ram', ...]
         if len(handler.parts) == 1:
-            return class_.notFound(handler, config)
+            class_.notFound(handler, config)
+            return
         # Re-join the splitted path to produce the key allowing to get the file
         # content in Static.ram.
         key = '/'.join(handler.parts[1:])
@@ -259,8 +205,9 @@ class Static:
         #                      "a/b/c/some.css"
         content = class_.ram.get(key)
         if content is None:
-            return class_.notFound(handler, config)
-        return class_.write(handler, key, config.created, content=content)
+            class_.notFound(handler, config)
+            return
+        class_.write(handler, key, config.created, content=content)
 
     @classmethod
     def get(class_, handler):
@@ -276,18 +223,21 @@ class Static:
                 # We are at the root of the search: "part" must correspond to
                 # the RAM root or to a key from config.map.
                 if part == config.ramRoot:
-                    return class_.writeFromRam(handler, config)
+                    class_.writeFromRam(handler, config)
+                    return
                 elif part in config.map:
                     path = config.map[part]
                 else:
-                    return class_.notFound(handler, config)
+                    class_.notFound(handler, config)
+                    return
             else:
                 path = path / part
                 if not path.exists():
-                    return class_.notFound(handler, config)
+                    class_.notFound(handler, config)
+                    return
         # We have walked the complete path: ensure it is a file
         if not path or not path.is_file():
             return class_.notFound(handler, config)
-        # Read the file content and return it
-        return class_.writeFromDisk(handler, path)
+        # Read the file content and write it in the HTTP response
+        class_.writeFromDisk(handler, path)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
