@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -59,6 +59,15 @@ class Traversal:
     # "Traversing" thus finally produces a result as a page that is returned to
     # the browser.
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    # When the traversal is in the default, "standard" mode, fields are supposed
+    # to be related to the currently walked object. If the traversal switches to
+    # "static" mode, fields are supposed to run independently of any instance.
+    # A switch from standard to static mode occurs as soon as a @-prefixed part
+    # is encountered.
+    STANDARD = 0
+    STATIC   = 1
+
     def __init__(self, handler=None, other=None):
         # A traversal can be initialised from another one
         if other:
@@ -84,6 +93,8 @@ class Traversal:
         self.context = None
         # The parent traversal, for a child traversal
         self.parent = None
+        # The traversal mode
+        self.mode = Traversal.STANDARD
 
     def init(self, other):
         for name, value in other.__dict__.items():
@@ -196,9 +207,14 @@ class Traversal:
         else:
             # No: determine default "traverse info" based on p_current's type
             if isinstance(current, Field):
-                # One may traverse it if he has the read permission on the
-                # related object.
-                r = 'perm:%s' % current.readPermission
+                if self.mode == Traversal.STANDARD:
+                    # One may traverse it if he has the read permission on the
+                    # related object.
+                    r = 'perm:%s' % current.readPermission
+                else:
+                    # The field is not related to any instance. At this
+                    # traversal level, no more security check can be performed.
+                    r = True
             elif isinstance(current, Transition):
                 # The transition is traversable if triggerable
                 r = current.isTriggerable(previous)
@@ -209,6 +225,13 @@ class Traversal:
                 # Any other element (method, attribute, property...) is not
                 # traversable by default.
                 r = False
+        # If we are walking an element of a Field in static mode, and this
+        # element is protected by an instance-related info (=a permission),
+        # convert this info to True: indeed, in that case, permission can't be
+        # checked (we have no instance).
+        if (self.mode == Traversal.STATIC) and isinstance(previous, Field) and \
+           isinstance(r, str) and r.startswith('perm:'):
+            r = True
         return r
 
     def executeMethod(self, method):
@@ -327,6 +350,8 @@ class Traversal:
                 if part and part.startswith('@'):
                     # Manage direct navigation to a special object
                     self.r = self.getSpecialObject(part[1:])
+                    # Switch to "static" mode
+                    self.mode = Traversal.STATIC
                 elif self.r is None:
                     # If we are at the start of the traversal, and "r" is still
                     # None, "part" must correspond to an object ID.
