@@ -1,7 +1,7 @@
 '''User-interface module'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -31,6 +31,7 @@ from appy.ui.iframe import Iframe
 from appy.ui.message import Message
 from appy.ui.portlet import Portlet
 from appy.ui.globals import Globals
+from appy.ui.template import Template
 from appy.ui.navigate import Siblings
 from appy.ui.includer import Includer
 from appy.ui.language import Language
@@ -39,56 +40,152 @@ from appy.ui.validate import Validator
 # Some elements in this module will be traversable - - - - - - - - - - - - - - -
 traverse = {'Language': True}
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# In the UI config as defined hereafter, several attributes will hold the path
-# to a static resource (image, CSS file, etc) coming from Appy itself, from your
-# app or ext. In your app/ext, if you use the standard Appy conventions, any
-# static resource is located in its "static" sub-folder. If you want to refer to
-# such a static resource from your app/ext, use a string of the form
-#
-#                "<app or ext name>/<name of the static resource>"
-#
-# Here is an example. Suppose you app is named "myApp". In myApp/static, you
-# have an image name "myBackground.png" that you want to use as background image
-# for the home page.
-#
-# In your UI Config instance, you must specify, for attribute "homeBackground",
-#
-#                         "myApp/myBackground.png"
-#
-# Resources coming from Appy itself follow the same syntax, with one difference:
-# the <app name> part is replaced with "appy".
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
     '''Represents user-interface configuration for your app'''
 
     # Defaults fonts used in the web user interface
-    fonts = "Rajdhani, sans-serif"
+    defaultFonts = "Rajdhani, sans-serif"
 
     # Regular expression for variables defined in CSS file templates
     cssVariable = re.compile('\|(\w+?)\|', re.S)
 
     def __init__(self):
-        # For specifying paths to static resources, consult the doc at the start
-        # of this file.
-        # ~~~
-        # The home page's background image
-        self.homeBackground = 'appy/sunrise.jpg'
-        # The logo shown on the home page, in the top left corner
-        self.homeLogo = None
-        # The logo within the page header
-        self.headerLogo = None
-        # The logo at the top of the portlet
-        self.portletLogo = 'appy/portletLogo.png'
-        # The following attributes determines when the header and footers must
-        # be shown. If you place a function in it, it will be called with, as
-        # args, the current PX and its context, and must return True
-        # whenever the corresponding UI element must be shown.
-        self.headerShow = True
+        '''Constructor for the UI configuration'''
+
+        # For any standard image provided by Appy (background images, icons...),
+        # if you want to use an alternate image provided by your app or ext:
+        # - create, in your app's or ext's "static" folder, an image having the
+        #   same name and extension as the Appy file you want to override,
+        #   located in appy/ui/static;
+        # - add, in dict "images" hereafter, an entry whose key is the name of
+        #   the image and whose value is the name of the app or ext whose
+        #   "static" folder stores your variant.
+        
+        # For example, suppose your app is named "MyApp" and you want to provide
+        # your own home background picture. In folder MyApp/static, create your
+        # own picture and store it under the name homeBG.jpg. Then, add the
+        # following entry into dict "images":
+        
+        #                      "homeBG.jpg" : "MyApp"
+
+        # The following table lists the names of the most frequent images you
+        # could override.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # Name            | Description
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # homeBG.jpg      | The home page's background image
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # baseBG.jpg      | The background image for every other page
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # homeLogo.png    | The logo shown on the home page for anonymous users,
+        #                 | in the top left corner, on top of the home
+        #                 | background.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # headerBG.png    | The background image for the header
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # portletLogo.png | The logo at the top of the portlet
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # loginLogo.png   | The logo at the top of the login box
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # ...             | Consult folder appy/ui/static for a complete list of
+        #                 | images to override. Do not try to override CSS or JS
+        #                 | files.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+        # This system implies that:
+        # - you are "forced" to use images having the same type and extension as
+        #   Appy standard ones. For example, because Appy uses SVG icons, your
+        #   replacement icons must be SVG, too ;
+        # - the names of the replacement images must be exactly the same as the
+        #   names of the files from appy/ui/static.
+        self.images = {}
+
+        # Name of the home page. Here are the possibilites.
+        # ----------------------------------------------------------------------
+        # "home"  | (the default one) The default home page displays the login
+        #         | box on top of a window-wide, single background image.
+        # ----------------------------------------------------------------------
+        # "homes" | This page displays the login box on top of a space divided
+        #         | in as much sections as there are defined background images
+        #         | in attribute "homesBackgrounds", each one rendered besides
+        #         | each other.
+        # ----------------------------------------------------------------------
+        self.home = 'home'
+
+        # The background images to use when the home page is "homes". Every
+        # entry looks like:
+        #
+        #            "<file_name>": ("<app_name>", "<image_width>")
+        #
+        # Here is an example, for a triptych:
+        #
+        #               {"bg1.jpg": ("MyApp", "30%"),
+        #                "bg2.jpg": ("MyApp", "40%"),
+        #                "bg3.jpg": ("MyApp", "30%")}
+        self.homesBackgrounds = {}
+
+        # Attribute "headerBackground" determines properties for the header's
+        # background image, as a tuple or list
+        # 
+        #                      (name, repetition, position)
+        #
+        # The default value as defined hereafter is appropriate for placing a
+        # logo at the center of the header. Because there may be several
+        # different headers (ie, a specific header may be defined on public
+        # pages), it is possible to define another name for the background
+        # image, rather than the default "headerBG.png". Attribute
+        # "headerBackground" may hold a function, accepting the current root PX
+        # and its context as args, and must return a tuple or list as descrived
+        # hereabove. If name of the background image is None or the empty
+        # string, no background will be rendered. Specifying None as global
+        # value for attribute "headerBackground" is invalid.
+        self.headerBackground = ['headerBG.png', 'no-repeat', 'center']
+
+        # The following attribute determines how the block of controls is
+        # aligned within the header. If the margin is defined to be "left", the
+        # block will be aligned to the right, and vice versa. The attribute may
+        # hold a function, accepting the current root PX and its context as
+        # args, and must return one of the above-mentioned values.
+        self.headerMargin = 'left'
+
+        # Attribute "headerShow" determines when and where the page header must
+        # be shown. Possible values are the following.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # "top" | The header will be placed on top of the page, in order to
+        #       | produce this global page schema:
+        #       |
+        #       |      H         e         a         d         e          r
+        #       |
+        #       |      Portlet   P  a  g  e   c  o  n  t  e  n  t   Sidebar
+        #       |
+        #       |      F         o         o         t         e          r
+        #       |
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # "sub" | The header will be placed on top of the page content, in
+        #       | between the portlet and sidebar, to produce this global page
+        #       | schema:
+        #       |
+        #       |      Portlet   H     e     a     d     e       r  Sidebar
+        #       |
+        #       |                P  a  g  e    c  o  n  t  e  n  t
+        #       |
+        #       |      F         o         o         t         e          r
+        #       |
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # None  | The header will be invisible
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # If you place a function in this attribute, it will be called with, as
+        # args, the current root PX and its context, and must return one of the
+        # above-mentioned values.
+        self.headerShow = lambda px, ctx: 'top' if not px.isHome(ctx) else None
+        # Attribute "footerShow" determines when the page footer must be shown.
+        # It can be a boolean value or a function. If a function is placed, it
+        # will be called with, as args, the current root PX and its context, and
+        # must return a boolean value.
         self.footerShow = False
         # Fonts in use
-        self.fonts = Config.fonts
+        self.fonts = Config.defaultFonts
         self.fontSize = '100%'
         # Among the fonts listed above, specify here, as a tuple, those being
         # Google fonts. That way, the corresponding "CSS include" will be
@@ -96,46 +193,155 @@ class Config:
         self.googleFonts = ('Rajdhani',)
         # If you want to add specific CSS classes to some standard Appy parts,
         # specify a function in the following attribute. The function will
-        # receive, as args, the name of the concerned part, the current PX and
-        # its context; it must return the name of one or more CSS classes or
-        # None when no class must be added. Currently defined parts are the
+        # receive, as args, the name of the concerned part, the current root PX
+        # and its context; it must return the name of one or more CSS classes,
+        # or None when no class must be added. Currently defined parts are the
         # following.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   body      | The page "body" tag
-        #   main      | The main zone of any page, behind the user strip
-        # ----------------------------------------------------------------------
+        #   main      | The main zone of any page
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.css = None
+        # Color used when a bright color is required
+        self.brightColor = 'white'
+        # Color used when a dark background is required
+        self.darkColor = '#002039'
+        # Alternate text color, also used for button borders and other discreet
+        # places.
+        self.altColor = '#009aa4'
+        # Variant being lighter
+        self.altColorLight = '#e9f2f3'
+        # In the popup, everything is smaller. If you want to achieve the same
+        # result in the main window, set "compact" to True.
+        self.compact = False
         # Some input fields will get this background color once they will
         # contain erroneous content.
         self.wrongTextColor = '#009ba4'
         # Text color representing a problem or warning (some flavour of red)
         self.warnColor = '#e15151'
         # Text color for home text
-        self.homeTextColor = 'white'
+        self.homeTextColor = self.brightColor
         # Text color in the header
-        self.headerColor = 'white'
+        self.headerColor = self.brightColor
         # Background color for the header
-        self.headerBgColor = '#002039'
+        self.headerBgColor = self.darkColor
         # Text color for links
-        self.linkColor = '#002039'
-        self.visitedColor = '#002039'
-        # Alternate text color, also used for button borders other discreet
-        # places.
-        self.altColor = '#009aa4'
-        # Variant being lighter
-        self.altColorLight = '#e9f2f3'
+        self.linkColor = self.darkColor
+        self.visitedColor = self.darkColor
+        # Border bottom for input fields
+        self.itextBottom = '1px solid'
+        self.itextBottomColor = '#afb8d4'
         # Color used as "fill" color, ie, for the message box
         self.fillColor = '#dab823'
-        # Header height
+        # Styling the login box
+        self.loginTitleWeight = 'normal'
+        self.loginBgColor = '#f5f3f2' 
+        self.loginColor = self.darkColor
+        self.loginColorPh = '#a9b1b7'
+        self.loginWidth = '240px'
+        self.loginPadding = '12px'
+        self.loginMargin = '8px 0'
+        self.loginBorder = '0px solid' # Background excepted
+        self.loginShadow = '0 4px 8px 0 rgba(160, 104, 132, 0.2), ' \
+                           '0 6px 20px 0 rgba(0, 0, 0, 0.19)'
+        self.loginBorderRadius = '0'
+        self.loginAlign = 'center'
+        self.loginTop = '50%' # Position of the login box on the y axis
+        # Styling the "connect" button
+        self.submitTop = '30px' # Vertical space between login/password inputs
+                                # and the "connect" button
+        self.submitColor = self.brightColor
+        self.submitAlign = 'center'
+        self.submitBgColor = self.darkColor
+        self.submitBgUrl = 'none'
+        self.submitBgRadius = '0'
+        self.submitWidth = '260px' # If you want it to have the same width as
+                                   # the login and connect fields, set a width
+                                   # being 20px more than these fields' widths.
+        self.submitHeight = 'inherit'
+        self.submitWeight = 'normal'
+        self.submitPadding = '12px'
+        self.boxBgColor = self.brightColor
+        # Portlet
+        self.portletWidth = '250px'
+        self.portletBgColor = 'transparent'
+        self.portletTextColor = self.darkColor
+        self.asMargin = '3px 0 10px 30px' # For link "*a*dvanced *s*earch"
+        # Live search (ls)
+        self.lsBgColor = 'transparent' # Bg color for the search field
+        self.lsrBgColor = self.brightColor # Bg color for search *r*esults
+        self.lsPadding = '0'
+        self.lsBottom = '1px solid' # Border bottom
+        self.lsBottomColor = '#afb8d4'
+        # Header
         self.headerHeight = '60px'
-        # Border style for tabs
-        self.tabBorder = '1px solid %s' % self.linkColor
-        # In the popup, everything is smaller. If you want to achieve the same
-        # result in the main window, set "compact" to True.
-        self.compact = False
+        # Footer
+        self.footerHeight = '20px'
+        self.footerBgColor = '#ececec'
+        self.footerBgTop = '1px solid #c4c4c4'
+        self.footerAlign = 'right'
+        self.footerFontSz = '65%'
+        # Tabs
+        self.tabSep = '1px' # Space between tabs and content
+        self.tabBorder = '1px solid %s' % self.linkColor # Top, l, r borders
+        self.tabBorderBottom = self.tabBorder # Bottom border
+        self.tabBorderRadius = '5px 5px 0 0'
+        self.tabsBorderBottom = 'unset' # Define a border bottom for tabs only
+                                        # if you define a tabSep > 0.
+        # Basically, you may produce 2 families of tabs:
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # (1) | (the default) Tabs and content are glued: tabSep is 1px, a
+        #     | border bottom is set for every tab (tabBorderBottom) and there
+        #     | is no global border bottom for all tabs (tabsBorderBottom is
+        #     | unset).
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # (2) | There is space between tabs and content. In that case, tabSep
+        #     | is > 1px, no border bottom must be set for every tab
+        #     | (tabBorderBottom must be 'unset') and a global border bottom
+        #     | must be defined (set tabsBorderBottom to tabBorder).
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # Styling phases (containing a picto per page)
+        self.phaseBgColor = self.brightColor # When unselected
+        self.phaseBgcColor = self.darkColor # When selected (*c*urrent)
+        self.phaseBorderColor = self.linkColor
+        self.phaseColor = self.linkColor # When unselected
+        self.phaseCcolor = self.brightColor # When selected (*c*urrent)
+        self.pageMargin = '0' # Margins between pages
+        # Pages' attributes, with alternate values when the UI is compact
+        # (see attribute "compact "hereabove).
+        self.pageHeight = '115px'
+        self.pageCHeight = '80px'
+        self.pageWidth = '125px'
+        self.pageCWidth = '110px'
+        self.pagePadding = '0'
+        self.pageCPadding = '0'
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # If attribute          | The name of the page 
+        # "pageDisplay" is ...  | will be rendered ...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # "block"               | under the picto ;
+        # "inline"              | besides the picto.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        self.pageDisplay = 'block'
+        # Styling block "Navigation to siblings", that appears in the phases
+        self.navsPadding = '10px 0 0 0'
+        # Size of pictos (width and height)
+        self.pictoSize = '55px'
+        self.pictoCSize = '35px'
+        self.pictoMargin = '0 0 6px 0'
+        self.pictoCMargin = '0 0 4px 0'
+        # Background color for buttons
+        self.buttonBgColor = 'transparent' # Standard buttons outside phases
+        self.sbuttonBgColor = 'transparent' # *S*mall buttons
+        # Within a page, by default, the "edit" icon has an "absolute" position.
+        # If you want to use standard positioning, set value "inherit".
+        self.epictoPosition = 'absolute'
+        self.epictoPadding = '0'
         # Application-wide default formats for hours and dates
         self.dateFormat = '%d/%m/%Y'
         self.hourFormat = '%H:%M'
+        # Fadeout duration for the message
+        self.messageFadeout = '6s'
         # Application-wide maximum results on a single page of query results
         self.maxPerPage = 30
         # Number of translations for every page on a Translation object
@@ -163,8 +369,9 @@ class Config:
         # language, will always use the forced language and will hide the
         # language selector, even if "languageSelector" hereabove is True.
         self.forcedLanguage = None
-        # Show the link to the user profile in the user strip
-        self.userLink = True
+        # When no translation is available in some language, Appy will fall back
+        # to translations in this language.
+        self.fallbackLanguage = 'en'
         # If you want to distinguish a test site from a production site, set the
         # "test" parameter to some text (lie "TEST SYSTEM" or
         # "VALIDATION ENVIRONMENT". This text will be shown on every page. This
@@ -192,9 +399,48 @@ class Config:
         # showability of the tool's icon in the UI. This function will receive
         # the tool as unique arg.
         self.toolShow = True
-        # If the following field is True, the login/password widget will be
-        # discreet. This is for sites where authentication is not foreseen for
-        # the majority of visitors (just for some administrators).
+        # Attribute "showRootPages" determines if the selector containing root
+        # pages, located in the page header, must be shown or not. It can be a
+        # function accepting the tool as unique arg.
+        self.showRootPages = True
+        # Attribute "showUserLink" determines if the link to logged user's
+        # profile, located in the page header, must be show or not. It can be a
+        # function accepting the tool as unique arg.
+        self.showUserLink = True
+        # In the page header, there is an icon allowing to go back to the user's
+        # home page, as defined by method tool.computeHomePage(). You can define
+        # the visibility of this icon via the following attribute. It can hold a
+        # function that will receive the tool as unique arg.
+        self.showHomeIcon = True
+        # When link "login" is shown (i, discreet login is enabled), must an
+        # icon be shown besides the link ?
+        self.showLoginIcon = True
+        # When "showUserLink" is True, but the following attribute "userLink" is
+        # False, the user's first name is unclickable.
+        self.userLink = True
+        # Must text "Disconnect" be renderer besides the "logout" icon ?
+        # Defaults to "no".
+        self.logoutText = False
+        # Attribute "discreetLogin" determines the app's home page and
+        # visibility of the login box. If the attribute is...
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # False   | (the default), users will hit the app on the default home
+        #         | page (see attribute "home") containing a login box inviting
+        #         | them to connect. The login box is not "discreet": 
+        #         | authentication is a prerequisite to most actions to perform
+        #         | with the app;
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # True    | users will hit the app on tool/public. The first contact
+        #         | with the app will be via public page(s); the login box will
+        #         | only be shown after the users clicks on a discreet icon (=
+        #         | the "login icon"). A click on this icon will show the login
+        #         | box as a popup.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # "home"  | similar to the previous case (True), but when users click on
+        #         | the "login icon", they are redirected to the default home
+        #         | page (see attribute "home") instead of getting the login box
+        #         | as a popup.
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.discreetLogin = False
 
     def formatDate(self, tool, date, format=None, withHour=True, language=None):
@@ -204,13 +450,13 @@ class Config:
         fmt = format or self.dateFormat
         # Resolve Appy-specific formatting symbols used for getting translated
         # names of days or months:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  %dt  | translated name of day
         #  %DT  | translated name of day, capitalized
         #  %mt  | translated name of month
         #  %MT  | translated name of month, capitalized
         #  %dd  | day number, but without leading '0' if < 10
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         if ('%dt' in fmt) or ('%DT' in fmt):
             day = tool.translate('day_%s' % date._aday, language=language)
             fmt = fmt.replace('%dt', day.lower()).replace('%DT', day)
@@ -232,37 +478,62 @@ class Config:
         if callable(r): r = test(tool)
         return r or ''
 
-    def getBackground(self, px, siteUrl, type):
+    def getBackground(self, px, ctx, type, popup=None):
         '''Return the CSS properties allowing to include this background p_image
            when appropriate.'''
         # Do not generate any background image when appropriate
-        stop = (px.name != 'home') if (type == 'home') else (px.name == 'home')
+        if px.name == 'home':
+            # Only the home background must be shown on the home page
+            stop = type != 'home'
+        elif px.name == 'public':
+            # The base or home backgrounds must not be shown on the public page
+            stop = type != 'header'
+        elif px.name == 'homes':
+            # Specific multiple backgrounds will be set outside this function
+            stop = True
+        else:
+            # On any other page, allow any background, home excepted
+            stop = type == 'home'
         if stop: return ''
-        if type == 'home':
-            # This is the background image for the home page
-            image = self.homeBackground
+        if type in ('home', 'base', 'popup'):
+            # The background image for the home page or any other page
+            image = '%sBG.jpg' % type
             attrs = 'background-size: cover'
+            repeat = 'no-repeat'
         elif type == 'header':
-            # This is the background logo for the page header
-            image = self.headerLogo
-            if not image: return ''
-            attrs = 'background-position: center'
-        return 'background-image: url(%s/static/%s); background-repeat: ' \
-               'no-repeat; %s' % (siteUrl, image, attrs)
-
-    def getImageUrl(self, siteUrl, name):
-        '''Returns the URL of the image whose relative path is in the config
-           attribute whose name is p_name.'''
-        return '%s/static/%s' % (siteUrl, getattr(self, name))
+            # The background for the page header
+            image, repeat, position = self.cget('headerBackground', ctx)
+            if not image: return
+            attrs = 'background-position: %s' % position
+        base = self.images.get(image) or 'appy'
+        return 'background-image: url(%s/static/%s/%s); background-repeat: ' \
+               '%s; %s' % (ctx.siteUrl, base, image, repeat, attrs)
 
     def _show(self, elem, px, context, popup):
-        ''''As a general rule, hide p_elem in the popup or on the home page.
-            Else, use the UI attribute defining visibility for p_elem.'''
-        if popup or (px.name == 'home'): return
+        ''''In any case, hide p_elem in the popup. In any other situation, use
+            the UI attribute defining visibility for p_elem.'''
+        #if popup or (px.name.startswith('home')): return
+        if popup: return
         # Use the corresponding config attribute
         r = getattr(self, '%sShow' % elem)
         return r(px, context) if callable(r) else r
 
+    def tget(self, name, tool):
+        '''Get, on p_self, attribute named p_name. If the attribute value is
+           callable, call it, with the p_tool as unique arg.'''
+        # "tget" stands for "get, with the *t*ool as arg".
+        r = getattr(self, name)
+        return r(tool) if callable(r) else r
+
+    def cget(self, name, ctx):
+        '''Get, on p_self, attribute named p_name. If the attribute value is
+           callable, call it, with the current root PX and its context as
+           args.'''
+        # "cget" stands for "get, with the current root PX and its *c*context
+        # as args".
+        r = getattr(self, name)
+        return r(ctx._px_, ctx) if callable(r) else r
+
     def showHeader(self, px, context, popup):
         return self._show('header', px, context, popup)
 
@@ -315,40 +586,52 @@ class Config:
 class LinkTarget:
     '''Represents information about the target of an HTML "a" tag'''
 
-    def __init__(self, class_=None, back=None, forcePopup=False):
+    def __init__(self, class_=None, back=None, popup=None, forcePopup=False):
         '''The HTML "a" tag must lead to a page for viewing or editing an
            instance of some p_class_. If this page must be opened in a popup
-           (depends on attribute p_class_.popup), and if p_back is specified,
-           when coming back from the popup, we will ajax-refresh a DOM node
-           whose ID is specified in p_back.'''
+           (depends on p_popup, if not None, or attribute p_class_.popup else),
+           and if p_back is specified, when coming back from the popup, we will
+           ajax-refresh a DOM node whose ID is specified in p_back.'''
         # The link leads to a instance of some Python p_class_
         self.class_ = class_
         # Does the link lead to a popup ?
-        toPopup = True if forcePopup else class_ and hasattr(class_, 'popup')
+        if popup or forcePopup:
+            toPopup = True
+        elif popup == False:
+            toPopup = False
+        else:
+            toPopup = class_ and hasattr(class_, 'popup')
         # Determine the target of the "a" tag
         self.target = toPopup and 'appyIFrame' or '_self'
         # If the link leads to a popup, a "onClick" attribute must contain the
         # JS code that opens the popup.
         if toPopup:
             # Create the chunk of JS code to open the popup
-            size = getattr(class_, 'popup', '350px')
+            size = popup or getattr(class_, 'popup', '350px')
             if isinstance(size, str):
                 params = "%s,null" % size[:-2] # Width only
             else: # Width and height
-                params = "%s, %s" % (size[0][:-2], size[1][:-2])
+                params = "%s,%s" % (size[0][:-2], size[1][:-2])
             # If p_back is specified, included it in the JS call
-            if back: params += ",'%s'" % back
+            if back: params += ",null,'%s'" % back
             self.onClick = "openPopup('iframePopup',null,%s)" % params
         else:
             self.onClick = ''
 
-    def getOnClick(self, back):
+    def getOnClick(self, back, o=None):
         '''Gets the "onClick" attribute, taking into account p_back DOM node ID
            that was unknown at the time the LinkTarget instance was created.'''
         # If we must not come back from a popup, return an empty string
         r = self.onClick
         if not r: return r
-        return r[:-1] + ",'%s')" % back
+        if o:
+            # Get a specific CSS class to apply to the popup
+            css = o.class_.getCssFor(o, 'popup')
+            css = css if css == 'popup' else ('%s popup' % css)
+            css = "'%s'" % css
+        else:
+            css = 'null'
+        return r[:-1] + ",%s,'%s')" % (css, back)
 
     def __repr__(self):
         return '<LinkTarget for=%s,target=%s,onClick=%s>' % \
@@ -435,8 +718,7 @@ class Sidebar:
       <x var="layout='view'">:o.pxFields</x>
      </div>''',
 
-     css='''.sidebar { padding: 28px 8px 30px 0px;
-                       position: sticky; position: -webkit-sticky; top: 0;
+     css='''.sidebar { padding: 28px 8px 30px 0px; position: sticky; top: 0;
                        overflow-y:auto; overflow-x: auto }''')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -526,13 +808,13 @@ class Footer:
     '''Footer for all (non-popup) pages'''
 
     px = Px('''<div class="footer">
-     <span class="footerContent">::_('footer_text')</span></div>''',
+     <div class="footerContent">::_('footer_text')</div></div>''',
 
      css='''
-      .footer { width:100%; font-size: 95%; height: 20px; padding: 0.8em 0;
-                position: fixed; bottom: 0; background-color: #CBCBC9;
-                border-top: 3px solid #e4e4e4; text-align:right }
-      .footerContent { padding: 0 0.5em }''')
+      .footer { width:100%; height:|footerHeight|; text-align:|footerAlign|;
+                position:fixed; bottom:0; background-color:|footerBgColor|;
+                border-top:|footerBgTop|; z-index:10; font-size:|footerFontSz| }
+      .footerContent { padding: 5px 1em 0 0 }''')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Browser:
