@@ -1,7 +1,7 @@
 '''Parsing of CSS properties'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -78,6 +78,10 @@ htmlColorNames = {
   'violet': '#ee82ee', 'wheat': '#f5deb3', 'white': '#ffffff',
   'whitesmoke': '#f5f5f5', 'yellow': '#ffff00', 'yellowgreen': '#9acd32'}
 
+# Valid CSS values for font-weight and font-style attributes
+fontWeigths = ('normal', 'bold', 'lighter', 'bolder')
+fontStyles = ('normal', 'italic', 'oblique')
+
 # ------------------------------------------------------------------------------
 # Default ratio for converting pixels to cm
 px2cm = 44.173513561
@@ -194,19 +198,23 @@ class CssValue:
         '''Returns p_self.value (or the given p_value) expressed in cm'''
         unit = self.unit
         if not unit: return
-        r = value if value is not None else self.value
+        if value is None:
+            r = self.value
+        else:
+            r = value
         if unit == 'px': # Convert pixels to cm
             r = r / ratio
-            if formatted:
-                r = formatNumber(r, sep='.', precision=3)
         elif unit in ('em', 'ex'):
             # For "ex", the ratio must be adapted
             if unit == 'ex': ratio = ratio * 1.6
             r = (r / ratio) * 10
-            if formatted:
-                r = formatNumber(r, sep='.', precision=3)
+        elif unit == 'pt':
+            # Use a specific ratio
+            r *= 0.0352778
         elif unit != 'cm':
             r = None
+        if formatted and (r is not None):
+            r = formatNumber(r, sep='.', precision=3)
         return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -233,7 +241,9 @@ class CssStyles:
     # Values, on combined attributes, that prevent splitting
     unsplittable = ('auto',)
     # Values that must be ignored
-    ignore = {'width': ['auto']}
+    ignore = {'*': {'auto':None, 'initial':None, 'inherit':None}}
+    # Values specified in ignore['*'] must not be ignored for these attributes
+    ignoreExcept = {'auto': {'table-layout':None}}
     # The '!important' CSS rule
     importantRule = '!important'
 
@@ -303,10 +313,13 @@ class CssStyles:
         if '-' in name: name = name.replace('-', '')
         return getattr(self, name, None)
 
-    def addClass(self, value):
+    def addClass(self, value, append=True):
         '''Add CSS class(es) p_value (as a string) to self.classes. p_value can
            hold several CSS space-separated classes.'''
-        self.classes = sadd(self.classes, value)
+        # If p_append is True, CSS classes in p_value are appended to
+        # p_self.classes. Else, they are inserted at the start of
+        # p_self.classes.
+        self.classes = sadd(self.classes, value, append=append)
 
     def deleteClass(self, value):
         '''Remove CSS class(es) in p_value from self.classes'''
@@ -343,7 +356,13 @@ class CssStyles:
         if isinstance(value, str) and value.endswith(self.importantRule):
             value = value[:-len(self.importantRule)].strip()
         # Do not add anything if the value must be ignored
-        if (name in self.ignore) and (value in self.ignore[name]): return
+        ignore = self.ignore
+        if (value in ignore['*']) or \
+           ((name in ignore) and (value in ignore[name])):
+               # Ignore it, excepted if there is a match in "ignoreExcept"
+               if (value not in self.ignoreExcept) or \
+                  (name not in self.ignoreExcept[value]):
+                   return
         attrName = name.replace('-', '')
         if not override and hasattr(self, attrName): return
         # Define the new value or combine it with the existing value when
