@@ -1,7 +1,7 @@
 '''Global elements to include in HTML pages'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -91,7 +91,7 @@ class Globals:
         if popup:
             iframe = ''
         else:
-            iframe = Iframe.view % (dright, url('close.svg'), url('arrows.svg'))
+            iframe = Iframe.view % (url('close.svg'), url('arrows.svg'))
         # Define variables, per popup
         vars = (
          # Global Javascript variables
