@@ -1,7 +1,7 @@
 '''Global elements to include in HTML pages'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -27,12 +27,13 @@ class Iframe:
     '''Represents the unique Appy iframe popup'''
 
     view = '''
+     <div id="iframeMask"></div>
      <div id="iframePopup" class="popup"
           onmousedown="dragStart(event)" onmouseup="dragStop(event)"
           onmousemove="dragIt(event)"
           onmouseover="dragPropose(event)" onmouseout="dragStop(event)">
       <!-- Icon for closing the popup -->
-      <img align="%s" src="%s" class="clickable iconXS"
+      <img src="%s" class="clickable iClose iconXS"
            onclick="closePopup('iframePopup',null,true)"/>
       <!-- Header icon -->
       <img class="iconM popupI" src="%s"/>
@@ -47,14 +48,18 @@ class Iframe:
     def goBack(class_, tool, initiator=None):
         '''Returns a HTML page allowing to close the iframe popup and refresh
            the base page.'''
-        # The initiator may force to go back to some URL
-        if initiator and initiator.backFromPopupUrl:
-            close = base64.b64encode(initiator.backFromPopupUrl.encode())
+        # A back URL may be forced by a request key or by an initiator
+        back = tool.req._back
+        if not back and initiator:
+            back = initiator.backFromPopupUrl
+        # Set the cookie containing information required for closing the popup
+        if back:
+            close = base64.b64encode(back.encode())
         else:
             close = 'yes'
         resp = tool.resp
         resp.setCookie('closePopup', close)
         # Include appy.js and call a Javascript function that will do the job
         return class_.back % (Includer.js(tool.buildUrl('appy.js')),
-                              tool.siteUrl)
+                              tool.siteUrl) 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
