@@ -1,7 +1,7 @@
 '''Manages the user interface language'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -19,8 +19,8 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from appy.px import Px
-from appy.utils import asDict
 from appy.data import Languages
+from appy.utils import noSwitchLayouts
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Language:
@@ -30,9 +30,6 @@ class Language:
     # This static attribute will store the unique appy.data.Languages instance
     languages = None
 
-    # On these layouts, switching language is not allowed
-    noSwitchLayouts = asDict(('edit', 'search', 'results'))
-
     @classmethod
     def getIsoLanguages(class_):
         '''Returns p_class_.languages or create it if it does not exist yet'''
@@ -48,7 +45,7 @@ class Language:
            switching the language is not allowed.'''
         if not config.languageSelector or config.forcedLanguage: return
         if len(config.languages) < 2: return
-        return layout not in class_.noSwitchLayouts
+        return layout not in noSwitchLayouts
 
     @classmethod
     def getName(class_, code, lowerize=False):
