@@ -1,7 +1,7 @@
 '''This module is about disposing graphical elements in the user interface'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -542,15 +542,22 @@ Layouts.w   = Layouts(Layout(b))
 # Layouts with content only (no label)
 Layouts.f   = Layouts(Layout('f', width=None))
 Layouts.wf  = Layouts(Layout('f'))
+Layouts.fv  = Layouts(edit=Layout(b), view=Layout('f'))
+Layouts.fvd = Layouts(edit=Layout(d), view=Layout('f'))
 Layouts.g   = Layouts('frvl')
 # The *d*escription is visible, even on the *v*iew layout
 Layouts.dv  = Layouts(edit=d, view='l-d-f')
+# The *d*escription is shown on the *v*iew layout but not on *e*dit (=*s*imple)
+Layouts.dvs = Layouts(edit=b, view='l-d-f')
 # "Grid" group-related layouts
 Layouts.gn  = Layouts(Layout('f;rvl=', width=None))
 Layouts.gd  = Layouts(Layout('frvl-d', width='99%'))
 Layouts.gdn = Layouts(Layout('d2-f;rvl=', width=None))
 Layouts.gh  = Layouts('fhrvl')
 Layouts.gdh = Layouts('fhrvl-d')
+# Base layouts, with space above
+Layouts.t = Layouts(Layout(b, width=None, css_class='topSpace'))
+Layouts.td = Layouts(Layout(d, css_class='topSpace'))
 
 # Fields being in groups having style "grid" deserve specific layouts. This why
 # there are 2 default layouts.
@@ -572,5 +579,8 @@ class PageLayouts:
     # Centered
     centered = Layouts(edit=Layout('b|-w|', width=None, align='center'),
                        view=Layout('e-b|-w|', align='center'))
+    # Headless (without header on "view")
+    headless = Layouts(edit=Layout('b-w', width=None), view=Layout('b-w'))
+
 Layouts.Page = PageLayouts
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
