@@ -1,7 +1,7 @@
 '''Navigation management'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -96,7 +96,7 @@ class Siblings:
     byType = {True: ('firstSibling', 'previousSibling'),
               False: ('nextSibling',  'lastSibling')}
 
-    # Icons for going to the current object's siblings    
+    # Icons for going to the current object's siblings
     pxNavigate = Px('''<div>
       <!-- Go to the source URL (search or referred object) -->
       <div if="not popup">::self.getGotoSource(url, _)</div>
@@ -120,11 +120,13 @@ class Siblings:
        <x>::self.getIcons(_ctx_, previous=False)</x>
 
        <!-- Go to the element number... -->
-       <x if="self.showGotoNumber()"
-          var2="field=self.field; sourceUrl=self.sourceObject.url;
-                total=self.total"><br/><x>:self.pxGotoNumber</x></x>
+       <div if="self.showGotoNumber()"
+            var2="field=self.field; sourceUrl=self.sourceObject.url;
+                  total=self.total">:self.pxGotoNumber</div>
       </div></div>''',
 
+     css='''.navSib { padding: |navsPadding|; font-size: 90% }''',
+
      js='''
        function gotoCustomSearch() {
          // Post a form allowing to re-trigger a custom search
