@@ -1,7 +1,7 @@
 '''Portlet management'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -46,28 +46,27 @@ class Portlet:
            cfg=config.ui;
            currentSearch=req.search;
            currentClass=req.className;
-           currentPage=handler.parts[-1];
+           currentPage=handler.parts[-1] if handler.parts else None;
            rootClasses=handler.server.model.getRootClasses()"
       id="appyPortlet" class="portlet" style=":collapse.style">
 
       <!-- The portlet logo when present -->
       <center>
-       <img if="cfg.portletLogo" src=":cfg.getImageUrl(siteUrl, 'portletLogo')"
-            class="portletLogo"/></center>
+       <img src=":url('portletLogo')" class="portletLogo"/></center>
 
       <!-- One section for every searchable root class -->
       <x for="class_ in rootClasses" if="class_.maySearch(tool, layout)"
          var2="className=class_.name">
 
-       <!-- A separator if required -->
-       <div class="portletSep"
-            if="not (loop.class_.first and not cfg.portletLogo)"></div>
+       <!-- A separator -->
+       <div class="portletSep"></div>
 
        <!-- Section title (link triggers the default search) -->
        <div class="portletContent"
             var="searches=class_.getGroupedSearches(tool, _ctx_);
                  labelPlural=_(className + '_plural');
-                 indexable=class_.isIndexable()">
+                 indexable=class_.isIndexable();
+                 viaPopup=None">
         <div class="portletTitle">
          <a if="indexable"
             var="queryParam=searches.default.name if searches.default else ''"
@@ -82,7 +81,8 @@ class Portlet:
 
         <!-- Create instances of this class -->
         <div if="guard.mayInstantiate(class_)"
-             var2="asButton=False; nav='no'">:class_.pxAdd</div>
+             var2="buttonType='portlet'; nav='no';
+                   label=None">:class_.pxAdd</div>
 
         <!-- Searches -->
         <x if="indexable and class_.maySearchAdvanced(tool)">
@@ -94,7 +94,6 @@ class Portlet:
          <div var="highlighted=(currentClass == className) and \
                                (req.search == 'customSearch')"
               class=":'portletAdv current' if highlighted else 'portletAdv'">
-          <img src=":url('search.svg')" class="iconM"/>
           <a var="text=_('search_title')"
              href=":'%s/Search/advanced?className=%s' % (toolUrl, className)"
              title=":text"><x>:text</x>...</a>
@@ -115,23 +114,24 @@ class Portlet:
 
      css='''
        .portlet { padding: 30px 50px 0 0; box-shadow: 2px 2px 5px #002039;
-                  position: sticky; position: -webkit-sticky; top: 0;
-                  overflow-y:auto; overflow-x: auto; width: 250px;
-                  min-width: 140px; font-size: 98%; z-index: 1 }
+                  position: sticky; top: 0; overflow-y:auto; overflow-x: auto;
+                  width: |portletWidth|; min-width: 140px; font-size: 98%;
+                  z-index: 4; background-color: |portletBgColor| }
+       .portlet a, .portlet a:visited { color: |portletTextColor| }
        .portletSearch a:hover { background-color:|linkColor|;
-                  color:|headerColor| }
-       .portletContent { padding-left: 18px; width: 200px }
-       .portletContent input[type=text] { background-color: white }
+                                color:|brightColor| }
+       .portletContent { padding-left: 18px; width: 100% }
        .portletTitle { padding: 5px 0; text-transform: uppercase }
-       .portletLogo { margin-bottom: 15px }
+       .portletLogo { margin: 0 0 15px 25px }
        .portletSep { border-top: 1px solid #002039; margin: 25px 0 20px 0 }
        .portletGroup { text-transform: uppercase; padding: 5px 0 0 0;
-                  margin: 0.5em 2px 0.3em }
-       .portletAdv { margin: 3px 0 10px }
-       .portletAdv img { margin: -2px 4px 0 1px; float: left}
+                       margin: 0.5em 2px 0.3em; color: |portletTextColor| }
+       .portletAdv { margin: |asMargin|; font-size: 90% }
        .portletSearch { text-align: left }
        .portletCurrent { font-weight: bold }
        .portlet form { margin-top: 5px }
-       input.buttonPortlet { border: 0; background-position: 0 }
+       input.buttonPortlet { border:0; background-position:0;
+                             background-color:transparent;
+                             color:|portletTextColor| }
      ''')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
