@@ -60,38 +60,23 @@ function toggleLoginBox(show) {
   loginBox.style.display = (show)? 'block': 'none';
 }
 
-function goto(url) { window.location = url }
+function goto(url) {
+  var win = (window.name == 'appyIFrame')? window.parent: window;
+  win.location = url;
+}
+
 function len(dict) {
   var res = 0;
   for (var key in dict) res += 1;
   return res;
 }
 
-function switchContext(select, siteUrl) {
-  goto(siteUrl + '/config/switchContext?ctx=' + select.value) }
-
-var isIe = (navigator.appName == "Microsoft Internet Explorer");
-
 function getElementsHavingName(tag, name, forceTop) {
-  if (!isIe) {
-    var r = window.document.getElementsByName(name);
-    if ((r.length == 0) && forceTop) {
-      r = window.top.document.getElementsByName(name);
-    }
-    return r;
-  }
-  else {
-    var elems = window.document.getElementsByTagName(tag),
-        r = new Array(), nameAttr;
-    if ((elems.length == 0) && forceTop) {
-      elems = window.top.document.getElementsByTagName(tag);
-    }
-    for (var i=0; i<elems.length; i++) {
-      nameAttr = elems[i].attributes['name'];
-      if (nameAttr && (nameAttr.value == name)) r.push(elems[i]);
-    }
-    return r;
+  var r = window.document.getElementsByName(name);
+  if ((r.length == 0) && forceTop) {
+    r = window.top.document.getElementsByName(name);
   }
+  return r;
 }
 
 // AJAX machinery
@@ -116,32 +101,23 @@ function evalInnerScripts(xhrObject, hookElem) {
   for (var i=0; i<scripts.length; i++) eval(scripts[i].innerHTML);
 }
 
-function injectChunk(elem, content, inner, searchTop){
-  var res = elem;
-  if (!isIe || (elem.tagName != 'TABLE')) {
-    if (inner) res.innerHTML = content;
-    else {
-      // Replace p_elem with a new node filled with p_content and return it
-      if ((elem.tagName == 'TR') && (content.substr(0,3) != '<tr')) {
-        /* Replace the whole table instead of the current row: it is probably
-           an error page. */
-        elem = elem.parentNode.parentNode;
-      }
-      var id = elem.id;
-      if (id && searchTop) id = ':' + id;
-      elem.outerHTML = content;
-      if (id) res = getNode(id); // Get the new element
-    }
-  }
+function injectChunk(tag, content, inner, searchTop){
+  // Injects the chunk of XHTML code p_content in this p_tag
+  var r = tag;
+  if (inner) r.innerHTML = content;
   else {
-    /* IE doesn't want to replace content of a table. Force it to do so via
-       a temporary DOM element. */
-    var temp = document.createElement('div');
-    temp.innerHTML = content;
-    temp.firstChild.id = elem.id;
-    elem.parentNode.replaceChild(temp.firstChild, elem);
+    // Replace p_tag with a new one filled with p_content and return it
+    if ((tag.tagName == 'TR') && content.startsWith('<!')) {
+      /* Replace the whole table instead of the current row: p_content is not a
+         chunk but a complete (probably, error) page. */
+      tag = tag.parentNode.parentNode;
+    }
+    var id = tag.id;
+    if (id && searchTop) id = ':' + id;
+    tag.outerHTML = content;
+    if (id) r = getNode(id); // Get the new element
   }
-  return res;
+  return r;
 }
 
 function getNode(id, forceTop) {
@@ -182,7 +158,7 @@ function getAjaxChunk(pos) {
       if (goto) window.top.location = goto;
       // Display the Appy message if present
       var msg = readCookie('AppyMessage');
-      if (msg) { showAppyMessage(msg); createCookie('AppyMessage', ''); }
+      if (msg) { showAppyMessage(msg,true); createCookie('AppyMessage', ''); }
     }
     rq.freed = 1;
   }
@@ -383,7 +359,7 @@ function askBunchFiltered(hook, filterKey) {
       value = filter.value;
   if (value && (filter.nodeName == 'INPUT')) {
     // Remove reserved chars and ensure it contains at least 3 chars
-    value = encodeURIComponent(value.trim()).replace(',','').replace(':','');
+    value = encodeURIComponent(value.trim().replace(',','.').replace(':',''));
     if (value.length < 3) {
       filter.style.background = wrongTextInput;
       return;
@@ -399,9 +375,9 @@ function askBunchMove(hook, start, id, move){
   var moveTo = move;
   if (typeof move == 'object'){
     // Get the new index from an input field
-    var id = move.id;
-    id = id.substr(0, id.length-4) + '_v';
-    var input = document.getElementById(id);
+    var inputId = move.id;
+    inputId = inputId.substr(0, inputId.length-4) + '_v';
+    var input = document.getElementById(inputId);
     if (isNaN(input.value)) {
       input.style.background = wrongTextInput;
       return;
@@ -423,6 +399,12 @@ function askBunchSwitchColset(hook, colset) {
   askAjax(hook, null, {'colset': colset});
 }
 
+function askBunchSwitchSearch(hook, name, searchName) {
+  var params = {}, key = name + '_view';
+  params[key] = searchName;
+  askAjax(hook, null, params);
+}
+
 function clickOn(node) {
   if (!node) return;
   // If node is a form, disable all form buttons
@@ -440,7 +422,8 @@ function clickOn(node) {
   /* For a button, show the preloader directly. For a link, show it only after
      a while, if the target page is still not there. */
   if (node.tagName == 'A') {
-    setTimeout(function(){injectChunk(node, loading('link'))}, 700);}
+    setTimeout(function(){injectChunk(node, loading('link'))}, 700);
+  }
   else {
     var name = (cn.search('buttonIcon') == -1)? 'button' : 'icon';
     injectChunk(node, loading(name));
@@ -925,7 +908,7 @@ function backFromPopup() {
   clearInterval(timer);
   if (close != 'yes') {
     // We must load a specific URL in the main page
-    window.parent.location = atob(close.slice(1,-1));
+    window.parent.location = atob(close.slice(2,-1));
   }
   else {
     // We must ajax-refresh, on the main page, the node as defined in the popup
@@ -1083,6 +1066,11 @@ function readCookie(name) {
   return null;
 }
 
+function deleteCookie(name) {
+  var expires = "expires=Thu, 01 Jan 1970 00:00:00 UTC; path=/;";
+  document.cookie = name + "=; " + expires;
+}
+
 function switchImage(img, nameA, nameB) {
   // If p_img is named nameA, its name is switched to nameB and vice versa
   var path = img.src.split('/'),
@@ -1131,7 +1119,7 @@ function toggleCookie(cookieId,display,defaultValue,expandIcon,collapseIcon) {
 }
 
 // Functions for opening and closing a popup
-function openPopup(popupId, msg, width, height, back, commentLabel) {
+function openPopup(popupId, msg, width, height, css, back, commentLabel) {
   // Put the message into the popup
   if (msg) {
     var msgHook = (popupId == 'alertPopup')? 'appyAlertText': 'appyConfirmText',
@@ -1143,22 +1131,28 @@ function openPopup(popupId, msg, width, height, back, commentLabel) {
     var labelHook = document.getElementById('appyCommentLabel');
     if (labelHook) labelHook.innerHTML = commentLabel;
   }
-  // Open the popup
+  // Get the popup
   var popup = document.getElementById(popupId),
       frame = popupId == 'iframePopup'; // Is it the "iframe" popup ?
   /* Define height and width. For non-iframe popups, do not set its height: it
      will depend on its content. */
-  if (!width)  { width =  (frame)? window.innerWidth -200: null }
-  if (!height) { height = (frame)? window.innerHeight-200: null }
+  if (!width)  { width =  (frame)? window.innerWidth -300: null }
+  if (!height) { height = (frame)? window.innerHeight-100: null }
   if (width) popup.style.width = width.toFixed() + 'px';
   if (height) popup.style.height = height.toFixed() + 'px';
   if (frame) {
-    // Set the enclosed iframe dimensions
-    var iframe = document.getElementById('appyIFrame');
+    // Set the enclosed iframe dimensions and show the mask
+    var iframe = document.getElementById('appyIFrame'),
+        imask = document.getElementById('iframeMask');
     iframe.style.width = (width - 20).toFixed() + 'px';
-    iframe.style.height = (height - 40).toFixed() + 'px';
+    iframe.style.height = (height - 20).toFixed() + 'px';
     popup['back'] = back;
+    imask.style.opacity = 0.7;
+    imask.style.zIndex = 99;
   }
+  // Apply the CSS class to the popup
+  popup.className = (css)? 'popup ' + css: 'popup';
+  // Show the popup
   popup.style.display = 'block';
 }
 
@@ -1169,7 +1163,6 @@ function closePopup(popupId, clean, tryCancel) {
       popup = container.getElementById(popupId);
   // Close the popup
   popup.style.display = 'none';
-  popup.style.width = null;
   // Clean field "clean" if specified
   if (clean) {
     var elem = popup.getElementsByTagName('form')[0].elements[clean];
@@ -1177,10 +1170,12 @@ function closePopup(popupId, clean, tryCancel) {
   }
   if (popupId == 'iframePopup') {
     // Try to click on a cancel button if found
-    var canceled = false;
+    var canceled = false, icontent=null;
     if (tryCancel) {
-      var iframe = popup.getElementsByTagName('iframe')[0],
-          cancels = iframe.contentDocument.getElementsByName('cancel'),
+      var iframe = popup.getElementsByTagName('iframe')[0];
+      // "contentDocument" may be null if the iframe points to an external site
+      icontent = iframe.contentDocument;
+      var cancels = (icontent)? icontent.getElementsByName('cancel'): [],
           cancel = (cancels.length > 0)? cancels[0]: null;
       if (cancel && (cancel.tagName == 'A')) {
         cancel.click();
@@ -1188,20 +1183,24 @@ function closePopup(popupId, clean, tryCancel) {
       }
     }
     if (!canceled) {
-      // Reinitialise the enclosing iframe
-      var iframe = container.getElementById('appyIFrame');
-      iframe.style.width = null;
-      while (iframe.firstChild) iframe.removeChild(iframe.firstChild);
-      // Leave the form silently if we are on an edit page
-      iframe.contentWindow.onbeforeunload = null;
+      /* Leave the form silently if we are on an edit page. If the iframe
+         pointed to a page from an external site, this action will be blocked by
+         the browser. */
+      if (icontent) iframe.contentWindow.onbeforeunload = null;
     }
+    if (icontent) icontent.removeChild(icontent.documentElement);
+    // Hide the mask
+    var imask = getNode(':iframeMask');
+    imask.style.opacity = 0;
+    imask.style.zIndex = 0;
   }
   return popup;
 }
 
-function showAppyMessage(message) {
+function showAppyMessage(message, forceTop) {
   // Fill the message zone with the message to display
-  var zone = getNode('appyMessageContent');
+  var prefix = (forceTop)? ':' : '',
+      zone = getNode(prefix + 'appyMessageContent');
   if (!zone) zone = getNode(':appyMessageContent');
   zone.innerHTML = message;
   // Display the message box
@@ -1231,7 +1230,7 @@ function askConfirm(actionType, action, msg, showComment, popupWidth,
   }
   else area.value = '';
   area.rows = (commentRows)? commentRows: 3;
-  openPopup("confirmActionPopup", msg, popupWidth, null, null, commentLabel);
+  openPopup('confirmActionPopup', msg, popupWidth, null,null,null,commentLabel);
 }
 
 // Function triggered when an action confirmed by the user must be performed
