@@ -1,7 +1,7 @@
 '''Utility modules and functions for Appy'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -46,10 +46,20 @@ class Version:
             currentVersion = [int(i) for i in class_.short.split('.')]
             return currentVersion >= paramVersion
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+def asDict(seq):
+    '''Returns a dict whose keys are elements from p_seq ad values are None'''
+    return {elem: None for elem in seq}
+
 # Global variables - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 sequenceTypes = (list, tuple, PersistentList)
 commercial = False
 
+# On these layouts, using a gobal selector and switching from one option to the
+# other is not allowed: it would reload the entire page. Examples are: the
+# language selector, or the authentication context selector.
+noSwitchLayouts = asDict(('edit', 'search'))
+
 # MIME-related stuff - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 od = 'application/vnd.oasis.opendocument'
 ms = 'application/vnd.openxmlformats-officedocument'
@@ -101,7 +111,14 @@ class CommercialError(Exception):
           'only available in the free, open source (GPL) version of Appy.'
     def __init__(self): Exception.__init__(self, self.MSG)
 
-class MessageException(Exception): pass
+class MessageException(Exception):
+    '''User-friendly exception'''
+    # Message exceptions are raised when, due to some problem while handling a
+    # browser request, the current traversal must be interrupted, but we don't
+    # want a "technical" error to be raised (500). We don't want technical
+    # details to be logged nor rendered to the UI, such as a Python traceback:
+    # we simply want to display nice, translated info about the problem in a 200
+    # response.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class No:
@@ -218,13 +235,6 @@ def flipDict(d, byChar=False):
             r[v] = k
     return r
 
-def asDict(seq):
-    '''Returns a dict whose keys are elements from p_seq ad values are None'''
-    r = {}
-    for elem in seq:
-        r[elem] = None
-    return r
-
 def addPair(name, value, d=None):
     '''Adds key-value pair (name, value) to dict p_d. If this dict is None, it
        returns a newly created dict.'''
@@ -259,7 +269,7 @@ def executeCommand(cmd, stdin=None):
     return proc.communicate()
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CONVERT_ERROR = 'Program "convert" (imagemagick) must be installed and ' \
+CONVERT_ERROR = 'Program "convert" (ImageMagick) must be installed and ' \
                 'in the path for performing this operation (%s).'
 
 def convert(path, options):
@@ -360,7 +370,7 @@ def iconParts(icon):
     if '/' in icon:
         base, icon = icon.split('/', 1)
     else:
-        base = 'appy'
+        base = None
     return icon, base
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
