@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -19,10 +19,11 @@
 from base64 import encodebytes
 from mimetypes import guess_type
 import xml.sax, http.client, urllib, urllib.parse, ssl
-import os, sys, re, stat, time, socket, random, hashlib
+import os, sys, re, stat, time, socket, random, hashlib, gzip
 
+from appy.utils import copyData
 from appy.model.utils import Object
-from appy.utils import copyData, string
+from appy.utils.string import Normalize
 from appy.xml.marshaller import Marshaller
 from appy.xml.unmarshaller import Unmarshaller
 
@@ -93,7 +94,7 @@ class JsonDecoder:
         res = Object()
         for name, value in d.items():
             # Ensure "name" will be a valid attribute name for a Python object
-            n = string.normalize(name, 'alphanum_')
+            n = Normalize.alphanum(name, keepUnderscore=True)
             setattr(res, n, class_.convertValue(value))
         return res
 
@@ -267,7 +268,7 @@ class HttpResponse:
         for key, value in headers._headers:
             if key != 'Set-Cookie': continue
             cookie = value.split(';')[0]
-            name, value = cookie.split('=')
+            name, value = cookie.split('=', 1)
             # Delete the cookie if it has expired
             if value.strip('"') == 'deleted':
                 # Do not set this cookie, and remove it if already present
@@ -297,8 +298,7 @@ class HttpResponse:
             return authInfo and DigestRealm(authInfo) or None
         # Determine the response type from the HTTP response, or, if not found,
         # use p_responseType that may have been given.
-        if 'content-type' in headers:
-            responseType = headers['content-type']
+        responseType = headers.get('Content-Type')
         if not responseType: return
         # Apply some transform on the response content depending on its type
         contentType = self.extractContentType(responseType)
@@ -363,7 +363,9 @@ class Resource:
         else: raise self.Error('Wrong URL: %s' % str(url))
         # If some headers must be sent with any request sent through this
         # resource, you can store them in the following dict.
-        self.headers = {'Host': self.getHeaderHost(), 'User-Agent': 'Appy'}
+        self.headers = {'Host': self.getHeaderHost(), 'User-Agent': 'Appy',
+                        'Connection': 'close', 'Accept': '*/*',
+                        'Accept-Encoding': 'gzip, identity'}
         # Cookies defined hereafter will be included in self.headers at every
         # request.
         self.cookies = {}
@@ -397,8 +399,27 @@ class Resource:
             authorization = '%s %s' % (self.authMethod,
                                        encodebytes(creds).strip().decode())
             headers['Authorization'] = authorization
-        headers['Connection'] = 'close'
-        headers['Accept'] = '*/*'
+
+    def readResponse(self, response):
+        '''Reads the response content. Unzip the result when appropriate'''
+        headers = response.headers
+        if headers.get('Content-Encoding') == 'gzip':
+            # Unzip the content
+            f = gzip.GzipFile(fileobj=response)
+        else:
+            f = response
+        # Read response bytes
+        r = f.read()
+        # Decode the response when relevant
+        contentType = headers.get('Content-Type')
+        if contentType and contentType.startswith('image/'):
+            pass # No need to decode it
+        else:
+            try:
+                r = r.decode()
+            except UnicodeDecodeError:
+                pass
+        return r
 
     def send(self, method, uri, body=None, headers=None, bodyType=None,
              responseType=None):
@@ -437,7 +458,7 @@ class Resource:
         except socket.timeout as te:
             raise self.Error(self.TIMEOUT_ERROR % conn.timeout)
         if self.measure: endTime = time.time()
-        body = response.read()
+        body = self.readResponse(response)
         conn.close()
         # Return a smart object containing the various parts of the response
         duration = None
@@ -478,7 +499,7 @@ class Resource:
     def get(self, uri=None, headers=None, params=None, followRedirect=True,
             responseType=None):
         '''Perform a HTTP GET on the server. Parameters can be given as a dict
-           in p_params. p_responseType will be used if no "content-type" key is
+           in p_params. p_responseType will be used if no "Content-Type" key is
            found on the HTTP response.'''
         if not uri: uri = self.uri
         # Encode and append params if given
