@@ -1,7 +1,7 @@
 '''Date-related classes and functions'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,7 +18,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import sys
+import sys, time
 try:
     from DateTime import DateTime
     from DateTime.interfaces import DateError
@@ -181,4 +181,21 @@ def periodsIntersect(start1, end1, start2, end2):
     # Intervals intersect if they are not disjoint
     if (start1 > end2) or (start2 > end1): return
     return True
-# ------------------------------------------------------------------------------
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class Year:
+    '''Represents a year'''
+
+    @classmethod
+    def isLeap(class_, year=None):
+        '''Is this p_year (or the current year by default) a leap year ?'''
+        # In french, leap = "bissextile"
+        year = time.localtime()[0] if year is None else year
+        return ((year%400) == 0) or (((year%4) == 0) and ((year%100) != 0))
+
+    @classmethod
+    def getDays(class_, year=None):
+        '''Returns the number of days within this p_year (or the current year by
+           default).'''
+        return 366 if Year.isLeap(year) else 365
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
