@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -263,8 +263,13 @@ class Merger:
 class HtmlDiff:
     '''This class allows to compute differences between two versions of some
        HTML chunk.'''
-    insertStyle = 'color: blue; cursor: help'
-    deleteStyle = 'color: red; text-decoration: line-through; cursor: help'
+
+    # Default CSS styles to apply on a insertion or a deletion
+    insertStyle = 'color:blue; cursor:help'
+    deleteStyle = 'color:red; text-decoration:line-through; cursor:help'
+
+    # Template tag that wraps a diff, be it a deletion or an insertion
+    templateDiv = '<div style="%s" title="%s">%s</div>'
 
     def __init__(self, old, new,
                  insertMsg=u'Inserted text', deleteMsg=u'Deleted text',
@@ -318,21 +323,19 @@ class HtmlDiff:
 
            If p_msg is given, it will be used instead of the default
            p_type-related message stored on p_self.'''
-        # Will the surrouding tag be a div or a span?
-        if sep == '\n': tag = 'div'
-        else: tag = 'span'
-        # What message will it show in its 'title' attribute?
-        if not msg:
-            exec('msg = self.%sMsg' % type)
+        # Will the surrounding tag be a div or a span ?
+        tag = 'div' if sep == '\n' else 'span'
+        # What message will it show in its 'title' attribute ?
+        msg = msg or eval('self.%sMsg' % type)
         # What CSS class (or, if none, tag-specific style) will be used ?
-        exec('cssClass = self.%sCss' % type)
+        cssClass = eval('self.%sCss' % type)
         if cssClass:
             style = 'class="%s"' % cssClass
         else:
-            exec('style = self.%sStyle' % type)
+            style = eval('self.%sStyle' % type)
             style = 'style="%s"' % style
         # The 'name' attribute of the tag indicates the type of the update.
-        exec('tagName = self.%sName' % type)
+        tagName = eval('self.%sName' % type)
         # The idea is: if there are several lines, every line must be surrounded
         # by a tag. This way, we know that a surrounding tag can't span several
         # lines, which is a prerequisite for managing cumulative diffs.
@@ -344,11 +347,11 @@ class HtmlDiff:
             return '%s<%s name="%s" %s title="%s">%s</%s>%s' % \
                    (sep, tag, tagName, style, msg, seq, tag, sep)
         else:
-            res = ''
+            r = ''
             for line in seq:
-                res += '%s<%s name="%s" %s title="%s">%s</%s>%s' % \
-                       (sep, tag, tagName, style, msg, line, tag, sep)
-            return res
+                r += '%s<%s name="%s" %s title="%s">%s</%s>%s' % \
+                     (sep, tag, tagName, style, msg, line, tag, sep)
+            return r
 
     def applyDiff(self, line, diff):
         '''p_diff is a regex containing an insert or delete that was found
