@@ -1,7 +1,7 @@
 '''Functions for sending emails'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -26,20 +26,19 @@ from email.mime.base import MIMEBase
 from email.mime.text import MIMEText
 from email.mime.multipart import MIMEMultipart
 
-from appy.utils import sequenceTypes
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NO_CONFIG = 'must send mail but no SMTP server configured.'
-NO_SERVER = 'no mailhost defined'
-DISABLED = 'mail disabled%s: should send mail from %s to %d recipient(s): %s.'
-MSG_REPLY_TO = 'reply to: %s'
-MSG_SUBJECT = 'subject: %s'
-MSG_BODY = 'body: %s'
-MSG_ATTACHMENTS = '%d attachment(s).'
-MSG_SENDING = 'sending mail from %s to %s (subject: %s).'
-MAIL_RECIPIENTS_KO = 'could not send mail to some recipients. %s'
-MAIL_SENT = 'mail sent in %.2f secs.'
-MAIL_NOT_SENT = '%s: mail sending failed (%s)'
+NO_CONFIG  = 'Must send mail but no SMTP server configured.'
+NO_SERVER  = 'no mailhost defined'
+DISABLED   = 'Mail disabled%s: should send mail from %s to %d recipient(s): %s.'
+REPLY_TO   = 'reply to: %s'
+MSG_SUBJ   = 'Subject: %s'
+MSG_BODY   = 'Body: %s'
+MSG_ATTS   = '%d attachment(s).'
+MSG_SEND   = 'Sending mail from %s to %s (subject: %s).'
+MAIL_R_KO  = 'Could not send mail to some recipients. %s'
+MAIL_SENT  = "Mail sent in %.2f''."
+MAIL_NSENT = '%s: mail sending failed (%s).'
+CONNECT_OK = "Connected to %s in %.2f''."
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -86,11 +85,22 @@ class Config:
         if self.login: r += ' (login as %s)' % self.login
         return r
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+def logMailSent(r, start, log):
+    '''A mail a just been sent. Log its success or failure (depending on p_r)'''
+    if not log: return
+    if r:
+        log(MAIL_R_KO % str(r), type='warning')
+    else:
+        log(MAIL_SENT % (time.time() - start))
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def sendMail(config, to, subject, body, attachments=None, log=None,
-             replyTo=None):
+             replyTo=None, split=False):
     '''Sends a mail, via the SMTP server defined in the p_config'''
-    # p_config is an instance of class appy.utils.mail.Config above.
+
+    # p_config is an instance of class appy.utils.mail.Config hereabove.
+
     # This function sends a mail to p_to (a single email recipient or a list of
     # recipients). Every (string) recipient can be an email address or a string
     # of the form "[name] <[email]>".
@@ -105,8 +115,12 @@ def sendMail(config, to, subject, body, attachments=None, log=None,
     # - the second must be named "type" and will receive string
     #   "info", "warning" or "error".
 
-    # A p_replyTo mail address or recipient can be specified.
-    start = time.time()
+    # A p_replyTo mail address or recipient can be specified
+
+    # If p_split is False, a single mail will be send to all repicients
+    # specified at once in field "To". Else, a distinct mail will be sent to
+    # every recipient defined in p_to.
+
     to = [to] if isinstance(to, str) else to
     if not config:
         if log: log(NO_CONFIG)
@@ -120,11 +134,11 @@ def sendMail(config, to, subject, body, attachments=None, log=None,
             toLog = DISABLED % (msg, fromAddress, len(to), ', '.join(to))
             if replyTo: toLog += ' (%s).' % (REPLY_TO % replyTo)
             log(toLog)
-            log(MSG_SUBJECT % subject)
+            log(MSG_SUBJ % subject)
             log(MSG_BODY % body)
-        if attachments and log: log(MSG_ATTACHMENTS % len(attachments))
+        if attachments and log: log(MSG_ATTS % len(attachments))
         return
-    if log: log(MSG_SENDING % (fromAddress, str(to), subject))
+    if log: log(MSG_SEND % (fromAddress, ', '.join(to), subject))
     # Create the base MIME message
     body = MIMEText(body, 'plain')
     if attachments:
@@ -137,11 +151,6 @@ def sendMail(config, to, subject, body, attachments=None, log=None,
     msg['From'] = fromAddress
     msg['Date'] = formatdate(localtime=True)
     if replyTo: msg.add_header('reply-to', replyTo)
-    if len(to) == 1:
-        msg['To'] = to[0]
-    else:
-        msg['To'] = fromAddress
-        msg['Bcc'] = ', '.join(to)
     # Add attachments
     if attachments:
         for attachment in attachments:
@@ -160,25 +169,41 @@ def sendMail(config, to, subject, body, attachments=None, log=None,
             part.add_header('Content-Disposition',
                             'attachment; filename="%s"' % fileName)
             msg.attach(part)
-    # Send the email
+    # Send the mail(s)
     try:
+        start = time.time()
+        # Connect to the SMTP server
         smtpServer = smtplib.SMTP(config.server, port=config.port)
         if config.secure:
             smtpServer.ehlo()
             smtpServer.starttls()
         if config.login:
             smtpServer.login(config.login, config.password)
-        r = smtpServer.sendmail(fromAddress, to, msg.as_string())
+        # Log the time spent while connecting, if we perform a single connection
+        # for sending several mails.
+        if split and log:
+            log(CONNECT_OK % (time.time()-start))
+        # Precompute the mail body if a single mail must be sent
+        if not split:
+            msg['To'] = ', '.join(to)
+            body = msg.as_string()
+        # Send the mail(s)
+        if split:
+            for recipient in to:
+                start = time.time()
+                msg['To'] = recipient
+                body = msg.as_string()
+                r = smtpServer.sendmail(fromAddress, [recipient], body)
+                logMailSent(r, start, log)
+        else:
+            r = smtpServer.sendmail(fromAddress, to, body)
+            logMailSent(r, start, log)
+        # Disconnect from the server
         smtpServer.quit()
-        if log:
-            if r:
-                log(MAIL_RECIPIENTS_KO % str(r), type='warning')
-            else:
-                log(MAIL_SENT % (time.time() - start))
     except smtplib.SMTPException as e:
         if log:
-            log(MAIL_NOT_SENT % (config, str(e)), type='error')
+            log(MAIL_NSENT % (config, str(e)), type='error')
     except socket.error as se:
         if log:
-            log(MAIL_NOT_SENT % (config, str(se)), type='error')
+            log(MAIL_NSENT % (config, str(se)), type='error')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
