@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -249,8 +249,7 @@ class Marshaller:
                 for field in o.getFields('xml'):
                     # Dump only needed fields
                     if not self.mustDump(field.name, className): continue
-                    val = field.getValue(o, single=False) \
-                          if field.type == 'Ref' else field.getValue(o)
+                    val = field.getValue(o, single=False)
                     v = field.getXmlValue(o, val)
                     self.dumpField(r, field.name, v)
                 # Dump the object history
