@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2020 Gaetan Delannay
+# Copyright (C) 2007-2021 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -251,10 +251,10 @@ class Unmarshaller(Parser):
             if e.currentBasicType == 'base64':
                 content = content.strip()
             e.currentContent += content
-        elif e.currentElem.name in self.contentAttributes:
+        elif e.currentTag.name in self.contentAttributes:
             # Store p_content in attribute named according to
             # p_self.contentAttributes on the current object.
-            name = self.contentAttributes[e.currentElem.name]
+            name = self.contentAttributes[e.currentTag.name]
             current = e.containerStack[-1]
             if hasattr(current, name):
                 setattr(current, name, getattr(current, name) + content)
